/**********************************************************************
 *
 * CALC7BIG: Calculate gradients and probabilities given Y, DATA, and 
 * 			parameters for all possible combo choices.
 *
 * Matthew Gentzkow
 * July 14, 2003
 * Online Editions
 * 
 * MATLAB prototype:
 * P = calc7big(b,excl,Y1,Y7,X,RD,weights,CM,Cincl,Bcons,Mcons,Mincl,Bex,m)
 * [P Z] = calc7big(b,excl,Y1,Y7,X,RD,weights,CM,Cincl,Bcons,Mcons,Mincl,Bex,m)
 *
 * compiler command: 
 *	mex calc7big.c oedcalc.c oedcalcbig.c
 *
 * note: a*b*c in dimension means vectors stacked as a groups of b groups of c
 * note: This function is not currently written to accept missing values (i.e. assumes all variables are
 *			observed on both 1-day and 7-day basis 
 * note: # before range means only integer values are valid
 *
 *
 * INPUTS:
 *	b			PP			(-inf,inf)		param vec: c groups of (k+d) then f(pi)
 *	excl		PMAX		#(0,1)			vec of exclusion restrictions (=1 if excl)
 *	Y1			NN			#(0,CC-1)		one-day choices
 *	Y7			NN			#(0,CC-1)		seven-day choices
 *	X			KK x NN		(-inf,inf)		observable rhs vars
 *	RD			NN*DD x m	(-inf,inf)		random draws of unobservable rhs vars
 *	weights		NN x m		(-inf,inf)		weights of each obs (eg for imp sampling)
 *	CM			GG x CC		#(0,1)			i x j=1 if good i included in choice j
 *	Cincl		CCW x 2		#(0,CC-1)		row i is 1 and 7-day choices incl in combo choice i
 *	Bcons		CC x CC		#(0,1)			i x j=1 if choice j potentially consist with choice i
 *	Bex			CC x CC		#(0,CC-1)		i x j=k where 1-day ch i takes overall ch from j to k
 *	m			scalar		#(0,inf)		number of simulation draws
 *
 * OUTPUTS:
 *	P			CCW x NN	(0,1)			i*j is probability of i choosing j
 *	Z			NN*CCW x PP	(-inf,inf)		i*j*k is derivative of prob of j's choosing (combo) k
 *											w.r.t parameter i
 *
 ***********************************************************************/

#include <math.h>
#include "mex.h"
#include "oedcalc.h"
#include "oedcalcbig.h"

#define NINPUTS 12
#define NOUTMAX 2

/* ****************************************************** */
/* mexFunction: MATLAB gateway routine                    */
/*														  */
/* remember: m is rows, n is columns					  */
/*													      */
/* ****************************************************** */


void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[] )
{
  int n[NINPUTS], m[NINPUTS], ngood[NINPUTS], mgood[NINPUTS], integer[NINPUTS],
	  nout[NOUTMAX], mout[NOUTMAX];
  double min[NINPUTS], max[NINPUTS];
  char errstring[200];
  int ii;
  int mm;
  double *IPTRS[NINPUTS], *OPTRS[NOUTMAX];
  
  /* get m here to assign dimensions */
  mm = *mxGetPr(prhs[13]);

  /* set ngood and mgood arrays with expected input dimensions */
  mgood[0] = PP;			ngood[0] = 1;
  mgood[1] = PMAX;			ngood[1] = 1;
  mgood[2] = NN;			ngood[2] = 1;
  mgood[3] = NN;			ngood[3] = 1;
  mgood[4] = KK;			ngood[4] = NN;
  mgood[5] = NN*DD;			ngood[5] = mm;
  mgood[6] = NN;			ngood[6] = mm;
  mgood[7] = GG;			ngood[7] = CC;
  mgood[8] = CCW;			ngood[8] = 2;
  mgood[9] = CC;			ngood[9] = CC;
  mgood[10] = CC;			ngood[10] = CC;
  mgood[11] = 1;			ngood[11] = 1;

  /* set min and max arrays with allowable range of inputs */
  /* (enter 9999 or -9999 to represent +/- infinity)  */
  /* integer[ii]=1 indicates input must be an integer */
  min[0] = -9999;		max[0] = 9999;		integer[0] = 0;
  min[1] = 0;			max[1] = 1;			integer[1] = 1;
  min[2] = 0;			max[2] = CC-1;		integer[2] = 1;
  min[3] = 0;			max[3] = CC-1;		integer[3] = 1;
  min[4] = -9999;		max[4] = 9999;		integer[4] = 0;
  min[5] = -9999;		max[5] = 9999;		integer[5] = 0;
  min[6] = -9999;		max[6] = 9999;		integer[6] = 0;
  min[7] = 0;			max[7] = 1;			integer[7] = 1;
  min[8] = 0;			max[8] = CC-1;		integer[8] = 1;
  min[9] = 0;			max[9] = 1;			integer[9] = 1;
  min[10] = 0;			max[10] = CC-1;		integer[10] = 1;
  min[11] = 0;			max[11] = 9999;		integer[11] = 1;

  /*set nout and mout arrays with output dimensions */
  mout[0] = CCW;			nout[0] = NN;
  mout[1] = NN*CCW;			nout[1] = PP;

  /* check number of arguments */
  if(nrhs != NINPUTS)
    mexErrMsgTxt("NINPUTS inputs required.");
  if(nlhs > NOUTMAX)
    mexErrMsgTxt("At most NOUTMAX outputs allowed.");

  /* get pointers and dimensions of each input element */
  for (ii=0; ii<NINPUTS; ii++) {
	  IPTRS[ii] = mxGetPr(prhs[ii]);
	  n[ii] = mxGetN(prhs[ii]);
	  m[ii] = mxGetM(prhs[ii]);
  }

  /* check dimensions of inputs */
  for (ii=0; ii<NINPUTS; ii++) {
	  if(n[ii] != ngood[ii]) {
		  sprintf(errstring,"Input number %d: column dimension is %d but should be %d", ii+1, n[ii], ngood[ii]);
		  mexErrMsgTxt(errstring);
	  }
	  if(m[ii] != mgood[ii]) {
		  sprintf(errstring,"Input number %d: row dimension is %d but should be %d", ii+1, m[ii], mgood[ii]);
		  mexErrMsgTxt(errstring);
	  }
  }

  /* check range of inputs */
  for (ii=0; ii<NINPUTS; ii++) {
	  if( outofrange(IPTRS[ii],n[ii]*m[ii],min[ii],max[ii]) ) {
		  sprintf(errstring,"Input number %d: value out of range.", ii+1);
		  mexErrMsgTxt(errstring);
	  }
	  if(integer[ii]==1){
		  if( noninteger(IPTRS[ii],n[ii]*m[ii]) ) {
			sprintf(errstring,"Input number %d: values must be integers.", ii+1);
			mexErrMsgTxt(errstring);
		  }
	  }
  }

  /* initialize output elements */
  for (ii=0; ii< ((nlhs>0)? nlhs:1) ; ii++) {
	  plhs[ii] = mxCreateDoubleMatrix(mout[ii],nout[ii],mxREAL);
	  OPTRS[ii] = mxGetPr(plhs[ii]);
  }

  /* call C function */
  if (nlhs<=1) {
	calcp7big(OPTRS[0], IPTRS[0],IPTRS[1],IPTRS[2],IPTRS[3],IPTRS[4],IPTRS[5],
			IPTRS[6],IPTRS[7],IPTRS[8],IPTRS[9],IPTRS[10],(int)*IPTRS[11]);
  } else if (nlhs==2) {
	calcpdp7big(OPTRS[0],OPTRS[1], IPTRS[0],IPTRS[1],IPTRS[2],IPTRS[3],IPTRS[4],IPTRS[5],
			IPTRS[6],IPTRS[7],IPTRS[8],IPTRS[9],IPTRS[10],(int)*IPTRS[11]);
  }


}




/**********************************************************************
 *
 * OUTOFRANGE: Returns 1 if there is a value in vector greater
 *				than max or less than min; returns 0 otherwise.
 *				min/max values of +/-9999 are interpreted as +/- infinity.
 *				Returns 0 otherwise.
 *
 ***********************************************************************/

int outofrange(double *vector, int length, double min, double max) 
{

	int ii;

	for (ii=0; ii<length; ii++) {
		if ( (max!=9999 & vector[ii]>max) | (min!=-9999 & vector[ii]<min) ) {
			return(1);
		}
	}

	return(0);
}


/**********************************************************************
 *
 * NONINTEGER: Returns 1 if there is a value in vector whose
 *				decimal portion is not zero. Returns 0 otherwise
 *
 ***********************************************************************/

int noninteger(double *vector, int length) 
{

	int ii;

	for (ii=0; ii<length; ii++) {
		if ( fmod(vector[ii],1) ) {
			return(1);
		}
	}

	return(0);
}



