/**********************************************************************************
 *
 * PROBABILITIES1: Generate one-day probabilities for each consumer at each
 *					simulation draw for each choice.
 *
 * Matthew Gentzkow
 * July 15, 2003
 * Online Editions
 * 
 * MATLAB prototype:
 * P = probabilities1(X,RD,b,excl,CM,m)
 *
 * compiler command: 
 *	mex probabilities1.c oedcalc.c oedtools.c
 *
 * note: a*b*c in dimension means vectors stacked as a groups of b groups of c
 * note: # before range means only integer values are valid
 *
 * INPUTS:
 *	X			KK x NN		(-inf,inf)		observable rhs vars
 *	RD			NN*DD x m	(-inf,inf)		random draws of unobservable rhs vars
 *	b			PP			(-inf,inf)		param vec: c groups of (k+d) then f(pi)
 *	excl		PMAX		#(0,1)			vec of exclusion restrictions (=1 if excl)
 *	CM			GG x CC		#(0,1)			i x j=1 if good i included in choice j
 *	m			scalar		#(0,inf)		number of simulation draws
 *  ninput      scalar      #(0,inf)        number of observations
 *
 * OUTPUTS:
 *	P			m*CC x NN	(0,1)			Simulated probabilities of each (one-day) choice
 *											for each draw
 *
 ***********************************************************************************/

#include <math.h>
#include "choose_max.h"
#include "mex.h"
#include "oedcalc.h"
#include "oedtools.h"

#define NINPUTS 7
#define NOUTMAX 1

/* ****************************************************** */
/* mexFunction: MATLAB gateway routine                    */
/*														  */
/* remember: m is rows, n is columns					  */
/*													      */
/* ****************************************************** */


void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[] )
{
  int n[NINPUTS], m[NINPUTS], ngood[NINPUTS], mgood[NINPUTS], integer[NINPUTS],
	  nout[NOUTMAX], mout[NOUTMAX];
  double min[NINPUTS], max[NINPUTS];
  char errstring[200];
  int ii;
  int mm,nn,rdval;
  double *IPTRS[NINPUTS], *OPTRS[NOUTMAX];
  
  /* check number of arguments */
  if(nrhs != NINPUTS)
    mexErrMsgTxt("NINPUTS inputs required.");
  if(nlhs > NOUTMAX)
    mexErrMsgTxt("At most NOUTMAX outputs allowed.");
  
  /* get m here to assign dimensions */
  mm = *mxGetPr(prhs[5]);
  nn = *mxGetPr(prhs[6]);
  
  /* handle the case of DD==0 */
  rdval = choose_max(nn*DD,1);
  
  /* set ngood and mgood arrays with expected input dimensions */
  mgood[0] = KK;			ngood[0] = nn;
  mgood[1] = rdval;			ngood[1] = mm;
  mgood[2] = PP;			ngood[2] = 1;
  mgood[3] = PMAX;			ngood[3] = 1;
  mgood[4] = GG;			ngood[4] = CC;
  mgood[5] = 1;				ngood[5] = 1;
  mgood[6] = 1;             ngood[6] = 1;

  /* set min and max arrays with allowable range of inputs */
  /* (enter 9999 or -9999 to represent +/- infinity)  */
  /* integer[ii]=1 indicates input must be an integer */
  min[0] = -9999;		max[0] = 9999;		integer[0] = 0;
  min[1] = -9999;		max[1] = 9999;		integer[1] = 0;
  min[2] = -9999;		max[2] = 9999;		integer[2] = 0;
  min[3] = 0;			max[3] = 1;			integer[3] = 1;
  min[4] = 0;			max[4] = 1;			integer[4] = 1;
  min[5] = 0;			max[5] = 9999;		integer[5] = 1;
  min[6] = 0;           max[6] = 99999;     integer[6] = 1;

  /*set nout and mout arrays with output dimensions */
  mout[0] = mm*CC;			nout[0] = nn;

  /* get pointers and dimensions of each input element */
  for (ii=0; ii<NINPUTS; ii++) {
	  IPTRS[ii] = mxGetPr(prhs[ii]);
	  n[ii] = mxGetN(prhs[ii]);
	  m[ii] = mxGetM(prhs[ii]);
  }

  /* check dimensions of inputs */
  for (ii=0; ii<NINPUTS; ii++) {
	  if(n[ii] != ngood[ii]) {
		  sprintf(errstring,"Input number %d: column dimension is %d but should be %d", ii+1, n[ii], ngood[ii]);
		  mexErrMsgTxt(errstring);
	  }
	  if(m[ii] != mgood[ii]) {
		  sprintf(errstring,"Input number %d: row dimension is %d but should be %d", ii+1, m[ii], mgood[ii]);
		  mexErrMsgTxt(errstring);
	  }
  }

  /* check range of inputs */
  for (ii=0; ii<NINPUTS; ii++) {
	  if( outofrange(IPTRS[ii],n[ii]*m[ii],min[ii],max[ii]) ) {
		  sprintf(errstring,"Input number %d: value out of range.", ii+1);
		  mexErrMsgTxt(errstring);
	  }
	  if(integer[ii]==1){
		  if( noninteger(IPTRS[ii],n[ii]*m[ii]) ) {
			sprintf(errstring,"Input number %d: values must be integers.", ii+1);
			mexErrMsgTxt(errstring);
		  }
	  }
  }

  /* initialize output elements */
  for (ii=0; ii< ((nlhs>0)? nlhs:1) ; ii++) {
	  plhs[ii] = mxCreateDoubleMatrix(mout[ii],nout[ii],mxREAL);
	  OPTRS[ii] = mxGetPr(plhs[ii]);
  }

  /* call C function */
	probabilities1(OPTRS[0], IPTRS[0],IPTRS[1],IPTRS[2],IPTRS[3],IPTRS[4],(int)*IPTRS[5],(int)*IPTRS[6]);

}



/**********************************************************************
 *
 * OUTOFRANGE: Returns 1 if there is a value in vector greater
 *				than max or less than min; returns 0 otherwise.
 *				min/max values of +/-9999 are interpreted as +/- infinity.
 *				Returns 0 otherwise.
 *
 ***********************************************************************/

int outofrange(double *vector, int length, double min, double max) 
{

	int ii;

	for (ii=0; ii<length; ii++) {
		if ( (max!=9999 & vector[ii]>max) | (min!=-9999 & vector[ii]<min) ) {
			return(1);
		}
	}

	return(0);
}


/**********************************************************************
 *
 * NONINTEGER: Returns 1 if there is a value in vector whose
 *				decimal portion is not zero. Returns 0 otherwise
 *
 ***********************************************************************/

int noninteger(double *vector, int length) 
{

	int ii;

	for (ii=0; ii<length; ii++) {
		if ( fmod(vector[ii],1) ) {
			return(1);
		}
	}

	return(0);
}



