%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CALCGAMMAS.M: Calculate distribution of cap-gamma parameters
%
% Matthew Gentzkow
% November 29, 2004
% Online Editions
%
% note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
%
% INPUTS:
%   nchbundle	    CC					number of consumers choosing each bundle
%   bstar           PP                  true parameter vector
%   bstarmat        PP x mstderr        each column is one random draw on the parameter vector
%	excl		    PP					vector of exclusion restrictions
%	Xall		    NNtot x KK			    observable rhs vars
%   RD              NNtot*DD x msee        random draws
%	CM			    CC x GG				i*j=1 if good j included in choice i
% 	surveyweights	NNtot			        sampling weights of each observation
%   mstderr         scalar              number of monte carlo draws
%	
% OUTPUTS:
%   gammas      1 x CC-1            mean of interaction terms
%   gammasse    1 x CC-1            std. error of mean of interaction terms (across monte carlo draws)
%   gammasmin   1 x CC-1            min of interaction across consumers
%   gammasmax   1 x CC-1            max of interaction across consumers
%   gammasvar   1 x CC-1            var of interaction across consumers
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [gammas,gammasse,gammasmin,gammasmax,gammasvar] = ...
            calcgammas(nchbundle,bstar,bstarmat,excl,Xall,RD,CM,surveyweights,mstderr)
global GG CC KK DD PP ZZ msee NNtot

gammasmat = zeros(mstderr,CC-1);
bstarmat = [bstar bstarmat];

for mm = 1:mstderr+1   
    b = bstarmat(:,mm);
    U = utilities(b,excl,Xall',RD,CM',msee,NNtot);
    U = reshape(U,CC-1,msee*NNtot)';                % reshape so it's (msee*NNtot x CC-1)
    U = [zeros(msee*NNtot,1) U];
    weights = repmat(surveyweights,msee,1);

    % gammas
    Usm = U(1:NNtot,:);
    gam = zeros(NNtot,GG);
    for ii = 2:CC
        if sum(CM(ii,:))>1
            Ucombo = Usm(:,ii);
            indicator = repmat([0 CM(ii,:) zeros(1,CC-GG-1)],NNtot,1);
            Usum = sum(Usm.*indicator,2);
            gam = [gam Ucombo-Usum];
        end
    end

    if mm==1
        % only for true value of bstar
        gammas = mean(gam);
        gammasmin = min(gam);
        gammasmax = max(gam);
        gammasvar = var(gam);
    else
        % for simulated values of bstar to calculate SE
        gammasmat(mm,:) = mean(gam);
    end
end

if mstderr>1
    gammasse=sqrt(var(gammasmat));
else
    gammasse=zeros(1,CC);
end

	