%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CTESTSIM.M: Generates simulations of the DGP at a given parameter vector,
% 				X matrix, and matrix of random draws for a single consumer;
%				used in calculating bias test statistic.
%
% Matthew Gentzkow
% September 16, 2002
% Online Editions
%
% note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
%
% INPUTS:
%	X			NN*KK					observable rhs vars
%	bstar		PP					parameter vector CC groups of (KK+DD) followed by f(pi)
%	excl		PMAX				vector of exclusion restrictions (=1 if excluded)
%	CM			CC*GG					i*j=1 if good j included in choice i
%	indiv		integer				index of consumers whose choices are to be simulated
%
%	
% OUTPUTS:
%	Y1			NN					one-day choices
%	Y7			NN					seven-day choices
%	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [Y1,Y7] = ctestsim(X,bstar,excl,CM,indiv)

global NN GG CC KK DD PP PMAX m mtest

bbig = reverseselect(bstar,1-excl);

randn('state',sum(100*clock));
CMm = reshape(CM,GG,CC)';
bmat = reshape(bbig(1:PMAX-1),KK+DD,CC-1);
beta = bmat(:,1:GG);
gamma = bmat(:,GG+1:CC-1);
pi = exp(bbig(PP))/(1+exp(bbig(PMAX)));
Xm = reshape(X,KK,NN)';
DATA = [repmat(Xm(indiv,:),mtest,1) randn(mtest,DD)];			% use independent simulations here

% generate matrix of probabilities
Us = DATA*beta;				% Utility of single goods
U = Us;
if CC>GG+1
    for i=GG+2:CC
        U = [U sum(repmat(CMm(i,:),mtest,1).*Us,2)+repmat(gamma(i-GG-1),mtest,1)];
    end
end
U = [zeros(mtest,1) U];
Ue = exp(U);
Ue_sum = repmat(sum(Ue,2),1,CC);
P = Ue./Ue_sum;

Y1 = simulate(P,1);
Y7 = zeros(mtest,1);
Ymat = CMm(Y1,:);
for j = 1:mtest
   	for kk = 1:4
       	if rand(1)<=pi
            Ynew = simulate(P(j,:),1);
            Ymat(j,:) = min(Ymat(j,:)+CMm(Ynew,:),1);
		end
   	end
end
	
for j = 1:mtest
	for kk = 1:CC
		if(Ymat(j,:)==CMm(kk,:)), Y7(j)=kk; end
   	end
end

% subtract 1 so range of Y1 and Y7 is 0 to (CC-1)
Y1 = Y1-1;
Y7 = Y7-1;