%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LOGITSIM7.M: Generate data from OED model using a specified covariance
%				matrix of rhs variables then siumlates choices given that
%				data, randomly generated unobservables, and a given parameter vector.
%
% Matthew Gentzkow
% June 17, 2002
% Online Editions
%
% note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
%
% INPUTS:
%	
% OUTPUTS:
%	Y1			NN					one-day choices
%	Y7			NN					seven-day choices
%	X			NN*KK					observable rhs vars
%	CM			CC*GG					i*j=1 if good j included in choice i
%	Cincl			2*CCW				1:2*i gives the 1 and 7-day choices incl in combo choice i
%	Bcons		CC*CC					i*j=1 if choice i potentially consistent with choice j
%	Bex			CC*CC					i*j=KK where 1-day choice j takes overall choice from i to KK
%	Mcons		CC					first vector for missing 1-day data
%	Mincl		CC					second vector for missing 1-day data
%	header		KK+DD					array of variable names
%   CCW          scalar              number of combo choices
%	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [Y1,Y7,X,CM,Cincl,Bcons,Bex,Mcons,Mincl,header,excl,R,r] = logitsim7()

global NN GG CC KK DD PP
setparameters;

S = eye(KK-1);	 % covariance of X variables

% true model parameters: true coefficients (vars in rows (KK+DD), goods in cols (GG))
%                        first row is constant term
beta = [   -1 -1;...
            -1 1;...
			1 -2;...
			2 -1;...
			1 0];
gamma = [1;...
	     0;...
		 0;...
		 0;...
		 0];
pi = 0.3;

excl = zeros(16,1);
excl(12)=1;
excl(13)=1;
excl(14)=1;
excl(15)=1;

R = zeros(12,2);
R(5,1)=1;
R(12,2)=1;
R = R';
r = [1; 0];

header = {'v1';'v2';'v3'};    % header should be KK+DD x 1

b = [reshape(beta,(KK+DD)*GG,1); gamma; pi];

% choice matrix elements for combination choices
combos = [1 1];

% check consistency of parameters
test = 0;
if      PMAX ~= (KK+DD)*(CC-1)+1|...
        size(b)~=PMAX | size(beta,1)~=KK+DD | size(beta,2)~=GG | size(gamma,1)~=(KK+DD) | size(gamma,2)~=CC-GG-1 |...
        size(combos,1)~=CC-GG-1 | size(combos,2)~=GG
    
    test = 1;    
    
end
if(test==1), error('parameter/data inconsistency'); end

% define CM
CM = [zeros(1,GG); eye(GG); combos];

randn('state',sum(100*clock));
draws = randn(NN,KK-1);
X = [ones(NN,1) (S^.5*draws')']; % include constant term
if DD>0
   N = randn(NN,DD);
   DATA = [X N];
else
   DATA = X;
end

% generate matrix of probabilities
Us = DATA*beta;				% Utility of single goods
if CC>GG+1
    for i=GG+2:CC
        U = [Us sum(repmat(CM(i,:),NN,1).*Us,2)+repmat(gamma(i-GG-1),NN,1)];
    end
else
    U = Us;
end
U = [zeros(NN,1) U];
Ue = exp(U);
Ue_sum = repmat(sum(Ue,2),1,CC);
P = Ue./Ue_sum;

% simulate Y1 and Y7
days = zeros(NN,1);
Y1 = simulate(P,1);
Y7 = zeros(NN,1);
Ymat = CM(Y1,:);
for i = 1:NN
    for j = 1:4
        if rand(1)<=pi
            Ynew = simulate(P(i,:),1);
            Ymat(i,:) = min(Ymat(i,:)+CM(Ynew,:),1);
            days(i) = days(i)+1;
        end
    end
end
for i = 1:NN
    for j = 1:CC
        if(Ymat(i,:)==CM(j,:)), Y7(i)=j; end
    end
end
if min(Y7)<=0, error('Choice matrix not complete'); end

% subtract 1 so range of Y1 and Y7 is 0 to (CC-1)
Y1 = Y1-1;
Y7 = Y7-1;

% build Bcons and Bex
Bcons = zeros(CC,CC);
Bex = zeros(CC,CC);
for i = 1:CC
    for j = 1:CC
        if CM(i,:)<=CM(j,:), Bcons(i,j)=1; end
		Bex(i,j) = findinmat(min(CM(i,:)+CM(j,:),1),CM);
    end
end
Bex = Bex-1;

Mcons = zeros(CC,1);
Mincl = zeros(CC,1);

% build CM and calculate CCW
Cincl=[];
CCW=0;
for i=1:CC
	for j=1:CC
		if Bcons(i,j)==1
	        CCW = CCW+1;
			Cincl = [Cincl;[i j]];
		end
	end
end
Cincl = Cincl-1;

% redefine matrices as vectors for use in C function
CM = reshape(CM',GG*CC,1);                    % CM is CC blocks of GG each
Cincl = reshape(Cincl,CCW*2,1);					% Cincl is 2 blocks of CCW each
Bcons = reshape(Bcons',CC^2,1);              % Bcons is CC blocks of CC each
Bex = reshape(Bex',CC*CC,1);  				% Bex is CC blocks of CC each