%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% PROBWEIGHT.M: Calculate probabilities and weights
%
% Matthew Gentzkow
% September 25, 2003
% Online Editions
%
% note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
%       The calculation of probabilities for models with UseDyn==1 and
%       UseDyn==0 is different because the probabilities functions for
%       7-day choices return only the probability of the chosen option
%       whereas the 1-day function returns the probability for all choices.
%       
%
% INPUTS:
%	Xall		NNtot x KK			    observable rhs vars
%	Y1all		NNtot					one-day choices
%	Y7all		NNtot					seven-day choices
%	X		    NN x KK        			observable rhs vars
%	Y1		    NN			    		one-day choices
%	Y7  		NN					    seven-day choices

%   RD          NNtot*DD x msee            random draws
%   bstar       PP                      parameter vector
%	excl		PP					    vector of exclusion restrictions
%	CM			CC x GG				    i*j=1 if good j included in choice i
%	Bcons		CC x CC				    i*j=1 if choice i potentially consistent with choice j
%	Bex			CC x CC				    i*j=KK where 1-day choice j takes overall choice from i to KK
%   nchbundle	CC					    number of consumers choosing each bundle
%   Y1dc        nn x CC                 i x j=1 if consumer i chose bundle j and 0 otherwise
% 	surveyweights	nn			        sampling weights of each observation
%   mstderr     scalar                  number of monte carlo draws
%	UseDyn		scalar				    indicator for using panel data
%   ascribed    NNtot x 1               =1 if ascribed
%	
% OUTPUTS:
%   predshares  CC x 1              predicted shares of each good
%	P1avg		CC x 1      		average 1-day probability of each choice
%   P1minP1avg  CC x 1              average of P1(1-P1) over all consumers and draws (for marginal FX)
%   mweights   msee*NNtot x 1      probability weights for welfare calculation
%   P1cond      NN x CC             probability of each 1-day choice for each consumer with
%                                       expectation over unobservables conditional on 7-day choice
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [predshares,prednchgood,P1avg,P1minP1avg,mweights,P1cond,P1] = ...
    probweight(Xall,Y1all,Y7all,X,Y1,Y7,RD,bstar,excl,CM,Bcons,Bex,msee,UseDyn,ascribed,surveyweights)

global NN NNtot DD CC

% calculate P1 and P1(1-P1)
if DD>0
    RDsmall = randn(NN*DD,msee);
else
    RDsmall = zeros(1,msee);
end
P1 = probabilities1(X',RDsmall,bstar,excl,CM',msee,NN);   % msee*CC x NN
P1avg = mean(reshape(P1,CC,NN*msee),2);
P1minP1 = P1.*(1-P1);
P1minP1avg = mean(reshape(P1minP1,CC,NN*msee)')';

% calculate P1 at each draw
P1allbig = probabilities1(Xall',RD,bstar,excl,CM',msee,NNtot); %(this returns Pr for ALL choices msee*CC x NNtot)

% calculate mweights and P1cond
if UseDyn==1
    % calculate P7 at each draw for the given choice (NNtot x msee)
    P7all = probabilities7(Xall',Y1all,Y7all,RD,bstar,excl,CM',Bcons,Bex,msee,NNtot);
    
    % calculate the average probability across draws for each consumer
    P7allavg = mean(P7all,2);
    
    % calculate a NNtot x msee matrix with (msee times) the posterior probability of each 
    % draw for each consumer
    mweights = P7all./repmat(P7allavg,1,msee); 
else
    Y1allmat = dummyvar(Y1all+1)';
    P1all = repmat(Y1allmat,msee,1).*P1allbig;
    P1all = reshape(P1all,CC,NNtot*msee);
    P1all = sum(P1all);
    P1all = reshape(P1all,msee,NNtot)';
    
    % calculate the average probability across draws for each consumer
    P1allavg = mean(P1all,2);
 
    % calculate a NNtot x msee matrix with (msee times) the posterior probability of each 
    % draw for each consumer
    mweights = P1all./repmat(P1allavg,1,msee);
end
    
% calculate expected P1 for each choice conditional on observed choice
factormat = selectinmat(mweights,1-ascribed); %NNtot x msee
factormat = reshape(factormat',1,NN*msee);
factormat = repmat(factormat,CC,1);
P1cond = reshape(P1,CC,NN*msee);
P1cond = P1cond.*factormat;
P1cond = reshape(P1cond',msee,CC*NN);
P1cond = mean(P1cond,1);
P1cond = reshape(P1cond,NN,CC);

% calculate predicted shares
P1allbig = reshape(P1allbig,CC,NNtot*msee);
P1allbig = reshape(P1allbig',msee,CC*NNtot);
P1allbig = mean(P1allbig,1);
P1allbig = reshape(P1allbig,NNtot,CC);
predshares = P1allbig.*repmat(surveyweights,1,CC);
predshares = sum(predshares)'/sum(surveyweights);
prednchgood =(CM'*predshares*sum(surveyweights))';

% reshape mweights as a vector
mweights = reshape(mweights,msee*NNtot,1);


