%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% UTIL.M: Calculate utilities of each choice, disaggregated into
% 		utility based on observed and unobserved chars, as well
% 		as matrix of probabilities (this is embedded also in C functions,
%		but it's sometimes nice to be able to have the MATLAB code).
%
% Matthew Gentzkow
% September 16, 2002
% Online Editions
%
% note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
%
% INPUTS:
%	X			NN*KK					observable rhs vars
%	RD		    msee*NN*DD			    random draws of unobservable rhs vars
%	bstar		PP						parameter vector
%	CM			CC*GG					i*j=1 if good j included in choice i
%	NN			scalar					let NN be input here so can vary
%	
% OUTPUTS:
%	U			msee*NN x CC			i*j is consumer i's utility for choice j
%	Uobs**		msee*NN x CC			utility setting all unobservables=0 (**optional)
%	Uunobs**	msee*NN x CC			utility setting all observables=0 (**optional)
%	P**			msee*NN x CC			i*j is probability that i chooses j (**optional)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [U,Uobs,Uunobs,P] = util(X,RD,bstar,excl,CM,NN)

global GG CC KK DD PP PMAX ZZ msee

bbig = reverseselect(bstar,1-excl);

CM = reshape(CM,GG,CC)';
Xm = reshape(X',KK,NN)';
if DD>0
	RDm = reshape(RD',DD,msee*NN)';
else
	RDm = [];
end
DATAm = [repmat(Xm,msee,1) RDm];
beta = reshape(bbig(1:(KK+DD)*GG),KK+DD,GG);
gamma = reshape(bbig((KK+DD)*GG+1:PMAX-1),KK+DD,CC-GG-1);
Us = DATAm*beta;				% Utility of single goods
Uc = DATAm*gamma;			    % Extra utility of combinations
U = Us;
if CC>GG+1
    for i=GG+2:CC
        U = [U sum(repmat(CM(i,:),NN*msee,1).*Us,2)+Uc(:,i-GG-1)];
    end
end
U = [zeros(NN*msee,1) U];

if nargout > 1
	betaobs = beta(1:KK,:);
	gammaobs = gamma(1:KK,:);
	Us = DATAm(:,1:KK)*betaobs;				% Utility of single goods
	Uc = DATAm(:,1:KK)*gammaobs;			% Extra utility of combinations
	Uobs = Us;
	if CC>GG+1
    	for i=GG+2:CC
        	Uobs = [Uobs sum(repmat(CM(i,:),NN*msee,1).*Us,2)+Uc(:,i-GG-1)];
    	end
	end
	Uobs = [zeros(NN*msee,1) Uobs];
end

if nargout > 2 & DD>0
	betaunobs = beta(KK+1:KK+DD,:);
	gammaunobs = gamma(KK+1:KK+DD,:);
	Us = DATAm(:,KK+1:KK+DD)*betaunobs;				% Utility of single goods
	Uc = DATAm(:,KK+1:KK+DD)*gammaunobs;			% Extra utility of combinations
	Uunobs = Us;
	if CC>GG+1
    	for i=GG+2:CC
 	       Uunobs = [Uunobs sum(repmat(CM(i,:),NN*msee,1).*Us,2)+Uc(:,i-GG-1)];
 	   end
	end
	Uunobs = [zeros(NN*msee,1) Uunobs];
elseif nargout > 2 & DD==0
	Uunobs = zeros(NN*msee,CC);
end
	
if nargout > 3
	Ue = exp(U);
	Ue_sum = repmat(sum(Ue,2),1,CC);
	P = Ue./Ue_sum;
end