%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% RUNOED.M: Shell function for estimation of OED Model.
%
% Matthew Gentzkow
% September 16, 2002
% Online Editions
%
% note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
%
% INPUTS:
%	filename	    string				key of input file in current directory (run###) where ### is key
%	b0		        PP				    starting value of parameters (**OPTIONAL)
%	RD		        NN*DD x mf          random draws (**OPTIONAL)
%   weights         NN x mf             importance sampling weights
%                                       (**OPTIONAL)
%
% VARIABLES IN INPUT FILE:
%	Y1all		    ntot				one-day choices
%	Y7all		    ntot				seven-day choices
%	Xall		    ntot x KK			observable rhs vars
%	CM			    CC x GG				i,j=1 if good j included in choice i
%	Cincl			CCW x 2				i,1:2 gives the 1 and 7-day choices incl in combo choice i
%	Bcons		    CC x CC				i,j=1 if choice j potentially consistent with choice i
%	Bex			    CC x CC				i,j=k where 1-day choice i takes overall choice from j to k
%	header		    KK+DD				array of variable names
% 	surveyweights	NN				    sampling weights of each observation
%	ascribed	    NN					=1 if observation is ascribed
%	release		    NN					=1 if observation is in the single release (to be used in calc)
%	R			    (PP-ZZ) x PP		matrix of linear restrictions
%	r			    (PP-ZZ)				vector of linear restrictions (Rb=r)
%	excl		    PP					vector of exclusion restrictions (=1 if excluded)
%	lhslist1	    GG					names of goods included in estimation
%	
%
% FILES SAVED:
%	OEDest### (final parameter estimate, standard error, S, 
%				L, dL, CM, excl, lhslist1, and parameters)
%	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function RUNOED(filekey,b0,RD,weights)

global UseDyn ImpSamp NN NNtot GG CC KK DD PP PMAX ZZ CCW tolf0 tolff tolx0 tolxf...
       mfe0 mfef miter0 miterf m0 mis msteps mf mtest msee melast

% set filenames
inputfile = strcat('run',filekey);
outputfile = strcat('OEDest',filekey);
outputfileRD = strcat('OEDestRD',filekey);

% load input variables
cd 'M:\PROJECTS\PrintOnline\current run\input'
load(inputfile,'Y1all','Y7all','Xall','CM','Cincl','Bcons','Bex','headers','surveyweights',...
				'ascribed','R','r','excl','lhslist1','buildparams')
if (wrongbuildparams(buildparams))
    error('Parameters in data do not match those with which C-files were compiled');
end

% set parameters            
[NNtot NN GG CC KK DD PMAX PP ZZ CCW XX UseDyn] = extract(buildparams);
[ImpSamp,tolf0,tolff,tolx0,tolxf,mfe0,mfef,miter0,miterf,m0,mis,msteps,mf,mtest,msee,melast]=setestparams;           
         
% change to output directory
cd 'M:\PROJECTS\PrintOnline\current run\results'

% seed random number generator
randn('state',sum(100*clock));

% remove ascribed observations
surveyweights = selectinmat(surveyweights,1-ascribed);
Y1 = selectinmat(Y1all,1-ascribed);
Y7 = selectinmat(Y7all,1-ascribed);
X = selectinmat(Xall,1-ascribed);

% generate transformation matrices to impose constraints
[T A] = transmats(R,r);

% set starting value of parameters b0 (if not given)
if nargin~=1 & nargin~=2 & nargin~=4, error('Wrong number of arguments'); end
if nargin==1 & DD>0
    b0 = genb0(Y1,Y7,X,excl,T,A,CM,Bcons,Bex);
elseif nargin==1 & DD==0
	b0 = zeros(PP,1);
end

% generate random draws and weights
if nargin<3 & ImpSamp==1
	[RD, weights] = genis(X,Y1,Y7,CM,Bcons,Bex,b0,excl,mf);
elseif nargin<3 & ImpSamp==0 & DD>0
    RD = randn(NN*DD,mf);
	weights = ones(NN,mf);
elseif nargin<3 & ImpSamp==0 & DD==0
    RD = 0;
    weights = ones(NN,1);
end

% estimation routine: SML estimates
[bsml,SE1,S1,SE2,S2,SE3,S3,L,dL,dLc,Hc] = ESTsml(b0,excl,Y1,Y7,X,RD,weights,T,A,CM,Bcons,Bex);
bstar = bsml;

% reparameterize pi and gam to be in [0,1] and adjust std errors by delta method
for ii=PP-1:PP
    bstar(ii) = exp(bstar(ii))/(1+exp(bstar(ii)));
    df = 1/(1+exp(bstar(ii)))^2;
    S1(ii,:) = S1(ii,:).*repmat(df,1,PP);
    S1(:,ii) = S1(:,ii).*repmat(df,PP,1);
    SE1(ii) = SE1(ii).*df;
    S2(ii,:) = S2(ii,:).*repmat(df,1,PP);
    S2(:,ii) = S2(:,ii).*repmat(df,PP,1);
    SE2(ii) = SE2(ii).*df;
    S3(ii,:) = S3(ii,:).*repmat(df,1,PP);
    S3(:,ii) = S3(:,ii).*repmat(df,PP,1);
    SE3(ii) = SE3(ii).*df;
end
    
% save
sprintf('Saving results')
buildparams = [NNtot NN GG CC KK DD PMAX PP ZZ CCW];
estparams = [UseDyn,ImpSamp,tolf0,tolff,tolx0,tolxf,mfe0,mfef,miter0,miterf,m0,mis,mf,mtest,msee,melast];
save(outputfile,'bstar','dL','L','dLc','Hc','SE1','S1','SE2','S2','SE3','S3','headers','lhslist1','buildparams','estparams','msteps');
save(outputfileRD,'RD','weights');

