/***************************************************************************
 *
 * OEDCALCPROB.H: Header file for functions to calculate choice probabilities.
 *
 *	note: calling file must include "matlab.h"
 *
 * Matthew Gentzkow
 * November 27, 2004
 * Online Editions
 *
 ****************************************************************************/


 /*********************************************************************************************
 *
 * CALCP1ALL: Function that calculates one-day probabilities and derivatives of probabilities
 *              for all simulation draws for all choices for a given consumer.
 *
 *              *** if dP1==NULL calculates probabilities but not derivatives ***
 *
 *	P1			m*CC				(OUTPUT) vector of probabilities of each choice at each draw
 *	dP1			m*CC*PP				(OUTPUT) m*i*j is derivative of P(i) w.r.t. parameter j for draw m
 *	U			m*NN*(CC-1)			vector of utilities
 *	b			PP					unconstrained parameter vector
 *	excl		PMAX				vector of exclusion restrictions (=1 if excluded)
 *	X			NN*KK				observable rhs vars
 *	RD			m*NN*DD				random draws of unobservable rhs vars
 *	CM			CC*GG				i*j=1 if good j included in choice i
 *  uadd        CC-1                incremental utility to add with probability gam
 *  constants   KK                  location in parameter vector of constants for each good
 *  gam         scalar              probability of a good news day
 *  dgam        scalar              derivative of the unconstrained parameter gamhat w.r.t. gam
 *  m           scalar              number of simulation draws
 *  consindex   scalar              number of the consumer for which to calculate
 *
 *********************************************************************************************/

void calcp1all(double *P1, double *dP1, double *U, double *b, double *excl, double *X, 
                double *RD, double *CM, double *uadd, int *constants, double gam, double dgam, 
                int m, int consindex);
                  
        
 /*********************************************************************************************
 *
 * CALCP7ALL: Function that calculates seven-day probabilities and derivatives of probabilities
 *              for all simulation draws for a given consumer.
 *
 *              *** if dP7==NULL calculates probabilities but not derivatives ***      
 *
 *	P7			m   				(OUTPUT) vector of probabilities of each choice at each draw
 *	dP7			m*PP				(OUTPUT) m*j is derivative of P w.r.t. parameter j for draw m
 *	U			m*NN*(CC-1)			vector of utilities
 *	b			PP					unconstrained parameter vector
 *	excl		PMAX				vector of exclusion restrictions (=1 if excluded)
 *	X			NN*KK				observable rhs vars
 *	RD			m*NN*DD				random draws of unobservable rhs vars
 *	CM			CC*GG				i*j=1 if good j included in choice i
 *	Bcons		CC*CC				i*j=1 if choice i potentially consistent with choice j
 *	Bex			CC*CC				i*j=k where 1-day choice j takes overall choice from i to k
 *	coeff		5					coeff(i) is pr(excatly i indep choices were made)
 *	dcoeff		5					dcoeff(i) is deriv of coeff(i) w.r.t. pi
 *	Y1			NN					one-day choices
 *	Y7			NN					seven-day choices
 *  uadd        CC-1                incremental utility to add with probability gam
 *  constants   KK                  location in parameter vector of constants for each good
 *  gam         scalar              probability of a good news day
 *  dgam        scalar              derivative of the unconstrained parameter gamhat w.r.t. gam
 *  m           scalar              number of simulation draws
 *  consindex   scalar              number of the consumer for which to calculate
 *
 *********************************************************************************************/

void calcp7all(double *P7, double *dP7, double *U, double *b, double *excl, double *X, double *RD, double *CM, 
				   double *Bcons, double *Bex, double *coeff, double *dcoeff, double *Y1, double *Y7, 
                   double *uadd, int *constants, double gam, double dgam, int m, int consindex);

    
/*********************************************************************************************
 *
 * CALC1TG: Function to calculate one-day probabilities using tau and gam parameters.
 *          Calculates probabilities for a single consumer/draw.
 *
 *          *** if dP1==NULL does not calculate derivatives ***
 *
 *	P1			CC                  (OUTPUT) vector of probabilities of each choice at each draw
 *	dP1			CC*PP				(OUTPUT) m*i*j is derivative of P(i) w.r.t. parameter j for draw m
 *	U			(CC-1)              vector of utilities
 *	b			PP					unconstrained parameter vector
 *	excl		PMAX				vector of exclusion restrictions (=1 if excluded)
 *	X			KK      			observable rhs vars
 *	RD			DD      			random draws of unobservable rhs vars
 *	CM			CC*GG				i*j=1 if good j included in choice i
 *  uadd        CC-1                incremental utility to add with probability gam
 *  constants   KK                  location in parameter vector of constants for each good
 *  gam         scalar              probability of a good news day
 *  dgam        scalar              derivative of the unconstrained parameter gamhat w.r.t. gam
 *
 *********************************************************************************************/
   
void calc1tg(double *P1, double *dP1, double *U, double *b, double *excl, double *X, double *RD, 
                double *CM, double *uadd, int *constants, double gam, double dgam); 


/*********************************************************************************************
 *
 * CALCP1: Function that calculates probability of each 1-day choice for a particular consumer
 *
 * note: a*b*c in dimension means vectors stacked as a groups of b groups of c
 *
 * INPUTS:
 *	P1			CC					(OUTPUT) vector of probabilities of each choice
 *	U			CC-1				utilities of each choice (other than choice 0)
 *
 *
 *********************************************************************************************/

void calcp1(double *P1, double *U); 


/***********************************************************************************************
 *
 * CALCDP1: Function that calculates derivatives of 1-day probabilities for single consumer.
 *
 * note: this routine does not set the derivatives w.r.t. pi and all derivatives
 *		 with respect to excluded parameters (for which b==0 exactly) are set
 *		 equal to zero.	
 *
 * note: a*b*c in dimension means vectors stacked as a groups of b groups of c
 *
 * INPUTS:
 *	dP1			CC*PP				(OUTPUT) i*j is derivative of P(i) w.r.t. parameter j
 *	P1			CC					vector of probabilities of each choice
 *	b			PP					unconstrained parameter vector
 *	excl		PMAX				vector of exclusion restrictions (=1 if excluded)
 *	X			KK					observable rhs vars for given consumer
 *	RD			DD					unobservable draws for given consumer
 *	CM			CC*GG				i*j=1 if good j included in choice i
 *
 *********************************************************************************************/

void calcdp1(double *dP1, double *P1, double *b, double *excl, double *X, double *RD, double *CM);


/***********************************************************************************************
 *
 * CALCPDP7: Function that calculates probabilities and derivatives of observed
 *			 7-day choice for a given consumer.
 *
 *          *** if dP7==NULL calculates probabilities but not derivatives ***
 *
 * note: a*b*c in dimension means vectors stacked as a groups of b groups of c
 *
 * INPUTS:
 *	P7			scalar				(OUTPUT) probability of observed choice
 *	dP7			PP					(OUTPUT) derivatives of observed choice
 *	c1			scalar				1-day choice
 *	c7			scalar				7-day choice
 *	P1			CC					probabilities of 1-day choices
 *	dP1			CC*PP				i*j is derivative of P(i) w.r.t. parameter j
 *	Bcons		CC*CC				i*j=1 if choice i potentially consistent with choice j
 *	Bex			CC*CC				i*j=k where 1-day choice j takes overall choice from i to k
 *	coeff		5					coeff(i) is pr(excatly i indep choices were made)
 *	dcoeff		5					dcoeff(i) is deriv of coeff(i) w.r.t. pi
 *
 *********************************************************************************************/

void calcpdp7(double *P7, double *dP7, int c1, int c7, double *P1, double *dP1, double *Bcons, 
					   double *Bex, double *coeff, double *dcoeff);


/*********************************************************************************************
 *
 * CALCL: Function that calculates likelihood and derivatives with bias correction for
 *		  a single consumer (averaged over m simulation draws).
 *
 * note: a*b*c in dimension means vectors stacked as a groups of b groups of c
 *
 * INPUTS:
 *	Li			scalar				(OUTPUT) (log) likelihood value
 *	dLi			PP					(OUTPUT) derivatives of likelihood
 *	P7			m					7-day probabilities of observed choices for m draws
 *	dP7			m*PP				derivatives of 7-day probabilities
 *	weights		m					observation weights
 *	m			scalar				number of simulation draws
 *	
 *********************************************************************************************/

void calcL(double *Li, double *dLi, double *P7, double *dP7, double *weights, int m);


/***********************************************************************************************
 *
 * CALCP7: Function that calculates probabilities of observed 7-day choice for a given consumer,
 *          with P1 as an input.
 *
 * note: a*b*c in dimension means vectors stacked as a groups of b groups of c
 *
 * INPUTS:
 *	P7			scalar				(OUTPUT) probability of observed choice
 *	c1			scalar				1-day choice
 *	c7			scalar				7-day choice
 *	P1			CC					probabilities of 1-day choices
 *	Bcons		CC*CC				i*j=1 if choice i potentially consistent with choice j
 *	Bex			CC*CC				i*j=k where 1-day choice j takes overall choice from i to k
 *	coeff		5					coeff(i) is pr(excatly i indep choices were made)
 *
 *********************************************************************************************/

void calcp7(double *P7, int c1, int c7, double *P1, double *Bcons, double *Bex, double *coeff);


 /*********************************************************************************************
 *
 * CALCP1ALLND: Function that calculates one-day probabilities (but not derivatives) 
 *              for all simulation draws for all choices for a given consumer.
 *
 *	P1			m*CC				(OUTPUT) vector of probabilities of each choice at each draw
 *	U			m*NN*(CC-1)			vector of utilities
 *  uadd        CC-1                incremental utility to add with probability gam
 *  gam         scalar              probability of a good news day
 *  m           scalar              number of simulation draws
 *  consindex   scalar              number of the consumer for which to calculate
 *
 *********************************************************************************************/
                
void calcp1allnd(double *P1, double *U, double *uadd, double gam, int m, int consindex);


    
/*********************************************************************************************
 *
 * CALCP7ALLND: Function that calculates seven-day probabilities (but not derivative)
 *              for all simulation draws for the actual choice of a given consumer.
 *
 *	P7			m   				(OUTPUT) vector of probabilities of each choice at each draw
 *	U			m*NN*(CC-1)			vector of utilities
 *	Bcons		CC*CC				i*j=1 if choice i potentially consistent with choice j
 *	Bex			CC*CC				i*j=k where 1-day choice j takes overall choice from i to k
 *	coeff		5					coeff(i) is pr(excatly i indep choices were made)
 *	Y1			NN					one-day choices
 *	Y7			NN					seven-day choices
 *  uadd        CC-1                incremental utility to add with probability gam
 *  gam         scalar              probability of a good news day
 *  m           scalar              number of simulation draws
 *  consindex   scalar              number of the consumer for which to calculate
 *
 *********************************************************************************************/

void calcp7allnd(double *P7, double *U, double *Bcons, double *Bex, double *coeff, double *Y1, 
                    double *Y7, double *uadd, double gam, int m, int consindex);



