 /*******************************************************************************************
 *
 * CROSSDERIVW: Calculate cross-derivatives (i.e. change in demand for
 *				i with respect to utility of good j). Used, then, to
 *				compute elasticities. This version calculates a weighted average
 *				over simulation draws as for importance sampling.
 *
 * Matthew Gentzkow
 * July 14, 2003
 * Online Editions
 * 
 * MATLAB prototype:
 * CD = crossderiv2(b,excl,m,ninput,X,RD,CM,mweights)
 *
 * compiler command: 
 *	mex crossderiv2.c oedcalc.c oedtools.c
 *
 * note: a*b*c in dimension means vectors stacked as a groups of b groups of c
 * note: # before range means only integer values are valid
 *
 * INPUTS:
 *	b			PP				(-inf,inf)		param vec: c groups of (k+d) then f(pi)
 *	excl		PMAX			#(0,1)			vec of exclusion restrictions (=1 if excl)
 *	m			scalar			#(0,inf)		number of simulation draws
 *	ninput		scalar			#(0,inf)		number of consumers
 *	X			KK x ninput		(-inf,inf)		observable rhs vars
 *	RD			ninput*DD x m	(-inf,inf)		random draws of unobservable rhs vars
 *	CM			GG x CC			#(0,1)			i x j=1 if good i included in choice j
 *	mweights	m*ninput		(0,1)			i*j is weight for draw j for consumer i
 *
 * OUTPUTS:
 *	CD			GG*GG x ninput	(-inf,inf)		i*j*k is change in prob that i chooses j w.r.t.
 *												utility of k (or change in prob of k w.r.t. j)
 *
 **********************************************************************************************/

#include <math.h>
#include "mex.h"
#include "choose_max.h"
#include "oedcalc.h"
#include "oedtools.h"

#define NINPUTS 8
#define NOUTMAX 1

/* ****************************************************** */
/* mexFunction: MATLAB gateway routine                    */
/*														  */
/* remember: m is rows, n is columns					  */
/*													      */
/* ****************************************************** */


void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[] )
{
  int n[NINPUTS], m[NINPUTS], ngood[NINPUTS], mgood[NINPUTS], integer[NINPUTS],
	  nout[NOUTMAX], mout[NOUTMAX];
  double min[NINPUTS], max[NINPUTS];
  char errstring[200];
  int ii, ninput, mm, rdval;
  double *IPTRS[NINPUTS], *OPTRS[NOUTMAX];
  
  /* get dimension-relevant variable ninput early so we can use it for error-checks */
  mm = (int)*mxGetPr(prhs[2]);
  ninput = (int)*mxGetPr(prhs[3]);

  /* handle the case of DD==0 */
  rdval = choose_max(ninput*DD,1);
  
  /* set ngood and mgood arrays with expected input dimensions */
  mgood[0] = PP;			ngood[0] = 1;
  mgood[1] = PMAX;			ngood[1] = 1;
  mgood[2] = 1;				ngood[2] = 1;
  mgood[3] = 1;				ngood[3] = 1;
  mgood[4] = KK;			ngood[4] = ninput;
  mgood[5] = rdval;         ngood[5] = mm;
  mgood[6] = GG;			ngood[6] = CC;
  mgood[7] = ninput*mm;		ngood[7] = 1;

  /* set min and max arrays with allowable range of inputs */
  /* (enter 9999 or -9999 to represent +/- infinity)  */
  /* integer[ii]=1 indicates input must be an integer */
  min[0] = -9999;		max[0] = 9999;		integer[0] = 0;
  min[1] = 0;			max[1] = 1;			integer[1] = 1;
  min[2] = 0;			max[2] = 9999;		integer[2] = 1;
  min[3] = 0;			max[3] = 9999;		integer[3] = 1;
  min[4] = -9999;		max[4] = 9999;		integer[4] = 0;
  min[5] = -9999;		max[5] = 9999;		integer[5] = 0;
  min[6] = 0;			max[6] = 1;			integer[6] = 1;
  min[7] = 0;			max[7] = 9999;			integer[7] = 0;

  /*set nout and mout arrays with output dimensions */
  mout[0] = GG*GG;	nout[0] = ninput;

  /* check number of arguments */
  if(nrhs != NINPUTS)
    mexErrMsgTxt("NINPUTS inputs required.");
  if(nlhs > NOUTMAX)
    mexErrMsgTxt("At most NOUTMAX outputs allowed.");

  /* get pointers and dimensions of each input element */
  for (ii=0; ii<NINPUTS; ii++) {
	  IPTRS[ii] = mxGetPr(prhs[ii]);
	  n[ii] = mxGetN(prhs[ii]);
	  m[ii] = mxGetM(prhs[ii]);
  }

  /* check dimensions of inputs */
  for (ii=0; ii<NINPUTS; ii++) {
	  if(n[ii] != ngood[ii]) {
		  sprintf(errstring,"Input number %d: column dimension is %d but should be %d", ii+1, n[ii], ngood[ii]);
		  mexErrMsgTxt(errstring);
	  }
	  if(m[ii] != mgood[ii]) {
		  sprintf(errstring,"Input number %d: row dimension is %d but should be %d", ii+1, m[ii], mgood[ii]);
		  mexErrMsgTxt(errstring);
	  }
  }

  /* check range of inputs */
  for (ii=0; ii<NINPUTS; ii++) {
	  if( outofrange(IPTRS[ii],n[ii]*m[ii],min[ii],max[ii]) ) {
		  sprintf(errstring,"Input number %d: value out of range.", ii+1);
		  mexErrMsgTxt(errstring);
	  }
	  if(integer[ii]==1){
		  if( noninteger(IPTRS[ii],n[ii]*m[ii]) ) {
			sprintf(errstring,"Input number %d: values must be integers.", ii+1);
			mexErrMsgTxt(errstring);
		  }
	  }
  }

  /* initialize output elements */
  for (ii=0; ii< ((nlhs>0)? nlhs:1) ; ii++) {
	  plhs[ii] = mxCreateDoubleMatrix(mout[ii],nout[ii],mxREAL);
	  OPTRS[ii] = mxGetPr(plhs[ii]);
  }

  /* call C function */
  crossderiv(OPTRS[0], IPTRS[0],IPTRS[1],(int)*IPTRS[2],(int)*IPTRS[3],IPTRS[4],IPTRS[5],
			IPTRS[6],IPTRS[7]);


}



/**********************************************************************
 *
 * OUTOFRANGE: Returns 1 if there is a value in vector greater
 *				than max or less than min; returns 0 otherwise.
 *				min/max values of +/-9999 are interpreted as +/- infinity.
 *				Returns 0 otherwise.
 *
 ***********************************************************************/

int outofrange(double *vector, int length, double min, double max) 
{

	int ii;

	for (ii=0; ii<length; ii++) {
		if ( (max!=9999 & vector[ii]>max) | (min!=-9999 & vector[ii]<min) ) {
			return(1);
		}
	}

	return(0);
}


/**********************************************************************
 *
 * NONINTEGER: Returns 1 if there is a value in vector whose
 *				decimal portion is not zero. Returns 0 otherwise
 *
 ***********************************************************************/

int noninteger(double *vector, int length) 
{

	int ii;

	for (ii=0; ii<length; ii++) {
		if ( fmod(vector[ii],1) ) {
			return(1);
		}
	}

	return(0);
}



