%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% ESTmsm.M: Run MSM estimation routine.
%
% Matthew Gentzkow
% July 17, 2003
% Online Editions
%
% note: a*b*c in dimension means vectors stacked as a groups of b groups of c
%
% INPUTS:
%	b0			PP					starting value of (constrained) parameter vector
%	excl		PMAX				vector of exclusion restrictions (=1 if excluded)
%	Y1			NN					one-day choices
%	Y7			NN					seven-day choices
%	X			NN*KK				observable rhs vars
%	RD		    mf*NN*DD			   random draws of unobservable rhs vars
%	weights		mf*NN				weights of each obs (not = 1 in imp sampling, e.GG.)
%	T			ZZ*PP				constraint matrix
%	A			PP					constraint vector
%	CM			CC*GG				i*j=1 if good j included in choice i
%	Cincl		2*CCW				1:2*i gives the 1 and 7-day choices incl in combo choice i
%	Bcons		CC*CC				i*j=1 if choice i potentially consistent with choice j
%	Mcons		CC					first vector for missing 1-day data
%	Mincl		CC					second vector for missing 1-day data
%	Bex			CC*CC				i*j=KK where 1-day choice j takes overall choice from i to KK
%
%	
% OUTPUTS:
%	bstar		PP					parameter vector CC groups of (KK+DD) followed by f(pi)
%	SE			PP					vector of standard errors
%	L			scalar				(maximized) log likelihood
%	dL			PP					gradient vector (derivatives of log likelihood)
%
% FILES SAVED:
%	bmsm (msm parameter estimate)
%	MSMout (final parameter estimate and standard error)
%	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [bstar,SE,S,L,dL] = ESTmsm(b0,excl,Y1,Y7,X,RD,weights,T,A,CM,Cincl,Bcons,Mcons,Mincl,Bex)
	
global UseDyn ImpSamp NN GG CC CCW KK DD PP ZZ tolfmsm mfe0 mfef m0 mis msteps mf mz mmsm

Tm = reshape(T,PP,ZZ);
CCm = reshape(Cincl,CCW,2);
b0c = (b0'*Tm)';

% calculate complete gradient vector (optimal instruments for MSM)
sprintf('Calculating MSM instruments')
%tic
%bZ = b0;
%RDz = randn(mz*NN*DD,1);			% use equal weighting and no imp sampling
%weightsz = ones(mz*NN,1);
%[P,Z] = calc7big(bZ,excl,Y1,Y7,X,RDz,weightsz,CM,Cincl,Bcons,Mcons,Mincl,Bex,mz);
%clear RDz weightsz;
%toc
%save ztemp Z;
load ztemp Z;

% create big vector of dummies for combination choices
Cdum = zeros(NN,CCW);
for i = 1:NN
	Cdum(i,findinmat([Y1(i) Y7(i)],CCm))=1;
end
Cdum = reshape(Cdum',NN*CCW,1);

% generate new random draws (for now no importance sampling... too many choices?)
RD = randn(mmsm*NN*DD,1);
weights=  ones(mmsm*NN,1);

% estimation step
OPTIONS = optimset('Display','iter','MaxFunEvals',mfef,'TolFun',tolfmsm,'TolX',tolfmsm,...
    'GradObj','on','LargeScale','off','DerivativeCheck','off');
sprintf('MSM estimation step')
tic
[bc,L,exitflag,output,dLc,Hc] = fminunc('MSMobj',b0c,OPTIONS,excl,Z,Y1,Y7,Cdum,X,RD,weights,T,A,CM,Cincl,Bcons,Mcons,Mincl,Bex,mmsm);
toc
bstar = (bc'*Tm'+A)'
save btemp bstar

% transform
L = -L; dLc = -dLc; Hc = -Hc;
dL = (dLc'*Tm'+A)';

% calculate standard errors 
[Pstar,Zstar] = calc7big(bstar,excl,Y1,Y7,X,RD,weights,CM,Cincl,Bcons,Mcons,Mincl,Bex,mmsm);
Z = reshape(Z,NN*CCW,PP);
Zstar = reshape(Zstar,NN*CCW,PP);
G = (1/NN)*-Z'*Zstar;                                    % derivative of moments
M = reshape(Z,NN*CCW,PP).*repmat(Cdum-Pstar,1,PP);          % moments
V = (1/NN)*(M'*M);                                       % sample variance of moments
Su = (1/NN)*inv(G'*G)*G'*((1/NN)*M'*M)*G*inv(G'*G);       % asym var of estimator (w/out constraints)
S = (Tm*Tm')*Su*(Tm*Tm');                                   % impose constraints
SE = sqrt(diag(S));

save MSMest bstar SE;
