%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% RUNOED.M: Shell function for estimation of OED Model.
%
% Matthew Gentzkow
% September 16, 2002
% Online Editions
%
% note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
%
% INPUTS:
%	filename	    string				key of input file in current directory (run###) where ### is key
%	b0		        PP				    starting value of parameters (**OPTIONAL)
%
% VARIABLES IN INPUT FILE:
%	Y1all		    ntot				one-day choices
%	Y7all		    ntot				seven-day choices
%	Xall		    ntot x KK			observable rhs vars
%	CM			    CC x GG				i,j=1 if good j included in choice i
%	Cincl			CCW x 2				i,1:2 gives the 1 and 7-day choices incl in combo choice i
%	Bcons		    CC x CC				i,j=1 if choice j potentially consistent with choice i
%	Bex			    CC x CC				i,j=k where 1-day choice i takes overall choice from j to k
%	header		    KK+DD				array of variable names
%	Mcons		    CC					first vector for missing 1-day data
%	Mincl		    CC					second vector for missing 1-day data
% 	surveyweights	NN				    sampling weights of each observation
%	ascribed	    NN					=1 if observation is ascribed
%	release		    NN					=1 if observation is in the single release (to be used in calc)
%	R			    (PP-ZZ) x PP		matrix of linear restrictions
%	r			    (PP-ZZ)				vector of linear restrictions (Rb=r)
%	excl		    PP					vector of exclusion restrictions (=1 if excluded)
%	lhslist1	    GG					names of goods included in estimation
%	
%
% FILES SAVED:
%	OEDest### (final parameter estimate, standard error, S, 
%				L, dL, CM, excl, lhslist1, and parameters)
%	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function RUNOED(filekey,b0)

global UseDyn ImpSamp NN NNtot GG CC KK DD PP PMAX ZZ CCW tol0 tolf...
       mfe0 mfef m0 mis msteps mf mtest mz msee melast

% set filenames
inputfile = strcat('run',filekey);
outputfile = strcat('OEDest',filekey);
outputfileRD = strcat('OEDestRD',filekey);

% load input variables
cd N:/RESEARCH/OED/Input
%cd /homes/nber/gentzkow/RESEARCH/OED/Input
load(inputfile,'Y1all','Y7all','Xall','CM','Cincl','Bcons','Bex','headers','Mcons','Mincl','surveyweights',...
				'ascribed','R','r','excl','lhslist1','buildparams')
if (wrongbuildparams(buildparams))
    error('Parameters in data do not match those with which C-files were compiled');
end

% set parameters            
[NNtot NN GG CC KK DD PMAX PP ZZ CCW] = extract(buildparams);
[UseDyn ImpSamp tol0 tolf tol0msm tolfmsm mfe0 mfef m0 mis mf mtest mz mmsm msee melast] = setestparams;            

% change to output directory
cd N:/RESEARCH/OED/Results
%cd /homes/nber/gentzkow/RESEARCH/OED/Results

% seed random number generator
randn('state',sum(100*clock));

% remove ascribed observations
surveyweights = selectinmat(surveyweights,1-ascribed);
Y1 = selectinmat(Y1all,1-ascribed);
Y7 = selectinmat(Y7all,1-ascribed);
X = selectinmat(Xall,1-ascribed);

% generate transformation matrices to impose constraints
[T A] = transmats(R,r);

% set starting value of parameters b0 (if not given)
if nargin~=1 & nargin~=2, error('Wrong number of arguments'); end
if nargin==1 & DD>0
    b0 = genb0(Y1,Y7,X,excl,T,A,CM,Bcons,Mcons,Mincl,Bex);
elseif nargin==1 & DD==0
	b0 = zeros(PP,1);
end

% generate random draws and weights
if ImpSamp==1 & DD>0
	[RD, weights] = genis(X,Y1,Y7,CM,Bcons,Mcons,Mincl,Bex,b0,excl,mf);
elseif ImpSamp==0 & DD>0
    RD = randn(NN*DD,mf);
	weights = ones(NN,mf);
elseif DD==0
    mf = 1;
    RD = 0;
    weights = ones(NN,1);
end

% estimation routine: SML estimates
[bsml,SE,S,SE1,S1,L,dL,dLc,Hc] = ESTsml(b0,excl,Y1,Y7,X,RD,weights,T,A,CM,Bcons,Mcons,Mincl,Bex);

% estimation routine:  MSM estimates
%if DD>0
	%[bstar,SE,S,L,dL] = ESTmsm(bsml,excl,Y1,Y7,X,RD,weights,T,A,CM,Cincl,Bcons,Mcons,Mincl,Bex);
    %else
	%bstar = bsml;
    %end

bstar = bsml;

% reparameterize pi to be in [0,1] and adjust std errors by delta method
bstar(PP) = exp(bstar(PP))/(1+exp(bstar(PP)));
df = 1/(1+exp(bstar(PP)))^2;
S(PP,:) = S(PP,:).*repmat(df,1,PP);
S(:,PP) = S(:,PP).*repmat(df,PP,1);
SE(PP) = SE(PP).*df;
S1(PP,:) = S1(PP,:).*repmat(df,1,PP);
S1(:,PP) = S1(:,PP).*repmat(df,PP,1);
SE1(PP) = SE1(PP).*df;

% save
sprintf('Saving results')
buildparams = [NNtot NN GG CC KK DD PMAX PP ZZ CCW];
estparams = [UseDyn ImpSamp tol0 tolf tol0msm tolfmsm mfe0 mfef m0 mis mf mtest mz mmsm msee melast];
save(outputfile,'bstar','dL','L','dLc','Hc','SE','S','SE1','S1','headers','lhslist1','buildparams','estparams','msteps');
save(outputfileRD,'RD');

