%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CTESTSTAT.M: Calculate test statistic for no bias in SML estimator.
%
% Matthew Gentzkow
% September 16, 2002
% Online Editions
%
% note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
%
% INPUTS:
%	bstar		PP					parameter vector CC groups of (KK+DD) followed by f(pi)
%	excl		PMAX				vector of exclusion restrictions (=1 if excluded)
%	X			NN*KK					observable rhs vars
%	weights		NN					weights of each obs (not = 1 in imp sampling, e.GG.)
%	CM			CC*GG					i*j=1 if good j included in choice i
%	Bcons		CC*CC					i*j=1 if choice i potentially consistent with choice j
%	Mcons		CC					first vector for missing 1-day data
%	Mincl		CC					second vector for missing 1-day data
%	Bex			CC*CC					i*j=KK where 1-day choice j takes overall choice from i to KK
%
%	
% OUTPUTS:
%	W			scalar				test statistic (distributed chi2 with PP deg of freedom)
%	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [W,pval] = cteststat(bstar,excl,X,weights,CM,Bcons,Mcons,Mincl,Bex)

global NN PP KK DD mf mtest

G = zeros(mtest,PP);
m = zeros(1,PP);
V = zeros(PP,PP);

Xm = reshape(X,KK,NN)';

for i = 1:NN
   
    Xtest = repmat(Xm(i,:),mtest,1);
    Xtest = reshape(Xtest',mtest*KK,1);
    RDtest = randn(mf*mtest*DD,1);
	[Ydraws1,Ydraws7] = ctestsim(X,bstar,excl,CM,i);
	gvec = testgrad(bstar,excl,Ydraws1,Ydraws7,Xtest,RDtest,weights,CM,Bcons,Mcons,Mincl,Bex,mf);
	G = reshape(gvec,PP,mtest)';
	m = m + (1/mtest)*sum(G,1);
	V = V + cov(G);
end
m = (1/NN)*m';
V = (1/NN)*V;

W = mtest*NN*m'*V^-1*m;
pval = 1-chi2cdf(W,PP);
	