%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% PROFITWEBPRICE.M: Calculate Post company profit for a given web price
%
% Matthew Gentzkow
% September 25, 2003
% Online Editions
%
% note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
%
% INPUTS:
%   price           scalar                  price of web edition
%   tcost           scalar                  transaction cost of buying
%   U               msee*NNtot x CC-1       utilities (at price of 0)
%	CM			    CC x GG				    i*j=1 if good j included in choice i
% 	surveyweights	NNtot			        sampling weights of each observation
%   piweb           scalar                  profit per reported web reader
%   piprint         scalar                  profit per reported print reader
%   issuesperyear   1 x GG                  number of issues of each good per year
%   postpos         scalar                  position of Post in lhslist1
%   postcompos      scalar                  position of post.com in lhslist1
%   alpha           scalar                  price coefficient
%   msee            scalar                  number of simulation draws
%   mweights        msee*NNtot              unobservable weights
%   gam             scalar                  gamma parameter (pr of good
%                                           news day)
%   Uadd            CC x 1                  utility to add on good news day
%	
% OUTPUTS:
%   negprofit       CC                      (negative) Post company profit per day
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [negprofit] = profitwebprice(price,tcost,U,CM,surveyweights,piweb,piprint,issuesperyear,postpos,postcompos,alpha,msee,mweights,gam,Uadd)

global CC GG NNtot
    
U = [zeros(NNtot*msee,1) U];
U = U + repmat(CM(:,postcompos)',NNtot*msee,1).*repmat(alpha*(price+tcost),NNtot*msee,CC);
Unews = U+repmat(Uadd',msee*NNtot,1);
P = exp(U)./repmat(sum(exp(U),2),1,CC);
P = P.*repmat(mweights,1,CC).*repmat(surveyweights,msee,CC);
Pnews = exp(Unews)./repmat(sum(exp(Unews),2),1,CC);
Pnews = Pnews.*repmat(mweights,1,CC).*repmat(surveyweights,msee,CC);
P = (1-gam)*P+gam*Pnews;
nchbundle = sum(P)/msee;
nchgood = zeros(GG,1);
for ii = 1:CC
	for jj = 1:GG
		if(CM(ii,jj)==1), nchgood(jj) = nchgood(jj)+nchbundle(ii); end
	end
end
piweb = piweb+price;
profit = nchgood(postpos)*piprint*issuesperyear(postpos) + nchgood(postcompos)*piweb*issuesperyear(postcompos);

negprofit = -profit;

