import subprocess
import os
import shutil
import pandas as pd
import sys

sys.path.append('../../../lib')
import load_yaml
constants = load_yaml.load_yaml('../../../../config_global.yaml')

def main(constants):
    '''
    Compute numerical gradient of c by age. We call c the
    difference between observed and simulated counterfactual attendance probabilities.
    '''
    step_size = 6.5535e-7
    input_lines, param_indices = load_inputs()
    call_compile_and_run(constants)
    # Change to output directory.
    orig_dir = os.getcwd()
    build_dir = '../../../../%s' % \
        constants['build']['analysis']['ams_2011']['counterfactual']
    os.chdir(build_dir)
    # Compute derivative
    numerical_gradient = compute_numerical_gradient(
        step_size, input_lines, param_indices)
    write_numerical_gradient(numerical_gradient, constants)
    os.chdir(orig_dir)
    return None

def load_inputs():
    '''
    Load
    * starting values from file-input to simulation
    * indices of file-input we want to include in our gradient
    Second line skips cutoffs o-probit estimation of 17-years-olds stock of schooling.
    '''
    # This file is in source folder.
    with open('PARAM_OA_PNP_eval.out', 'rU') as f:
        input_lines = [line.strip() for line in f]
    param_indices = range(35, 91) + range(102, 148)
    return input_lines, param_indices

def call_compile_and_run(constants):
    '''
    Compile and run simulation program.
    '''
    try:
        subprocess.check_output(['sh', 'compile_and_run.sh'])
    except subprocess.CalledProcessError as e:
        print e.output
        raise e
    return None

def compute_numerical_gradient(step_size, input_lines, param_indices):
    '''
    Compute numerical derivative of change in enrollment probability by age
    under alternative grant (w/o GE effects)
    at each input parameter using perturbation of step_size.
    '''
    store = []
    for store_index, param_index in enumerate(param_indices):
        c_derivative, baseline_derivative = compute_derivative(step_size, input_lines, param_index)
        storeline = [store_index, c_derivative, baseline_derivative]
        store.append(storeline)
    return store

def compute_derivative(step_size, input_lines, param_index):
    '''
    Compute numerical derivative at one input parameter using step_size.
    Derivative computed using two-sided difference.
    '''
    perturbed_c = {1: None, -1: None}
    perturbed_baseline = {1: None, -1: None}

    for direction in perturbed_c.keys():
        simulate_counterfactual(step_size, input_lines, param_index, direction)
        (c, baseline) = compute_c()
        perturbed_c[direction] = c
        perturbed_baseline[direction] = baseline

    c_derivative = (perturbed_c[1] - perturbed_c[-1]) / (2 * step_size)
    baseline_derivative = (perturbed_baseline[1] - perturbed_baseline[-1]) / (2 * step_size)

    return c_derivative, baseline_derivative

def simulate_counterfactual(step_size, input_lines, param_index, direction):
    '''
    Identically rewrite file-input used in simulation line-by-line, except we
    replace the line at param_index with
    its paramater value perturbed in direction by step_size.
    '''
    newlines = []
    for line_index, oldline in enumerate(input_lines):
        if line_index == param_index:
            perturbed_param_value = float(oldline) + step_size * direction
            newline = '%.32f' % perturbed_param_value
        else:
            newline = oldline
        newlines.append(newline)
    newlines = '\n'.join(newlines)
    # These files are in the build folder.
    with open('PARAM_OA_PNP_eval.out', 'wb') as f:
        f.write(newlines)
    os.remove('SIMUL_OA_PNP.out')
    subprocess.check_call(['./main_PNP'])
    return None

def compute_c():
    '''
    Read simulated data and
    compute difference in simulated school enrollment probability by age.
    Return a pandas Series of the differences by age.
    Names and definitions taken from graphs.do in replication materials.
    '''
    cols = ['age', 'benef', 'fben', 'grade', 'enrol', 'progresa',
            'wage', 'wage_ge', 'xconst2', 'xconst3', 'xconst4', 'xconst5',
            'xconst6', 'xconst7', 'xsec', 'prob1', 'prob2', 'prob3',
            'prob4', 'prob5', 'prob6', 'prob7', 'prob8', 'prob9',
            'prob10', 'prob11', 'prob12', 'prob13', 'prob14']
    df = pd.read_csv('SIMUL_OA_PNP.out', delim_whitespace = True,
                     header = None, names = cols)
    # Baseline grant effect, fixed wages (fig.1, left) = prob1 - prob2
    # Counterfactual grant effect, fixed wages (fig 2) = prob3 - prob2
    # Interested counterfactual (fig 2 -fig 1)         = prob3 - prob1
    df = df.loc[df.benef == 1, ['age', 'prob3', 'prob1', 'prob2']]
    df.loc[:, 'c'] = df.prob3 - df.prob1
    df.loc[:, 'baseline'] = df.prob1 - df.prob2
    gb = df.groupby('age')
    c_age = gb.mean()['c']
    baseline_age = gb.mean()['baseline']
    c = float(c_age.sum())
    baseline = float(baseline_age.sum())
    return c, baseline

def write_numerical_gradient(numerical_gradient, constants):
    '''
    Write numerical gradient to disk in |-delimited file, and copy to release.
    Parameter index and step size are keys.
    Derivative of difference in counterfactual enrollment probability at each age is value.
    '''
    gradient_outfile = 'gradient.txt'
    cols = ['parameter_index', 'derivative_c', 'derivative_baseline']
    lines = [cols] + numerical_gradient
    with open(gradient_outfile, 'wb') as f:
        for line in lines:
            stringline = [str(el) for el in line]
            outline = '|'.join(stringline)
            f.write(outline + '\n')
    release_path = '../../../../%s' \
        % constants['release']['analysis']['ams_2011']['counterfactual']
    shutil.copy(gradient_outfile, release_path)
    return None

# Execute
main(constants)
