program define main
     /***
    - Replicate Table 2 of Attanasio Meghir Santiago 2011 REStud. 
    - Produce "GMM"-moments of the regressions in Table 2
    - Produce the 2nd moment statistic of the treatment variable `d` for each age group
    ***/
    housekeeping
    replicate_tbl_2
    omnibus_regression
    omnibus_grades
    write_to_file
end


program define housekeeping
    yaml read path using config_global.yaml
    yaml global build = path.build.analysis.ams_2011.experiment
    import delimited "$build/prog_n.csv", delimiter("") encoding(ISO-8859-1) clear

    * Limit to evaluation sample
    keep if age >= 10 & age <= 16

    * Generate useful globals
    levelsof age
    global ages = "`r(levels)'"
    global n : word count $ages
    levelsof yrs
    global grades = "`r(levels)'"
    global g : word count $grades
end


program define replicate_tbl_2
    * Pure replication of Table 2 in AMS
    matrix tbl2 = J($n, $n, .) 
    matrix rownames tbl2 = $ages
    matrix colnames tbl2 = enrol_control_elig poor97_b poor97_se poor9798_b poor9798_se nonelig_b nonelig_se
    gen nonelig = (b | (~c&~a))

    forvalues i = 1/$n {   
        local age : word `i' of $ages  
        * Tbl2 enrolment rates in control villages (eligible)   
        sum school if (poorc & ~d) & age == `age'   
        matrix tbl2[`i', 1] = `r(mean)' 
        * Tbl2 impact on Poor 97    
        reg school d if poorc & (age == `age'), vce (cluster town)  
        matrix beta = r(table)  
        matrix tbl2[`i', 2] = beta[1, 1]    
        matrix tbl2[`i', 3] = beta[2, 1]    
        local b = beta[1,1] 
        * Tbl2 impact on Poor 97-98     
        reg school d if (a|c) & (age == `age'), vce(cluster town)   
        matrix beta = r(table)  
        matrix tbl2[`i', 4] = beta[1, 1]    
        matrix tbl2[`i', 5] = beta[2, 1]    
        * Tbl2 impact on non-eligible   
        reg school b if nonelig & (age == `age'), vce(cluster town) 
        matrix beta = r(table)  
        matrix tbl2[`i', 6] = beta[1, 1]    
        matrix tbl2[`i', 7] = beta[2, 1]    
    }   
end

program define omnibus_regression
    * Replicate Tbl2 impact on Poor 97 and impact on non-elig using the "Omnibus regression"
    foreach x in "jacobian_elig" "jacobian_non" {
        matrix `x' = J($n, 1, .)  // define jacobians for each of elig and non
        matrix rownames `x' = $ages
    }
    * define one for both
    matrix jacobian_both = J($n*2, 1, .) 
    matrix rownames jacobian_both = $ages $ages

    * generate interaction terms 
    foreach age of global ages {
        gen d_`age' = d * (age == `age') * poorc 
        gen b_`age' = b * (age == `age') * nonelig
    }    

    * run regressions and keep residuals for moment calculations
    regress school d_* i.age age#poorc poorc, vce(robust) // d_* coef the same as tbl2 poor97
    predict temp_residuals_d, residuals 
    regress school b_* i.age age#nonelig nonelig, vce(robust) // b_* coef the same as tbl2 nonelig
    predict temp_residuals_b, residuals 
    regress school d_* b_* i.age age#poorc age#nonelig poorc nonelig, vce(robust) // combine d_* and b_* impacts
    predict temp_residuals_both, residuals 

    * generate m(D_i, gamma) and Jacobian M
    forvalues i = 1/$n {
        local age: word `i' of $ages 
        * moments
        gen m_di_gamma_elig_`age' = d_`age' * temp_residuals_d
        gen m_di_gamma_non_`age' = b_`age' * temp_residuals_b
        gen m_di_gamma_both_elig_`age' = d_`age' * temp_residuals_both
        gen m_di_gamma_both_non_`age' = b_`age' * temp_residuals_both
        * Jacobians
        sum d_`age' 
        mat jacobian_elig[`i',1] = -1 * (`r(Var)'  + `r(mean)' ^ 2)
        sum b_`age' 
        mat jacobian_non[`i',1] = -1 * (`r(Var)'  + `r(mean)' ^ 2)
        * combine the elig and non Jacobians for the both Jacobians
        mat jacobian_both[`i',1] = jacobian_elig[`i', 1]
        mat jacobian_both[`i'+7,1] = jacobian_non[`i', 1]
    }

    drop temp_residuals*
end


program define omnibus_grades
    * Regression that interacts the omnibus regression terms with grades
    * order of iteration is always D/B ->  ages -> grades

    * useful locals to iterate over. 
    local treatments   "d     b"
    local restrictions "poorc nonelig"
    local rest_type    "elig  non"

    * generate interaction terms
    forvalues i = 1/2 {
        local x: word `i' of `treatments'
        local r: word `i' of `restrictions'
        local keptcount_`x' = 0
        foreach age of global ages {
            foreach grade of global grades {
                local var = "g_`x'_`age'_`grade'"
                gen `var' = `x' * (age == `age') * (yrs == `grade') * `r'
                sum `var' // drop interactions that don't vary
                if r(Var) == 0 { // it's so that we don't have singular Jacobians!
                    drop `var' // isn't math fun
                }
                else {
                    local keptcount_`x' = `keptcount_`x'' + 1
                }
            }
        }
    }
    
    * define Jacobians, dimension is based on how many interaction terms we kept
    matrix jacobian_gelig  = J(`keptcount_d', 1, .)
    matrix jacobian_gnon   = J(`keptcount_b', 1, .)
    matrix jacobian_gboth  = J(`keptcount_d' + `keptcount_b', 1, .) 

    * Run the regressions with triple interactions, each is analogous to tbl2. 
    regress school g_d_* i.age i.yrs age#yrs age#poorc yrs#poorc poorc, vce(robust) 
    predict temp_residuals_gelig, residuals
    regress school g_b_* i.age i.yrs age#yrs age#nonelig yrs#nonelig nonelig, vce(robust)
    predict temp_residuals_gnon, residuals
    regress school g_d_* g_b_* i.age i.yrs age#yrs age#poorc age#nonelig yrs#poorc yrs#nonelig poorc nonelig, vce(robust)
    predict temp_residuals_gboth, residuals

    * get m(D_i, gamma) and Jacobian M for d i.e. poor97 first, b i.e. nonelig second, and gboth while looping through
    forvalues i = 1/2 { 
        local x: word `i' of `treatments'
        local t: word `i' of `rest_type'
        local rownames_`x' = ""
        local row = 1
        foreach age of global ages {
            foreach grade of global grades {
                capture confirm variable g_`x'_`age'_`grade' // only use compute moment and Jacobian if variable is there 
                if !_rc { // if it doesn't throw an error
                    gen m_di_gamma_g`t'_`age'_`grade' =  g_`x'_`age'_`grade' * temp_residuals_g`t'
                    gen m_di_gamma_gboth_`t'_`age'_`grade' = g_`x'_`age'_`grade' * temp_residuals_gboth
                    * fill in Jacobian matrices
                    sum g_`x'_`age'_`grade'
                    mat jacobian_g`t'[`row', 1] = -1 * (`r(Var)'  + `r(mean)' ^ 2)
                    * indexing for gboth should skip the first `keptcount_d` row if (`i' == 2)
                    mat jacobian_gboth[`row' + (`keptcount_d' * (`i'-1)),1] = -1 * (`r(Var)'  + `r(mean)' ^ 2) 
                    * save pretty row names for later
                    local rownames_`x' = "`rownames_`x'' `age'_`grade'"
                    local row = `row' + 1
                }
            }
        }
    }

    * give pretty rownames to the Jacobians
    mat rownames jacobian_gelig = `rownames_d'
    mat rownames jacobian_gnon  = `rownames_b'
    mat rownames jacobian_gboth = `rownames_d' `rownames_b'

    drop temp_residuals*
end


program define write_to_file
    * Write replication table to file
    local fmt "%8.3f %8.4f %8.3f %8.4f %8.3f %8.4f %8.3f"
    matrix_to_txt, matrix(tbl2) saving("$build/table_2_rep.csv") format("`fmt'") ///
        userownames usecolnames replace 
    * Write m_di_gamma to file 
    order famid orderofbirth m_di_gamma_elig_* m_di_gamma_non_* m_di_gamma_both_elig_* ///
        m_di_gamma_both_non_* m_di_gamma_gelig* m_di_gamma_gnon* m_di_gamma_gboth*

    local suffixes = "elig non both gelig gnon gboth"
    foreach x of local suffixes {
        * write m(D_i,gamma) to file
        preserve
        keep famid orderofbirth m_di_gamma_`x'*
        save_data "$build/m_di_gamma_`x'.csv", replace outsheet key(famid orderofbirth)
        restore
        * write jacobian to file
        matrix_to_txt, matrix(jacobian_`x') saving("$build/jacobian_`x'.csv") format(%8.4f) ///
            userownames replace
    }

end

main 
