import pandas as pd
import numpy as np
import difflib
import sys
import os
sys.path.append('source/lib/')
from load_yaml import load_yaml

# globals
paths = load_yaml('config_global.yaml')
est_path = paths['build']['analysis']['ams_2011']['score_hessian']
exp_path = paths['build']['analysis']['ams_2011']['experiment']
cnt_path = paths['build']['analysis']['ams_2011']['counterfactual']
info_path = paths['build']['analysis']['ams_2011']['informativeness']
cnt_release_path = paths['release']['analysis']['ams_2011']['counterfactual']
key_cols = ['famid', 'orderofbirth']  # these are keys for observations from estimation


def main():
    # prepare phi that is used for computing sens/infos in all modes
    phi_theta = get_phi_theta()
    phi_c = get_phi_c(phi_theta,  c_col='c')
    phi_b = get_phi_c(phi_theta,  c_col='baseline')
    phi_np = get_phi_c(phi_theta, c_col='c', non_param = True)
    phi_cx = get_phi_c(phi_theta, c_col='c', x_impact = True) # account for covariates impact

    # compute sens/infos
    output_dict = {}
    output_dictx = {}
    output_dictnp = {}
    for mode in ['elig', 'non', 'both', 'gelig', 'gnon', 'gboth']:
        output_dict[mode]   = produce_sens_infos(phi_theta, phi_c,  phi_b, mode=mode, prefix = 'c')
        output_dictx[mode]  = produce_sens_infos(phi_theta, phi_cx, phi_b, mode=mode, prefix = 'b')
        output_dictnp[mode] = produce_sens_infos(phi_theta, phi_np, phi_b, mode=mode, prefix = 'np')

    # keep all the standard errors in one place
    compute_se(phi_theta, phi_c, phi_b)

    # create table for Stata plotting
    with open('%s/info_plotting.csv' % info_path, 'w') as file:
        file.writelines('%s_%s %s\n' % ('info_%s' % y, x, output_dict[x]['info_%s' % y][0, 0])
                        for x in ['elig', 'non', 'both', 'gboth']
                        for y in ['b', 'c'])

    # create tables for paper and text.pdf
    cntf_table = np.array([
        [output_dict['gboth']['info_c'][0, 0]],
        [output_dict['gelig']['info_c'][0, 0]],
        [output_dict['gnon']['info_c'][0, 0]]
    ])
    write_matrix(cntf_table, 'ams_cntf_info', fmt='%.4f')
    cntfx_table = np.array([
        [output_dictx['gboth']['info_c'][0, 0]],
        [output_dictx['gelig']['info_c'][0, 0]],
        [output_dictx['gnon']['info_c'][0, 0]]
    ])
    write_matrix(cntfx_table, 'ams_cntfx_info', fmt='%.4f')
    base_table = np.array([
        [output_dict['gboth']['info_b'][0, 0]],
        [output_dict['gelig']['info_b'][0, 0]],
        [output_dict['gnon']['info_b'][0, 0]]
    ])
    write_matrix(base_table, 'ams_base_info', fmt='%.4f')
    np_table = np.array([
        [output_dictnp['gboth']['info_c'][0, 0]],
        [output_dictnp['gelig']['info_c'][0, 0]],
        [output_dictnp['gnon']['info_c'][0, 0]]
    ])
    write_matrix(np_table, 'ams_np_info', fmt='%.4f')

    most_info_exp = pd.concat([
        get_most_info_exp(phi_c, covar_cells=['age']),
        get_most_info_exp(phi_c, covar_cells=['yrs']),
        get_most_info_exp(phi_c, covar_cells=['poorc']),
        get_most_info_exp(phi_c, covar_cells=['age', 'yrs']),
        get_most_info_exp(phi_c, covar_cells=['age', 'poorc']),
        get_most_info_exp(phi_c, covar_cells=['yrs', 'poorc']),
        get_most_info_exp(phi_c, covar_cells=['age', 'yrs', 'poorc'])
    ])
    write_matrix(most_info_exp, 'most_info_exp', fmt='%.4f')

    return None


def get_phi_theta():

    # read in hessian
    hess = pd.read_csv('%s/hess_df.csv' % est_path,
                       delimiter='\t', header=None)
    np_inv_hess = np.linalg.inv(hess.values)

    # read in score
    score = pd.read_csv('%s/score_df.csv' % est_path,
                        delimiter='\t')
    score.columns = key_cols + [x.replace(" ", "")  # key_cols names are slightly different from what we have
                                for x in score.columns if x not in ['family_id', 'birth_order']]
    score[key_cols] = score[key_cols].astype(int)
    score = score.set_index(key_cols)

    phi_theta = {}  # output dictionary, indexed by key_cols tuple
    for i, row in score.iterrows():
        np_score_transpose = np.matrix(row).transpose()
        phi_theta[i] = -1 * np.matmul(np_inv_hess, np_score_transpose)

    return phi_theta


def get_phi_c(phi_theta, c_col, x_impact = False, non_param = False):
    if non_param:
        # Use the age based gammas and read in the jaconian
        mode        = 'gelig'
        jacobian    = read_jacobian('jacobian_%s' % mode)
        # Get a vector of the number of grades for each age
        age_grade_np   =  np.asarray(jacobian.index)
        age_grade_pd   =  pd.DataFrame(np.zeros((len(age_grade_np), 2)))
        for i in range(0,len(age_grade_np)):
            temp = np.char.split(age_grade_np[i], '_')
            temp = temp.tolist()
            age_grade_pd.iloc[i:i+1, 0:1] = int(temp[0])
            age_grade_pd.iloc[i:i+1, 1:2] = int(temp[1])
        age_grade_pd = age_grade_pd.rename(columns={0:'age', 1:'grade'})
        age_grade_pd = age_grade_pd.groupby(['age']).count()
        age_grade    = np.repeat(age_grade_pd['grade'], age_grade_pd['grade'])
        age_grade    = 1 / age_grade
        # Compute phi_c
        np_jacobian = np.diag(jacobian['M'])
        m_di_gamma = pd.read_csv('%s/m_di_gamma_%s.csv' % (exp_path, mode),
                             delimiter='\t', index_col=['famid', 'orderofbirth'])
        phi_c = {}  # output dictionary, indexed by key_cols tuple
        for i, row in m_di_gamma.iterrows():
            # create column vector of m_di_gamma for each obs
            np_m_di_gamma = np.matrix(row).transpose()
            # compute the moment condition for each obs and store it in the dictionary
            moment   = -1 * np.matmul(np.linalg.inv(np_jacobian), np_m_di_gamma)
            phi_c[i] = np.matrix(age_grade.dot(moment))
    else: 
        # read in gradient of counterfactual w.r.t theta
        grad_transpose = pd.read_csv('%s/gradient.txt' % cnt_release_path,
                                     delimiter='|', index_col='parameter_index')
        grad_transpose = grad_transpose[['derivative_%s' % c_col]]
        np_grad = np.matrix(grad_transpose).transpose()
        cf = get_ind_cf()
        phi_c = {}  # output dictionary, indexed by key_cols tuple
        for i, phi_theta_i in phi_theta.items():
            cf_ind = cf.loc[(cf['famid'] == str(i[0])) & (cf['orderofbirth'] == i[1])]
            if x_impact and (not cf_ind.empty):
                num_age = cf_ind['num_age'].values[0]
                c_ind =  cf_ind[c_col].values[0]
                phi_c[i] = n_obs / num_age * c_ind + np.matmul(np_grad, phi_theta_i)
            else:
                phi_c[i] = np.matmul(np_grad, phi_theta_i)
    return phi_c

def get_ind_cf():
    '''
    Load simulated counterfactual values for each observation
    '''
    cols = ['famid', 'orderofbirth', 'age', 'benef', 'fben', 'grade', 'enrol', 'progresa',
            'wage', 'wage_ge', 'xconst2', 'xconst3', 'xconst4', 'xconst5',
            'xconst6', 'xconst7', 'xsec', 'prob1', 'prob2', 'prob3',
            'prob4', 'prob5', 'prob6', 'prob7', 'prob8', 'prob9',
            'prob10', 'prob11', 'prob12', 'prob13', 'prob14']

    # load "SIMUL_OA_PNP_ID.out" file, which stores simulated probability for each individual 
    # identified by key (family id, birth order)
    cf = pd.read_csv('%s/SIMUL_OA_PNP_ID.out' % cnt_path, delim_whitespace = True,
                     header = None, names = cols) 

    # calculate two counterfactuals: 'c' and 'baseline'
    cf = cf.loc[cf.benef == 1, key_cols + ['age', 'prob3', 'prob1', 'prob2']]
    cf.loc[:, 'c'] = cf.prob3 - cf.prob1
    cf.loc[:, 'baseline'] = cf.prob1 - cf.prob2
    cf['num_age'] = cf.groupby('age')['age'].transform('count')
    return cf

def produce_sens_infos(phi_theta, phi_c, phi_b, mode, prefix):
    phi_gamma = get_phi_gamma(mode=mode)

    # intermediate objects for computing sens/info
    (sigma_gammagamma, sigma_cc, sigma_cgamma, sigma_thetatheta,
        sigma_thetagamma, sigma_bb, sigma_bgamma) = take_sample_averages(phi_gamma, phi_theta, phi_c, phi_b, mode=mode)

    # store output in a dictionary
    sens_info_dict = {
        'sens_c': get_sens(sigma_cgamma, sigma_gammagamma, mode=mode),
        'sens_b': get_sens(sigma_bgamma, sigma_gammagamma, mode=mode),
        'sens_theta': get_sens(sigma_thetagamma, sigma_gammagamma, mode=mode)
    }
    sens_info_dict['info_c'] = get_info(sens_info_dict['sens_c'], sigma_gammagamma, sigma_cc)
    sens_info_dict['info_b'] = get_info(sens_info_dict['sens_b'], sigma_gammagamma, sigma_bb)
    sens_info_dict['info_theta'] = get_info(
        sens_info_dict['sens_theta'], sigma_gammagamma, sigma_thetatheta)

    # write to file
    for (k, v) in sens_info_dict.items():
        write_matrix(v, '%s_%s_%s' % (prefix, k, mode), fmt='%.4f')

    return sens_info_dict


def get_phi_gamma(mode):
    '''
    Return the influence function of each observation on the experimental
    parameters gamma.
    '''
    n_gamma = ngamma_dict[mode]

    # Read in objects (M and m(D_i, gamma)) from the experimental results replication
    jacobian = read_jacobian('jacobian_%s' % mode)
    np_jacobian = np.diag(jacobian['M'])
    m_di_gamma_path = '%s/m_di_gamma_%s.csv' % (exp_path, mode)
    m_di_gamma = pd.read_csv(m_di_gamma_path,
                             delimiter='\t', index_col=['famid', 'orderofbirth'])

    phi_gamma = {}  # output dictionary, indexed by key_cols tuple
    for i, row in m_di_gamma.iterrows():
        # create column vector of m_di_gamma for each obs
        np_m_di_gamma = np.matrix(row).transpose()
        # compute the moment condition for each obs and store it in the dictionary
        phi_gamma[i] = -1 * np.matmul(np.linalg.inv(np_jacobian), np_m_di_gamma)
    return phi_gamma


def take_sample_averages(phi_gamma, phi_theta, phi_c, phi_b, mode):
    # produce LHS objects by taking averages over multiplied-vectors stored in dictionaries
    n_gamma = ngamma_dict[mode]
    sigma_gammagamma = mult_dicts_of_vectors(phi_gamma)
    sigma_cc = mult_dicts_of_vectors(phi_c)
    sigma_cgamma = mult_dicts_of_vectors(phi_c, phi_gamma)
    sigma_thetatheta = mult_dicts_of_vectors(phi_theta)
    sigma_thetagamma = mult_dicts_of_vectors(phi_theta, phi_gamma)
    sigma_bb = mult_dicts_of_vectors(phi_b)
    sigma_bgamma = mult_dicts_of_vectors(phi_b, phi_gamma)
    output = (sigma_gammagamma, sigma_cc, sigma_cgamma,
              sigma_thetatheta, sigma_thetagamma, sigma_bb, sigma_bgamma)
    return output


def mult_dicts_of_vectors(A, B=None):
    if B is None:
        B = A
    collect = {}
    for i in A.keys():
        collect[i] = np.matmul(A[i], B[i].transpose())
    sigma = np.mean(collect.values(), axis=0)
    return sigma


def get_sens(sigma_cgamma, sigma_gammagamma, mode):
    n_gamma = ngamma_dict[mode]
    sens = np.matmul(sigma_cgamma, np.linalg.inv(sigma_gammagamma))
    return sens


def get_info(sens, sigma_top, sigma_bottom):
    info_len = len(sens)
    info = np.matrix(np.zeros(info_len)).transpose()
    for j in range(info_len):
        e_j = np.matrix(np.zeros(info_len)).transpose()
        e_j[j, 0] = 1
        numerator = np.linalg.multi_dot([e_j.transpose(), sens, sigma_top, sens.transpose(), e_j])
        denominator = np.linalg.multi_dot([e_j.transpose(), sigma_bottom, e_j])
        info[j, 0] = numerator / denominator
    return info


def get_most_info_exp(phi_c, covar_cells):
    '''
    Compute most informative experiment-implied moment for phi_c.
    Covariates are determined by covar_cells (a list).
    Cells must contain two or more units.
    '''
    all_vars = covar_cells + ['d']
    all_vars = list(set(all_vars))
    # Convert influence function to data frame
    phi = [[k[0], k[1], float(v)] for k, v in phi_c.items()]
    phi = pd.DataFrame(phi)
    phi.columns = ['famid', 'orderofbirth', 'phi_c']
    # Load covars
    inpath = '%s/prog_n.csv' % exp_path
    full_data = pd.read_csv(inpath)
    full_data = full_data.merge(phi, how='inner')
    full_data = full_data.loc[:, ['phi_c'] + all_vars]
    nobs = float(full_data.shape[0])
    # Assign integer ids to covariate cells to make our lives easier.
    X = full_data.loc[:, covar_cells]
    X = X.groupby(covar_cells)
    if X.size().min() < 2:
        msg = 'There must be at least two people in each covariate cell'
        raise TypeError(msg)
    X = X.size().reset_index()
    X['x'] = range(X.shape[0])
    full_data = full_data.merge(X, how='inner')
    # Assignment variable
    t = full_data.d.sum() / nobs
    full_data['T'] = (full_data.d / t) - ((1 - full_data.d) / (1 - t))
    # Multiply influence function and assignment variable
    full_data['phi_c_t'] = full_data['phi_c'] * full_data['T']
    # Compute the most informative moment
    grouped = full_data.groupby('x')
    lo_estimates = []
    for name, g in grouped:
        lo_estimates.extend(estimate_lo_in_group(g))
    sigma_c = np.dot(full_data.phi_c, full_data.phi_c) / nobs
    sigma_t = (1 / t) + (1 / (1 - t))
    informativeness = (sum(lo_estimates) / nobs) / (sigma_t * sigma_c)
    out = pd.DataFrame([{
        'covars': ', '.join(covar_cells),
        'informativeness': informativeness
    }])
    return out


def estimate_lo_in_group(group):
    '''
    Compute leave-out estimates of square of conditional expectation.
    '''
    lo_nobs = group.shape[0]
    phi_c_t = np.array(group['phi_c_t'])
    square_phi_c_t = np.repeat(phi_c_t, lo_nobs).reshape(lo_nobs, lo_nobs)
    np.fill_diagonal(square_phi_c_t, 0)
    lo_estimates = np.matmul(square_phi_c_t, phi_c_t) / (lo_nobs - 1)
    return list(lo_estimates)


def write_matrix(matrix, filename, fmt=None):
    filepath = '%s/%s.csv' % (info_path, filename)
    pd.DataFrame(matrix).to_csv(filepath,
                                header=False, index=False, float_format=fmt, sep='\t')
    # prepend the Tablefill label
    label = '<tab:%s>' % filename
    with open(filepath, 'r+') as f:
        file_data = f.read()
        f.seek(0, 0)
        f.write(label.rstrip('\r\n') + '\n' + file_data)
    return None


def compute_se(phi_theta, phi_c, phi_b):

    # go through the motions to obtain sigmas
    phi_gamma = get_phi_gamma(mode='elig')
    (sigma_gammagamma, sigma_cc, _, sigma_thetatheta,
        _, sigma_bb, _) = take_sample_averages(phi_gamma, phi_theta, phi_c, phi_b, mode='elig')

    # standard errors
    se_dict = {
        'se_theta': get_se_theta(),
        'se_theta_from_sigma': get_se_from_sigma(sigma=sigma_thetatheta),
        'se_gamma_elig': get_se_from_sigma(sigma=sigma_gammagamma),
        'se_c': get_se_from_sigma(sigma=sigma_cc),
        'se_b': get_se_from_sigma(sigma=sigma_bb),
    }
    for k, v in se_dict.items():
        write_matrix(v, k, fmt='%.5f')
    return None


def get_se_theta():
    # read in hessian
    hess = pd.read_csv('%s/hess_df.csv' % est_path,
                       delimiter='\t', header=None)
    np_inv_hess = np.linalg.inv(hess.values)
    return(np.diag((np_inv_hess / n_obs) ** 0.5))


def get_se_from_sigma(sigma):
    return np.diag((sigma / n_obs) ** 0.5)


def get_dimensions():
    '''
    Return the number of experimental parameters, theta parameters,
    and observations.
    '''
    ngamma_dict = {
        'elig': len(read_jacobian('jacobian_elig')),
        'non': len(read_jacobian('jacobian_non')),
        'both': len(read_jacobian('jacobian_both')),
        'gelig': len(read_jacobian('jacobian_gelig')),
        'gnon': len(read_jacobian('jacobian_gnon')),
        'gboth': len(read_jacobian('jacobian_gboth'))
    }

    hess = pd.read_csv('%s/hess_df.csv' % est_path,
                       delimiter='\t', header=None)
    n_param = len(hess)
    m_di_gamma = pd.read_csv('%s/m_di_gamma_elig.csv' % exp_path,
                             delimiter='\t', index_col=['famid', 'orderofbirth'])
    n_obs = len(m_di_gamma)

    return (ngamma_dict, n_param, n_obs)


def read_jacobian(name):
    jacobian = pd.read_csv('%s/%s.csv' % (exp_path, name),
                           delimiter='\t',
                           names=['age', 'M'], index_col=['age'])
    return jacobian


(ngamma_dict, n_param, n_obs) = get_dimensions()
main()
