function compute_Delta_Lambda
    %% Read paths and format
    format long
    addpath(genpath('build/analysis/gentzkow_2007/replication/n_full'))
    analysis_path = 'build/analysis/gentzkow_2007';
    data_path     = 'build/external/gentzkow_2007';

    %% Read in data and assign sample
    data = dataset('File', sprintf('%s/data.csv', analysis_path), ...
                   'Delimiter', ',');
    
%{ 
    Get indices. 
       - data.omitted is an indicator for the observations in the full data
         that are dropped for the estimation of gamma. 
       - data.ascribed is an indicator for the observations in the full
         data that are imputed by scarbourough.
       - data.nnonews is an indicator for the observation is the full data
         for which there are no print media in the choice set
    score_index gives an indicator for the observations that are have
    print media in the choice set, subsequent to the ascribed observation
    being dropped. 
%}             
    sample       = logical(1 - data.omitted);
    score_subset =  data(logical(1 - data.ascribed),:);
    score_index  =  logical(1 - score_subset.nnonews);
    
    % Assign names for variables
    kept_controls = {'kept_ndtimes7', 'kept_page_m',   'kept_psex',     'kept_phs', ...
                     'kept_pcol',     'kept_pgrad',    'kept_pjcomp',   'kept_pjwhitecol',... 
                     'kept_pempft',   'kept_zmis2',    'kept_zmis3',    'kept_zmis4', ...
                     'kept_zmis5',    'kept_zmis6',    'kept_zmis7',    'kept_zmis8', ...
                     'kept_hloginc_m','kept_vdem',     'kept_vrep',     'kept_date_2', ...
                     'kept_date_3',   'kept_date_4',   'kept_date_5',   'kept_date_6', ...
                     'kept_hzone_2',  'kept_hzone_3',  'kept_hzone_4',  'kept_hzone_5', ...
                     'kept_hzone_6',  'kept_wnouse30', 'kept_cons'};
    
    omit_controls = {'omit_ndtimes7', 'omit_page_m',  'omit_psex',     'omit_phs', ...
                    'omit_pcol',     'omit_pgrad',    'omit_pjcomp',   'omit_pjwhitecol',... 
                    'omit_pempft',   'omit_zmis2',    'omit_zmis3',    'omit_zmis4', ...
                    'omit_zmis5',    'omit_zmis6',    'omit_zmis7',    'omit_zmis8', ...
                    'omit_hloginc_m','omit_vdem',     'omit_vrep',     'omit_date_2', ...
                    'omit_date_3',   'omit_date_4',   'omit_date_5',   'omit_date_6', ...
                    'omit_hzone_2',  'omit_hzone_3',  'omit_hzone_4',  'omit_hzone_5', ...
                    'omit_hzone_6',  'omit_wnouse30', 'omit_cons'};

    
    %% Get Estimation Components for counterfactual
    model = get_model_components(score_index, analysis_path);
    load(sprintf('%s/counterfactual', analysis_path), ...
        'cf_jacobian', 'Cdep');
    
    %% Get Estimation Components for IV regression
    iv_5day.Y_var = 'ndpost7';
    iv_5day.X_vars = [{'kept_nwpost7'}, kept_controls, ... 
                      {'omit_nwpost7'}, omit_controls];
    iv_5day.Z_vars = [kept_controls, ...
                      {'omit_nwpost7'}, omit_controls,...
                      {'kept_wlwork', 'kept_wued', ...
                       'kept_wuwork', 'kept_wcfast'}];
    iv = get_moment_components(iv_5day, data, 'iv_5day', '2sls', analysis_path);
    
    %% Get Estimation Components for panel regression
    ols_panel.Y_var = 'ndpost1';
    ols_panel.X_vars = [{'kept_nwpost1', 'kept_npostboth7', 'kept_ndpostonly7', 'kept_nwpostonly7'}, kept_controls, ...
                        {'omit_nwpost1', 'omit_npostboth7', 'omit_ndpostonly7', 'omit_nwpostonly7'}, omit_controls];
    panel = get_moment_components(ols_panel, data, 'ols_panel', 'eye', analysis_path);                                         
                               
    %% Compute Delta and Lambda for the IV, Panel, and Combined
    [Delta_iv,   Lambda_iv]     = compute_D_L_individual(iv, model, cf_jacobian, Cdep,...
                                                         sample, false);
    [Delta_panel,Lambda_panel]  = compute_D_L_individual(panel, model, cf_jacobian, Cdep,...
                                                         sample, false);
    [Delta_comb, Lambda_comb]   = compute_D_L_combined(iv, panel, model, cf_jacobian, Cdep,... 
                                                       sample, false);
                                            
                                                                                        
    [Delta_iv_d,   Lambda_iv_d]     = compute_D_L_individual(iv, model, cf_jacobian, Cdep,...
                                                             sample, true);
    [Delta_panel_d,Lambda_panel_d]  = compute_D_L_individual(panel, model, cf_jacobian, Cdep,...
                                                             sample, true);
    [Delta_comb_d, Lambda_comb_d]   = compute_D_L_combined(iv, panel, model, cf_jacobian, Cdep,... 
                                                           sample, true);
                                            
    %% Compute Standard Errors
    
    nobs = length(sample);
                                         
    [Se_inf_c,      Se_rep_c]       = get_se(model, true, cf_jacobian, Cdep, ...
                                             sample, false, ...
                                             analysis_path, nobs);
    [Se_inf_iv,     Se_rep_iv]      = get_se(iv, false, cf_jacobian, Cdep, ...
                                             sample, false, ...
                                             analysis_path, nobs);
    [Se_inf_panel,  Se_rep_panel]   = get_se(panel, false, cf_jacobian, Cdep, ...
                                             sample, false, ...
                                             analysis_path, nobs);                                     
                                         
                                           
    %% Save Results
                                          
    % Save Delta with no Data Dependency
    table_path = sprintf('%s/gentzkow_Delta.tsv', analysis_path);
    fid = fopen(table_path, 'w'); 
    fprintf(fid, '<tab:gentzkow_Delta>\n');
    fprintf(fid, '%.4f\n', Delta_comb);
    fprintf(fid, '%.4f\n', Delta_iv);
    fprintf(fid, '%.4f\n', Delta_panel);
    fclose(fid);
    
    % Save Delta with Data Dependency
    table_path = sprintf('%s/gentzkow_Delta_cdep.tsv', analysis_path);
    fid = fopen(table_path, 'w'); 
    fprintf(fid, '<tab:gentzkow_Delta_cdep>\n');
    fprintf(fid, '%.4f\n', Delta_comb_d);
    fprintf(fid, '%.4f\n', Delta_iv_d);
    fprintf(fid, '%.4f\n', Delta_panel_d);
    fclose(fid);

    % Save reported and influence function SE
    table_path = sprintf('%s/gentzkow_se.tsv', analysis_path);
    fid = fopen(table_path, 'w'); 
    fprintf(fid, '<tab:gentzkow_se>\n');
    fprintf(fid, '%.6f \t %.6f \n', [Se_inf_c; Se_rep_c]); 
    fprintf(fid, '%.6f \t %.6f \n', [Se_inf_iv; Se_rep_iv]);  
    fprintf(fid, '%.6f \t %.6f \n', [Se_inf_panel; Se_rep_panel]); 
    fclose(fid); 
                          
  exit
end

function model = get_model_components(score_index, analysis_path)
    %% Get the moment, score, and weighting function for the structural model
    in_path = sprintf('%s/gentzkow_aer_2007', analysis_path);
    load(in_path, 'score', 'vcov', 'T');
    
    % The score for c is computed for each observation after the ascribed
    % observations are dropped. So, we need to drop the observations with
    % no print media in the choice set. 
    model.G = -score'*score/size(score,1);
    model.phi_g = score(score_index,:)';
    model.W = eye(size(score,2));
end

function S = get_moment_components(S, data, spec, weight_type, analysis_path)
    %% Get the moment, score, and weighting function for the reduced form regressions
    in_path = sprintf('%s/beta_vcov/', analysis_path);
    mom = dlmread([in_path spec '_beta.txt']);
    S.vcov_mom = dlmread([in_path spec '_vcov.txt']);
    
    Y = data.(S.Y_var);
    X = get_subsample_matrix(data, S.X_vars);
    if isfield(S, 'Z_vars')
        Z = get_subsample_matrix(data, S.Z_vars);
        [S.M, S.phi_m, S.U] = get_linGMM_components(mom, Y, X, Z, weight_type);
    else
        [S.M, S.phi_m, S.U] = get_linGMM_components(mom, Y, X, X, weight_type);
    end
end

function X = get_subsample_matrix(data, vars)
    %% Select subset of data
    X = [double(data(:, vars))];
end

function [M, phi_m, U] = get_linGMM_components(b, Y, X, Z, weight_type)
    %% Get the moment, score, and weighting function for GMM
    nobs = size(X,1);
    k = size(Z,2);
    M = - Z'*X/nobs;
    phi_m = zeros(k, nobs);
    for i = 1:nobs
        phi_m(:,i) = (Z(i,:)'*(Y(i,:) - X(i,:)*b'))';
    end
    if strcmp(weight_type, '2sls')
        U = inv(Z'*Z/nobs);
    elseif strcmp(weight_type, 'eye')
        U = eye(k);
    end
end

function [Delta, Lambda] = compute_D_L_individual(S, model, cf_jacobian, Cdep, ...
                                                  sample, data_depend)
    %% Compute Delta and Lambda for the IV and Panel Regressions

    % Compute Influence Functions
    phi_gam = get_phi_gam(S);
    phi_c = get_phi_c(model, cf_jacobian, Cdep, sample, data_depend);
    
    % Get coefficient of interest
    phi_gam = phi_gam(1,:);
    
    % Compute Sigma
    nobs = size(phi_c,2);
    Sigma_cc = (phi_c * phi_c') / nobs;
    
    Sigma_gamgam = (phi_gam * phi_gam') / nobs;
    Sigma_cgam = (phi_c * phi_gam') / nobs;

    % Compute Delta and Lambda
    Lambda = Sigma_cgam / Sigma_gamgam;
    Delta = (Sigma_cgam * inv(Sigma_gamgam) * Sigma_cgam') ./ (Sigma_cc);

end

function [Delta, Lambda] = compute_D_L_combined(iv, panel, model, cf_jacobian, Cdep,... 
                                                sample, data_depend)
    %% Compute Delta and Lambda, combining iv and panel
 
    % Compute Influence Functions
    phi_c = get_phi_c(model, cf_jacobian, Cdep, sample, data_depend);
    phi_gam_panel = get_phi_gam(panel);
    phi_gam_iv = get_phi_gam(iv);
    
    % Get coefficients of interest
    phi_gam_panel = phi_gam_panel(1,:);
    phi_gam_iv = phi_gam_iv(1,:);
    
    % Compute Sigma. Note that in each case we get the variance or
    % covariance on the coefficient of interest.
    nobs = size(phi_c,2);
    Sigma_cc = phi_c * phi_c' / nobs;
    Sigma_c_iv = phi_c * phi_gam_iv' / nobs;
    Sigma_c_panel = phi_c * phi_gam_panel' / nobs;
    Sigma_iv_iv = phi_gam_iv * phi_gam_iv' / nobs;
    Sigma_panel_panel = phi_gam_panel * phi_gam_panel' / nobs;
    Sigma_iv_panel = phi_gam_iv * phi_gam_panel' / nobs;
    
    Sigma_gamgam = [Sigma_iv_iv, Sigma_iv_panel;...
                    Sigma_iv_panel', Sigma_panel_panel];
    Sigma_cgam   = [Sigma_c_iv,Sigma_c_panel]; 

    % Compute Delta and Lambda
    Lambda = Sigma_cgam / Sigma_gamgam;
    Delta = (Sigma_cgam * inv(Sigma_gamgam) * Sigma_cgam') ./ (Sigma_cc);

end

function phi_c = get_phi_c(model, cf_jacobian, Cdep, sample, data_depend)
    %% Get phi_c, assign to zeros if dropped in regression. Add counterfactal if data dependency. 
    phi_c_temp = - (cf_jacobian(1,:) * inv(model.G'*model.W*model.G) * (model.G'*model.W*model.phi_g));
    
    phi_c = zeros(size(phi_c_temp,1), length(sample));
    phi_c(:,sample) = phi_c_temp;
    
    if data_depend == true
       phi_c = (size(phi_c,2)*Cdep)' + phi_c;
    end
end

function phi_gam = get_phi_gam(S)
    %% Get phi_gam
    phi_gam = - (inv(S.M'*S.U*S.M) * (S.M'*S.U*S.phi_m));
end

function [Se_inf, Se_rep] = get_se(S, counterfactual, cf_jacobian, Cdep, ...
                                       sample, data_depend, ...
                                       analysis_path, nobs)
    %% Get the influence function and reported SE
    
    if counterfactual == true
        load(sprintf('%s/counterfactual', analysis_path),'cf_se');
        Se_rep = cf_se(1);
        
        phi = get_phi_c(S, cf_jacobian, Cdep, sample, data_depend);  
        
    else
        Se_rep = (S.vcov_mom(1,1))^(1/2); 
        
        phi = get_phi_gam(S);
        phi = phi(1,:);
    end
            
    Sigma = phi * phi' / nobs;
    Se_inf = ((1/(nobs)*Sigma)^(1/2));  

end
