%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% DEMCHANGES_CHAT_DEP.M: Calculate second choices
%
% Matthew Gentzkow
% November 29, 2004
% Online Editions
%
% note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
%
% note: SC and DCIRC will not add up to the same changes because SC forces
% consumers who chose a particular good to keep choosing it in the
% counterfactual, whereas DCIRC looks at the change in the model prediction
% without using this kind of information on actual choices.
%
% INPUTS:
%   nchbundle	    CC					number of consumers choosing each bundle
%   Y1dc            NNtot x CC             i x j=1 if consumer i chose bundle j and 0 otherwise
%   bstarmat        PP x mstderr        each column is one random draw on the parameter vector
%	excl		    PP					vector of exclusion restrictions
%	Xall		    nn x KK			    observable rhs vars
%   RD              nn*DD x msee        random draws
%	CM			    CC x GG				i*j=1 if good j included in choice i
% 	surveyweights	NNtot			    sampling weights of each observation
%   mweights     msee*NNtot x 1      probability weights for welfare calculation
%   mstderr         scalar              number of monte carlo draws
%	nn			    scalar				let this be an input so it can vary
%	
% OUTPUTS:
%	SC			CC x CC x GG		i x j x KK is number of consumers who had chosen i who would choose
%									j if good KK were not available
%	SCse		CC x CC x GG		std. errors of SC
%	DCIRC       GG x GG             i x j is change in demand for good j if take out good i
%   DCIRCse     GG x GG             std. errors of DCIRC
%   Cdep        NNtot x GG x GG     i x j x k is the change in the weighted
%                                   probability that the consumer i will choose good j if
%                                   take out good k
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [SC,SCse,D,Dse,Cdep] = demchanges_chat_dep(nchbundle,Y1dc,bstar,bstarmat,excl,Xall,RD,CM,surveyweights,mweights,mstderr)

global GG CC KK DD PP PMAX ZZ msee NNtot

bignegative = -50;          % number to subtract from utility to simulate removing from choice set

% reshape mweights to be CC*NNtot x msee
mweights = reshape(mweights,NNtot,msee);
mweights = repmat(mweights,CC,1);

% create matrix with bignegative in position of constant terms
neg = zeros(PMAX,GG);
for ii = 1:GG
    neg((ii-1)*(KK+DD) + KK, ii) = bignegative;
end
neg = selectinmat(neg,1-excl);

bstarmat = [bstar bstarmat];
Dmat = zeros(GG,GG,mstderr+1);
SCmat = zeros(CC,CC,GG,mstderr+1);
Cdepmat = zeros(NNtot,GG,GG,mstderr+1);

% loop over goods
for ii = 1:mstderr+1
    P1true = probabilities1(Xall',RD,bstarmat(:,ii),excl,CM',msee,NNtot); % P1 is msee*CC x NNtot
    P1true = reshape(P1true,CC,NNtot*msee);
    P1true = reshape(P1true',msee,CC*NNtot);

    % take a weighted average over draws conditioning on observed choices
    P1true = mean(P1true.*mweights',1);
    P1true = reshape(P1true,NNtot,CC);

    % multiply by sampling weights
    P1true = P1true.*repmat(surveyweights,1,CC);
    nchbundpred = sum(P1true); % predicted choices of each good
    
    for jj = 1:GG

        % calculate probabilities with good ii removed from the choice set
        bneg = bstarmat(:,ii)+neg(:,jj);
        P1 = probabilities1(Xall',RD,bneg,excl,CM',msee,NNtot); % P1 is msee*CC x NNtot
        P1 = reshape(P1,CC,NNtot*msee);
        P1 = reshape(P1',msee,CC*NNtot);

        % take a weighted average over draws conditioning on observed choices
        P1 = mean(P1.*mweights',1);
        P1 = reshape(P1,NNtot,CC);

        % multiply by sampling weights
        P1 = P1.*repmat(surveyweights,1,CC); 

        % changes in demand
        d = sum(P1)-sum(P1true);
        for kk = 1:GG
            Dmat(jj,kk,ii) = sum(d.*CM(:,kk)');
        end
        
        % counterfactual at each observation
        d2 = P1 - P1true;
        for kk = 1:GG
            for hh = 1:NNtot
                Cdepmat(hh,kk,jj,ii) = sum(d2(hh,:).*CM(:,kk)');
            end
        end

        % second choices
        for kk = 1:CC
            % if kk does not include jj demand is unchanged
            if CM(kk,jj)==0
                SCmat(kk,kk,jj,ii) = nchbundpred(kk);

            % otherwise look at these consumers' counterfactual demand
            else
                P1kk = selectinmat(P1,Y1dc(:,kk));
                SCmat(kk,:,jj,ii) = round(sum(P1kk));
            end
        end
        
    end
end

% SC, D, and Cdep are the arrays for the true parameter vector
SC=SCmat(:,:,:,1);
D=Dmat(:,:,1);
Cdep = Cdepmat(:,:,:,1);

if mstderr>1
    % 1 in std here is a flag to take second moment
    SCse=std(SCmat,1,4);
    Dse=std(Dmat,1,3);
else
    SCse=zeros(CC,CC,GG);
    Dse=zeros(GG,GG);
end