%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% SEEOED.M: Shell function for analysis of OED Model.
%
% Matthew Gentzkow
% September 25, 2003
% Online Editions
%
% note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
% note: MAKE SURE CROSSDERIV IS COMPILED W/ PROPER PARAMETER VALUES
%
% INPUTS:
%	filekey 	string				key of files in current directory (OEDout### and run###) where ### is key 
%
% VARIABLES IN INPUT FILE:
%	Y1all		NNtot				one-day choices
%	Y7all		NNtot				seven-day choices
%	Xall		NNtot x KK			observable rhs vars
%	CM			CC x GG					i*j=1 if good j included in choice i
%	headers		KK+DD					array of variable names
% 	surveyweights	NN					sampling weights of each observation
%	ascribed	NN					=1 if observation is ascribed
%	bstar		PP					final fitted parameter value
%	L			scalar				value of maximized likelihood
%	SE			PP					vector of standard errors
%	S			PP x PP				covariance matrix of parameter estimates
%	excl		PP					vector of exclusion restrictions
%	lhslist1	GG					names of goods included in estimation
%	
%
% FILES SAVED:
%	OEDout### (final parameter estimate, standard error, S, RESULT matrix, 
%				L, dL, CM, and parameters)
%	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function SEEOED(filekey)

global NN NNtot GG CC KK DD PP PMAX ZZ CCW mf mz msee melast mis

%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    PRELIMINARY STUFF    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

mstderr = 50;           % number of monte-carlo draws for calculating std errors
mistest = 5;            % number of monte-carlo draws for testing importance sampling
nowebpos = 33;          % position of no-web variable in Xall (0 if no such variable)
noprintpos = 32;        % position of no-print variable in Xall (0 if no such variable)

% set filenames
inputfile1 = strcat('run',filekey);
inputfile2 = strcat('OEDest',filekey);
outputfile = strcat('OEDout',filekey);

% load input variables
cd 'M:\PROJECTS\PrintOnline\current run\input'

load(inputfile1,'Y1all','Y7all','Xall','CM','surveyweights','ascribed',...
    'excl','lhslist1','Bcons','Bex','R','r','buildparams')

cd 'M:\PROJECTS\PrintOnline\current run\results'
load(inputfile2,'bstar','headers','lhslist1','dL','L','dLc','Hc','SE3','S3','estparams','msteps');

if (wrongbuildparams(buildparams))
    error('Parameters in data do not match those with which C-files were compiled');
end

[NNtot NN GG CC KK DD PMAX PP ZZ CCW] = extract(buildparams);
[UseDyn,ImpSamp,tolf0,tolff,tolx0,tolxf,mfe0,mfef,miter0,miterf,m0,mis,mf,mtest,msee,melast] = extract(estparams);
if DD==0
    msee = 1;              % number of draws to use in simulation
    mcd = 1;               % number of draws to use in calculating cross-derivatives
end
% pick standard errors
SE = SE3;
S=S3;
% new value for mis & msee
mis = 50;
msee = 100;

% seed random number generator and generate random draws
randn('state',sum(100*clock));
if DD>0
    RD = randn(NNtot*DD,msee);
else
    RD = zeros(1,msee);
end

% take out ascribed observations
Y1 = selectinmat(Y1all,1-ascribed);
Y7 = selectinmat(Y7all,1-ascribed);
X = selectinmat(Xall,1-ascribed);

% monte-carlo draws on bstar
sprintf('monte-carlo draws')
[T A] = transmats(R,r);
RDsterr = randn(ZZ,mstderr);
Sc = T'*S*T;
bstarc = (bstar'*T)';
Schol = chol(Sc)';
bstarcmat = repmat(bstarc,1,mstderr) + (Schol * RDsterr);
bstarmat = (bstarcmat'*T'+repmat(A',mstderr,1))';

% calculate all utilities
sprintf('calculate utilities')
Uall = utilities(bstar,excl,Xall',RD,CM',msee,NNtot);
Uall = reshape(Uall,CC-1,msee*NNtot)';  % reshape so it's (msee*NNtot x CC-1)
if DD>0
    Uobs = utilities(bstar,excl,Xall',zeros(NNtot*DD,msee),CM',msee,NNtot);
else
    Uobs = utilities(bstar,excl,Xall',zeros(1,msee),CM',msee,NNtot);
end    
Uobs = reshape(Uobs,CC-1,msee*NNtot)';
Uunobs = utilities(bstar,excl,zeros(KK,NNtot),RD,CM',msee,NNtot);
Uunobs = reshape(Uunobs,CC-1,msee*NNtot)';

% calculate variance/covariance components of utilities
sprintf('variance/covariance componenets')
webv = zeros(NNtot,1);
printv = zeros(NNtot,1);
if nowebpos>0
    webv = Xall(:,nowebpos);
end
if noprintpos>0
    printv = Xall(:,noprintpos);
end
csobs = 1-max(webv,printv);
NNweb = sum(csobs);
Uallcs = selectinmat(Uall,repmat(csobs,msee,1));
Uobscs = selectinmat(Uobs,repmat(csobs,msee,1));
Uunobscs = selectinmat(Uunobs,repmat(csobs,msee,1));
CORobs = corrcoef(Uobscs(1:NNweb,1:GG));
Vobs = cov(Uobscs(1:NNweb,1:GG));
if DD>0
    CORunobs = corrcoef(Uunobscs(:,1:GG));
    Vunobs = cov(Uunobscs(:,1:GG));
else
    CORunobs = zeros(GG,GG);
    Vunobs = zeros(GG,GG);
end
Vepsilon = (pi^2/6)*eye(GG);
Vtotal = Vobs+Vunobs+Vepsilon;
clear Uobs Uallcs Uunobs Uobscs Uunobscs;

% P, P(1-P), and Conditional Probability Weights
sprintf('probabilities and weights')
[predshares prednchgood P1avg P1minP1avg mweights P1cond P1] = ...
    probweight(Xall,Y1all,Y7all,X,Y1,Y7,RD,bstar,excl,CM,Bcons,Bex,msee,UseDyn,ascribed,surveyweights);

% calculate number choosing each good/bundle
sprintf('number choosing')
[nchgood nchbundle Y1dc] = numchoosing(Y1all,CM,surveyweights,NNtot);

% calculate distribution of interaction terms
sprintf('gamma terms')
[gammas,gammasse,gammasmin,gammasmax,gammasvar] = ...
            calcgammas(nchbundle,bstar,bstarmat,excl,Xall,RD,CM,surveyweights,mstderr);

% calculate distribution of second choices & changes in demand
sprintf('changes')
[SC,SCse,D,Dse] = demchanges(nchbundle,Y1dc,bstar,bstarmat,excl,Xall,RD,CM,surveyweights,mweights,mstderr);

% calculate cross-derivatives
sprintf('cross-derivatives')
[CDwTOT,CDwTOTcov,CDwTOTse,CDw] = crossderivatives(msee,mstderr,bstar,bstarmat,excl,Xall,CM,surveyweights,mweights);

% goodness of fit
sprintf('goodness of fit')
[probsyes probsno] = goodfit(Y1,P1');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    TEST IMPORTANCE SAMPLING            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%sprintf('test importance sampling')
%b0 = bstar.*(1+randn(PP,1)/2);
%[varis,pavgis,varstd,pavgstd] = testis(X,Y1,Y7,CM,Bcons,Mcons,Mincl,Bex,bstar,excl,mistest,msee,b0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    PARAMETER TABLE      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

sprintf('parameter table')
bstarfull = reverseselect(bstar,1-excl);
SEfull = reverseselect(SE,1-excl);
table = reshape([bstarfull(1:PMAX-3) SEfull(1:PMAX-3)]',(PMAX-3)*2,1);
table = reshape(table,2*(KK+DD),CC-1);
headlong =  reshape([headers repmat({''},(KK+DD),1)]',(KK+DD)*2,1);
tablemarg = reshape([bstarfull(1:PMAX-3) zeros(PMAX-3,1)]',(PMAX-3)*2,1);
tablemarg = reshape(tablemarg,2*(KK+DD),CC-1).*repmat(P1minP1avg(2:CC)',2*(KK+DD),1);
tablemargpct = tablemarg./repmat(P1avg(2:CC)',2*(KK+DD),1);

pie = bstar(PP);
gam = bstar(PP-1);
tau = bstar(PP-2);
if tau<0
    tau = -tau;
    gam = 1-gam;
end
tablenl = [tau;SE(PP-2);gam;SE(PP-1);pie;SE(PP)];

% save
save(outputfile,'RD','Uall','CORobs','CORunobs','Vobs','Vunobs','Vepsilon','Vtotal','nchgood','nchbundle',...
                'SC','SCse','D','Dse','gammas','gammasse','gammasmin','gammasmax','gammasvar','prednchgood',...
                'CDwTOT','CDwTOTse','CDwTOTcov','CDw','mweights','probsyes','probsno','table','headlong',...
                'tablemarg','tablemargpct','predshares','msee','lhslist1','CM','estparams','buildparams','tablenl');