/***************************************************************************
 *
 * OEDCALC.H: General header file for OED model. Contains routines to calculate
 *			utilities of each agent given data and parameters. Also conatains
 *			routines to calculate from these the probabilities of different
 *			one and seven-day choices. Information on the individual functions
 *			is given with their definitions below.
 *
 *	note: calling file must include "matlab.h"
 *
 * Matthew Gentzkow
 * December 20, 2004
 * Online Editions
 *
 ****************************************************************************/

/* define data parameters */
#define NN 15372                /* number of consumers after ascribed taken out */
#define GG 3					/* number of goods */
#define CC 8					/* number of choices */
#define KK 35                  /* number of observables */
#define DD 3					/* number of unobservables */
#define PMAX 269                /* total number of parameters before exclusion restrictions */
#define PP 110					/* number of parameters (net of exclusions restrictions) */
#define ZZ 106					/* number of free parameters after constraints imposed */
#define CCW 27					/* number of possible 1/7 day combinations */


 /*********************************************************************************************
 *
 * CALCU: Function that calculates observable component of utility for all choices
 *			and all consumers. Note that this function is a bit complicated and counter-
 *			intuitive because it's optimized to take account of the special structure
 *			of (1) observable component of utility that stays the same across the
 *			m unobservables draws and (2) utilities for combinations that include
 *			a sum over utilities of underlying goods. It therefore calculates the
 *			observable component of each good's utility only once and then adds each
 *			unobservable draw separately. It also calculates each good's utility only
 *			once and then adds them together to form the combinations.
 *
 * note: a*b*c in dimension means vectors stacked as a groups of b groups of c
 * note: these routines are set up to work find if d>0. In this case RD should be
 *		 a pointer to any address.
 *
 * INPUTS:
 *	U			m*NN*(CC-1)			(OUTPUT) vector of utilities
 *	b			PP					unconstrained parameter vector
 *	excl		PMAX				vector of exclusion restrictions (=1 if excluded)
 *	X			NN*KK				observable variables
 *	RD			m*NN*DD				random draws on unobservables			
 *	CM			CC*GG				i*j=1 if good j included in choice i
 *	minput		scalar				number of simulation draws
 *	ninput		scalar				number of observations
 *	
 *********************************************************************************************/

void calcu(double *U, double *b, double *excl, double *X, double *RD, double *CM, int minput, int ninput); 



 /*********************************************************************************************
 *
 * CALCUSINGLE: Function that calculates observable component of utility for all choices
 *			but a single consumer/draw. 
 *
 * note: a*b*c in dimension means vectors stacked as a groups of b groups of c
 *
 * INPUTS:
 *	U			(CC-1)				(OUTPUT) vector of utilities
 *	b			PP					unconstrained parameter vector
 *	excl		PMAX				vector of exclusion restrictions (=1 if excluded)
 *	data		(KK+DD)				observable rhs vars and random draws
 *	CM			CC*GG				i*j=1 if good j included in choice i
 *	
 *********************************************************************************************/

void calcusingle(double *U, double *b, double *excl, double *data, double *CM); 


 /*********************************************************************************************
 *
 * CALCNONLIN
 *
 * note: a*b*c in dimension means vectors stacked as a groups of b groups of c
 *
 * INPUTS:
 *	coeff		5					(OUTPUT) vector of coefficients
 *	dcoeff		5					(OUTPUT) vector of derivatives of coefficients
 *  gam         scalar              probability of a good news day
 *  dgam        scalar              derivative of the unconstrained parameter gamhat w.r.t. gam
 *  uadd        CC-1                incremental utility to add with probability gam
 *	b			PP                  parameter vector
 *	CM			CC*GG				i*j=1 if good j included in choice i
 *	
 *********************************************************************************************/

void calcnonlin(double *coeff, double *dcoeff, double *gam, double *dgam, double *uadd, 
                double *b, double *CM);


/* ********************************************************************************
 *
 * CALCCOEFF: calculate coefficients for each day's probability  
 *
 * note: a*b*c in dimension means vectors stacked as a groups of b groups of c
 *
 * INPUTS:
 *	coeff		5					(OUTPUT) vector of coefficients
 *	pi			scalar				probability of choice on a given day
 *
 * ******************************************************************************* */

void calccoeff(double *coeff, double pi);


/* ********************************************************************************
 *
 * CALCDCOEFF: calculate derivatives of coefficients for each day's probability 
 *
 * note: a*b*c in dimension means vectors stacked as a groups of b groups of c
 *
 * INPUTS:
 *	dcoeff		5					(OUTPUT) vector of derivatives of coefficients
 *	pi			scalar				probability of choice on a given day
 *  dpi			scalar				derivative of probability wrt a given param
 *
 * ******************************************************************************** */

void calcdcoeff(double *dcoeff, double pi, double dpi);


/*********************************************************************************************
 *
 * FINDCONSTANTS: Return the indices in the parameter vector of the constants for each good.
 *
 *  constants       KK                location in the parameter vector of the constant term for i
 *  excl            PP                exclusion restrictions
 *  X               NN*KK             data
 *	
 *********************************************************************************************/

void findconstants(int *constants, double *excl, double *X);



/* ********************************************************************************
 *
 * EXPAND: function to expand the parameter vector bc into full vector b. (See
 *		   STATA documentation on "constraint" for more information)
 *
 * note: a*b*c in dimension means vectors stacked as a groups of b groups of c
 *
 * INPUTS:
 *	b			PP					(OUTPUT) expanded vector
 *	bc			ZZ					constrained vector to expand
 *  T			ZZ*PP				transformation matrix
 *  A			PP					transformation vector
 *
 * ******************************************************************************** */

void expand(double *b, double *bc, double *T, double *A);


 /*********************************************************************************************
 *
 * CALCNONLINSM
 *
 * note: a*b*c in dimension means vectors stacked as a groups of b groups of c
 *
 * INPUTS:
 *  gam         scalar              probability of a good news day
 *  uadd        CC-1                incremental utility to add with probability gam
 *	b			PP                  parameter vector
 *	CM			CC*GG				i*j=1 if good j included in choice i
 *	
 *********************************************************************************************/

void calcnonlinsm(double *gam, double *uadd, double *b, double *CM);




