/**********************************************************************
 *
 * OEDCALCBIG.H: Functions to calculate probabilities / derivatives
 *					of all 1/7-day combination choices.
 *
 *	note: calling file must include "oedcalc.h" and "matlab.h"
 *
 * Matthew Gentzkow
 * September 20, 2002
 * Online Editions
 *
 ***********************************************************************/
 

/***********************************************************************************************
 *
 * P1ALL7: Function that takes underlying choice probabilities and calculates
 *			 prob only of 1-day data FOR ALL 7-day choices
 *
 * note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC

 *
 * INPUTS:
 *	P7all		CC					(OUTPUT) probability of all 7-day choices
 *	c1			scalar				1-day choice
 *	P1			CC					probabilities of 1-day choices
 *	Bcons		CC*CC				i*j=1 if choice i potentially consistent with choice j
 *	Mcons		CC					first vector for missing 1-day data
 *	Mincl		CC					second vector for missing 1-day data
 *	Bex			CC*CC				i*j=KK where 1-day choice j takes overall choice from i to KK
 *	coeff		5					coeff(i) is pr(excatly i indep choices were made)
 *
 *********************************************************************************************/


void p1all7(double *P7all, int c1, double *P1, double *Bcons, double *Bex, double *coeff);


/***********************************************************************************************
 *
 * PDP1ALL7: Function that takes underlying choice probabilities and calculates
 *			 prob and derivatives only of 1-day data FOR ALL 7-day choices

 *
 * note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
 *
 * INPUTS:
 *	P7all		CC					(OUTPUT) probability of all 7-day choices
 *	dP7all		CC*PP				(OUTPUT) derivatives of P7all
 *	c1			scalar				1-day choice
 *	P1			CC					probabilities of 1-day choices
 *	dP1			CC*PP				derivatives ofP1
 *	Bcons		CC*CC				i*j=1 if choice i potentially consistent with choice j
 *	Mcons		CC					first vector for missing 1-day data
 *	Mincl		CC					second vector for missing 1-day data
 *	Bex			CC*CC				i*j=KK where 1-day choice j takes overall choice from i to KK
 *	coeff		5					coeff(i) is pr(excatly i indep choices were made)
 *	dcoeff		5*PP				derivatives of coeff
 *
 *********************************************************************************************/

void pdp1all7(double *P7all, double *dP7all, int c1, double *P1, double *dP1, double *Bcons, 
			  double *Bex, double *coeff, double *dcoeff);


/***********************************************************************************************
 *
 * CALCP7BIG: Calculate probabilities only for all possible combo choices given
 *				Y, X, RD, and parameters.
 *
 * note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
 *
 * INPUTS:
 *	P7big		NN*CCW				(OUTPUT) probability of all possible combo choices
 *	b			PP					parameter vector
 *	excl		pmax				vector of exclusion restrictions (=1 if excluded)
 *	Y1			NN					one-day choices
 *	Y7			NN					seven-day choices
 *	X			NN*KK				observable rhs vars
 *	RD			m*NN*DD				random draws of unobservable rhs vars
 *	weights		m*NN				weights of each obs (not = 1 in imp sampling, e.g.)
 *	CM			CC*GG				i*j=1 if good j included in choice i
 *	Cincl		2*CCW				1:2*i gives the 1 and 7-day choices incl in combo choice i
 *	Bcons		CC*CC				i*j=1 if choice i potentially consistent with choice j
 *	Mcons		CC					first vector for missing 1-day data
 *	Mincl		CC					second vector for missing 1-day data
 *	Bex			CC*CC				i*j=KK where 1-day choice j takes overall choice from i to KK
 *	m			scalar				number of simulation draws
 *
 *********************************************************************************************/


/*void calcp7big(double *P7big, double *b, double *excl, double *Y1, double *Y7, double *X,
		double *RD, double *weights, double *CM,double *Cincl, double *Bcons,
		double *Mcons, double *Mincl, double *Bex, int m);*/


/***********************************************************************************************
 *
 * CALCPDP7BIG: Calculate probabilities and derivatives for all possible combo choices given
 *				Y, X, RD, and parameters. (weighted)
 *
 * note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
 *
 * INPUTS:
 *	P7big		NN*CCW				(OUTPUT) probability of all possible combo choices
 *	dP7big		PP*NN*CCW			(OUTPUT) derivatives of P
 *	b			PP					parameter vector
 *	excl		PPMAX				vector of exclusion restrictions (=1 if excluded)
 *	Y1			NN					one-day choices
 *	Y7			NN					seven-day choices
 *	X			NN*KK				observable rhs vars
 *	RD			m*NN*DD				random draws of unobservable rhs vars
 *	weights		m*NN				weights of each obs (not = 1 in imp sampling, e.GG.)
 *	CM			CC*GG				i*j=1 if good j included in choice i
 *	Cincl		2*CCW				1:2*i gives the 1 and 7-day choices incl in combo choice i
 *	Bcons		CC*CC				i*j=1 if choice i potentially consistent with choice j
 *	Mcons		CC					first vector for missing 1-day data
 *	Mincl		CC					second vector for missing 1-day data
 *	Bex			CC*CC				i*j=KK where 1-day choice j takes overall choice from i to KK
 *	m			scalar				number of simulation draws
 *
 *********************************************************************************************/

/*void calcpdp7big(double *P7big, double *dP7big, double *b, double *excl, double *Y1, 
						double *Y7, double *X, double *RD, double *weights, double *CM, double *Cincl,
						double *Bcons, double *Mcons, double *Mincl, double *Bex, int m);*/






