/***************************************************************************
 *
 * OEDOBJECTIVES.H: Header file for SML and MSM objective functions.
 *
 *	note: calling file must include "oedcalc.h" and "matlab.h"
 *
 * Matthew Gentzkow
 * July 11, 2003
 * Online Editions
 *
 ****************************************************************************/




/***********************************************************************************************
 *
 * LIKELIHOOD: Function to evaluate logit likelihood using only 1-day data.
 *			Likelihood is weighted for importance sampling estimation.
 * 
 *          *** if dLc==NULL won't calculate derivatives ***
 *          *** if Y7==NULL will calculate only using 1-day data ***
 *
 * note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
 * note: to run with no unobservables, simply set DD=0 and either m=0 or m=1.
 *
 * INPUTS:
 *  L			scalar				(OUTPUT) likelihood of observed data
 *  dLc			ZZ					(OUTPUT) score vector wrt constrained parameters
 *	bc			ZZ					(constrained) parameter vector
 *	minput		scalar				number of simulation draws
 *	excl		pmax				vector of exclusion restrictions (=1 if excluded)
 *	Y1			NN					one-day choices
 *	Y7			NN					seven-day choices
 *	X			NN*KK				observable rhs vars
 *	RD			m*NN*DD				random draws of unobservable rhs vars
 *	CM			CC*GG				i*j=1 if good j included in choice i
 *	Bcons		CC*CC				i*j=1 if choice i potentially consistent with choice j
 *	Bex			CC*CC				i*j=KK where 1-day choice j takes overall choice from i to KK
 *	T			ZZ*PP				constraint matrix
 *	A			PP					constraint vector
 *	weights		m*NN				weights of each obs (not = 1 in imp sampling, e.g.)
 *

 *********************************************************************************************/
   
void likelihood( double *L, double *dLc, double *bc, int minput, double *excl, double *Y1, 
				double *Y7, double *X, double *RD, double *CM, double *Bcons, 
                double *Bex, double *T, double *A, double *weights); 



/*********************************************************************************************
 *
 * MSMOBJECTIVE: Function to calculate method of moments objective function
 *				for the OED model. Returns both sum of squared moments and gradient vector.
 *
 * note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
 *
 * INPUTS:
 *	Gn			scalar				(OUTPUT) objective
 *	dGnc		ZZ					(OUTPUT) gradient vector (w.r.t. cons parameters)
 *	bc			ZZ					(constrained) parameter vector
 *	excl		pmax				vector of exclusion restrictions (=1 if excluded)
 *	Z			PP*NN*CCW			vector of instruments (gradient at preliminary parameter value)
 *	Y1			NN					one-day choices
 *	Y7			NN					seven-day choices
 *	Cdum		NN*CCW				i*j=1 if consumer i chose combination choice j =0 otherwise
 *	X			NN*KK				observable rhs vars
 *	RD			m*NN*DD				random draws of unobservable rhs vars
 *	weights		m*NN				weights of each obs (not = 1 in imp sampling, e.GG.)
 *	T			ZZ*PP				constraint matrix
 *	A			PP					constraint vector
 *	CM			CC*GG				i*j=1 if good j included in choice i
 *	Cincl			2*CCW			1:2*i gives the 1 and 7-day choices incl in combo choice i
 *	Bcons		CC*CC				i*j=1 if choice i potentially consistent with choice j
 *	Bex			CC*CC				i*j=KK where 1-day choice j takes overall choice from i to KK
 *	m			scalar				number of simulation draws
 *
 **********************************************************************************************/

/*void msmobjective(double *Gn, double *dGnc, double *bc, double *excl, double *Z, double *Y1, 
		double *Y7, double *Cdum, double *X, double *RD, double *weights, double *T, 
		double *A, double *CM, double *Cincl, double *Bcons, double *Bex, int m);*/


/*********************************************************************************************
 *
 * MSMOBJECTIVEND: Same as msmobjective except does not calculate derivatives.
 *
 * note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
 *
 * INPUTS:
 *	Gn			scalar				(OUTPUT) objective
 *	bc			ZZ					(constrained) parameter vector
 *	excl		PMAX				vector of exclusion restrictions (=1 if excluded)
 *	Z			PP*NN*CCW				vector of instruments (gradient at preliminary parameter value)
 *	Y1			NN					one-day choices
 *	Y7			NN					seven-day choices
 *	Cdum		NN*CCW				i*j=1 if consumer i chose combination choice j =0 otherwise
 *	X			NN*KK					observable rhs vars
 *	RD			m*NN*DD				random draws of unobservable rhs vars
 *	weights		m*NN				weights of each obs (not = 1 in imp sampling, e.GG.)
 *	T			ZZ*PP					constraint matrix
 *	A			PP					constraint vector
 *	CM			CC*GG					i*j=1 if good j included in choice i
 *	Cincl		2*CCW				1:2*i gives the 1 and 7-day choices incl in combo choice i
 *	Bcons		CC*CC					i*j=1 if choice i potentially consistent with choice j
 *	Bex			CC*CC					i*j=KK where 1-day choice j takes overall choice from i to KK
 *	m			scalar				number of simulation draws
 *
 ************************************************************************************************/

/*void msmobjectivend(double *Gn, double *bc, double *excl, double *Z, double *Y1, double *Y7, 
		double *Cdum, double *X, double *RD, double *weights, double *T, double *A, 
		double *CM, double *Cincl, double *Bcons, double *Bex, int m);*/



