/***************************************************************************
 *
 * OEDTOOLS.H: Header file for additional OED functions.
 *
 *	note: calling file must include "oedcalc.h" and "matlab.h"
 *
 * Matthew Gentzkow
 * July 11, 2003
 * Online Editions
 *
 ****************************************************************************/

 

/**********************************************************************************
 *
 * PROBABILITIES7: Generate vector of probabilities of observed one / seven-day
 *					choice for each consumer and simulation draw.
 *
 * note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
 *
 * INPUTS:
 *	P			m*ninput				(OUTPUT) Simulated probabilities of selected (seven-day) choice
 *											at each simulation draw
 *	X			ninput*KK				observable rhs vars
 *	Y1			ninput					one-day choices
 *	Y7			ninput					seven-day choices
 *	RD			m*ninput*DD				random draws of unobservable rhs vars
 *	b			PP						parameter vector CC groups of (KK+DD) followed by f(pi)
 *	excl		PMAX					vector of exclusion restrictions (=1 if excluded)

 *	CM			CC*GG					i*j=1 if good j included in choice i
 *	Bcons		CC*CC					i*j=1 if choice i potentially consistent with choice j
 *	Bex			CC*CC					i*j=KK where 1-day choice j takes overall choice from i to KK
 *	m			scalar					number of simulation draws
 *	ninput		scalar					number of consumers
 *
 ***********************************************************************************/
 
void probabilities7(double *P, double *X, double *Y1, double *Y7, double *RD, double *b, double *excl, 
				 double *CM, double *Bcons, double *Bex, int m, int ninput);


/**********************************************************************************
 *
 * PROBABILITIES1: Generate one-day probabilities for each consumer at each
 *					simulation draw for each choice.
 *
 * note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
 *
 * INPUTS:
 *	P1			NN*m*CC				(OUTPUT) Simulated probabilities of each (one-day) choice
 *									for each draw
 *	X			NN*KK				observable rhs vars
 *	RD			m*NN*DD				random draws of unobservable rhs vars
 *	b			PP					parameter vector CC groups of (KK+DD) followed by f(pi)
 *	excl		PMAX				vector of exclusion restrictions (=1 if excluded)
 *	CM			CC*GG				i*j=1 if good j included in choice i
 *	m			scalar				number of simulation draws
 *  ninput      scalar              number of observations
 *
 ***********************************************************************************/

void probabilities1(double *P1, double *X, double *RD, double *b, double *excl, double *CM, int m, int ninput);


/**********************************************************************
 *
 * CROSSDERIV: Calculate cross-derivatives (i.e. change in demand for
 *				i with respect to utility of good j). Used, then, to
 *				compute elasticities. Can calculate a weighted
 *				average over simulation draws (i.e. for importance sampling).
 *
 *              *** If mweights==NULL calculates unweighted average ***
 *
 * note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
 *
 * INPUTS:
 *	CD			ninput*GG*GG		(OUTPUT) i*j*KK is change in prob that i chooses j w.r.t.
 *									utility of KK (or change in prob of KK w.r.t. j)
 *	b			PP					parameter vector CC groups of (KK+DD) followed by f(pi)
 *	excl		PMAX				vector of exclusion restrictions (=1 if excluded)
 *	m			scalar				number of simulation draws
 *	ninput		scalar				number of consumers
 *	X			ninput*KK			observable rhs vars
 *	RD			m*NN*DD				random draws of unobservable rhs vars
 *	CM			CC*GG				i*j=1 if good j included in choice i
 *	mweights	m*ninput			i*j is weight for draw j for consumer i
 *
 ***********************************************************************/

void crossderiv(double *CD, double *b, double *excl, int m, int ninput, double *X, double *RD, double *CM, double *mweights); 


/***********************************************************************************************
 *
 * DELTAMAT: Calculate the average of the outer product of the gradients
 *			 for use in robust covariance matrix estimation. Note that this
 *			 by itself is a consistent estimator of the information matrix.
 *
 *          *** If Y7==NULL calculates using 1-day probabilities ***
 *
 * note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
 *
 * INPUTS:
 *	DM			PP*PP					matrix symmetric so order doesn't matter
 *	b			PP					parameter vector CC groups of (KK+DD) followed by f(pi)
 *	excl		PMAX				vector of exclusion restrictions (=1 if excluded)
 *	m			scalar				number of simulation draws
 *	X			NN*KK					observable rhs vars
 *	RD			m*NN*DD				random draws of unobservable rhs vars
 *	CM			CC*GG					i*j=1 if good j included in choice i
 *	Bcons		CC*CC					i*j=1 if choice i potentially consistent with choice j
 *	Bex			CC*CC					i*j=KK where 1-day choice j takes overall choice from i to KK
 *	weights		m*NN				weights of each obs (not = 1 in imp sampling, e.GG.)
 *	Y1			NN					one-day choices
 *	Y7			NN					seven-day choices
 *
 *********************************************************************************************/
 
void deltamat(double *DM, double *b, double *excl, int m, double *X, double *RD, double *CM, 
				   double *Bcons, double *Bex, double *weights, double *Y1, double *Y7);


/**********************************************************************
 *
 * GRADIENTS: Calculate gradients given Y, DATA, and parameters
 *
 *             *** if Y7==NULL calculates using 1-day data ***
 *
 * note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
 *
 * INPUTS:
 *	G			NN*PP				i*j is derivative of prob of i's observed choice
 *	b			PP					parameter vector CC groups of (KK+DD) followed by f(pi)
 *	excl		PMAX				vector of exclusion restrictions (=1 if excluded)
 *	minput		scalar				number of simulation draws
 *	X			NN*KK				observable rhs vars
 *	RD			minput*NN*DD		random draws of unobservable rhs vars
 *	CM			CC*GG				i*j=1 if good j included in choice i
 *	Bcons		CC*CC				i*j=1 if choice i potentially consistent with choice j
 *	Bex			CC*CC               i*j=KK where 1-day choice j takes overall choice from i to KK
 *	weights		minput*NN			weights of each obs (not = 1 in imp sampling, e.GG.)
 *	Y1			NN					one-day choices
 *	Y7			NN					seven-day choices
 *
 ***********************************************************************/

void gradients( double *G, double *b, double *excl, int minput, double *X, double *RD, double *CM, 
				   double *Bcons, double *Bex, double *weights, double *Y1, double *Y7);


/*********************************************************************************************
 *
 * ISDRAWS: Draw from importance sampling distribution. Return draws
 *		   along with probability of observed choice for each individual
 *		   and each draw. 
 *
 * note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
 *
 * INPUTS:
 *	P0			m*NN					prob of obs choice for each cons at each sim draw
 *	RD			m*NN*DD				imp sampling random draws for each of DD unobservables
 *	b			PP					parameter vector
 *	excl		PMAX				vector of exclusion restrictions (=1 if excluded)
 *	m			scalar				number of simulation draws
 *	Y1			NN					one-day choices
 *	Y7			NN					seven-day choices
 *	X			NN*KK					observable rhs vars
 *	CM			CC*GG					i*j=1 if good j included in choice i
 *	Bcons		CC*CC					i*j=1 if choice i potentially consistent with choice j
 *	Bex			CC*CC					i*j=KK where 1-day choice j takes overall choice from i to KK
 *	IJseed		scalar				first seed for C rn generator (0<=IJ<=31328)
 *	KLseed		scalar				second seed (0<=KL<=30081)
 *
 ************************************************************************************************/

void isdraws( double *P0, double *RD, double *b, double *excl, int m, double *Y1, 
				double *Y7, double *X, double *CM, double *Bcons, double *Bex, int IJseed, int KLseed); 



/**********************************************************************
 *
 * TESTGRAD: Calculate gradients given Y, DATA, and parameters. Identical
 *				to gradients.CC except that uses oedtest.h header file
 *				instead of oed.h so NN can be set to number of sims rather
 *				than (actual) number of observations
 *
 * note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
 *
 * INPUTS:
 *	G			NN*PP					i*j is derivative of prob of i's observed choice
 *									w.r.t parameter j
 *	b			PP					parameter vector CC groups of (KK+DD) followed by f(pi)
 *	excl		PMAX				vector of exclusion restrictions (=1 if excluded)
 *	Y1			NN					one-day choices
 *	Y7			NN					seven-day choices
 *	X			NN*KK					observable rhs vars
 *	RD			m*NN*DD				random draws of unobservable rhs vars
 *	weights		NN					weights of each obs (not = 1 in imp sampling, e.GG.)
 *	CM			CC*GG					i*j=1 if good j included in choice i
 *	Bcons		CC*CC					i*j=1 if choice i potentially consistent with choice j
 *	Bex			CC*CC					i*j=KK where 1-day choice j takes overall choice from i to KK
 *	mptr			scalar				number of simulation draws
 *
 ***********************************************************************/
 
/*void testgrad(double *G, double *b, double *excl, double *Y1, double *Y7, 
				  double *X, double *RD, double *weights, double *CM, double *Bcons,
				  double *Mcons, double *Mincl, double *Bex, int *mptr);*/





