/***********************************************************************************************
 *
 * DELTAMAT1: Calculate the average of the outer product of the gradients
 *			 for use in robust covariance matrix estimation with 1-day data. Note that this
 *			 by itself is a consistent estimator of the information matrix.
 *
 * Matthew Gentzkow
 * September 23, 2003
 * Online Editions
 * 
 * MATLAB prototype:
 * DM = deltamat1(b,excl,m,X,RD,CM,Bcons,Mcons,Mincl,Bex,weights,Y1,Y7)
 *
 * compiler command: 
 *	mex deltamat1.c oedcalc.c oedtools.c
 *
 * note: a*b*c in dimension means vectors stacked as a groups of b groups of c
 * note: # before range means only integer values are valid
 *
 * INPUTS:
 *	b			PP			(-inf,inf)		param vec: c groups of (k+d) then f(pi)
 *	excl		PMAX		#(0,1)			vec of exclusion restrictions (=1 if excl)
 *	m			scalar		#(0,inf)		number of simulation draws
 *	X			KK x NN		(-inf,inf)		observable rhs vars
 *	RD			NN*DD x m	(-inf,inf)		random draws of unobservable rhs vars
 *	CM			GG x CC		#(0,1)			i x j=1 if good i included in choice j
 *	Bcons		CC x CC		#(0,1)			i x j=1 if choice j potentially consist with choice i
 *	Bex			CC x CC		#(0,CC-1)		i x j=k where 1-day ch i takes overall ch from j to k
 *	weights		NN x m		(-inf,inf)		weights of each obs (eg for imp sampling)
 *	Y1			NN			#(0,CC-1)		one-day choices
 *
 * OUTPUTS:
 *	DM			PP x PP		(inf,inf)		matrix symmetric so order doesn't matter
 *
 *********************************************************************************************/

#include <math.h>
#include "mex.h"
#include "choose_max.h"
#include "oedcalc.h"
#include "oedtools.h"

#define NINPUTS 10
#define NOUTMAX 1

/* ****************************************************** */
/* mexFunction: MATLAB gateway routine                    */
/*														  */
/* remember: m is rows, n is columns					  */
/*													      */
/* ****************************************************** */


void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[] )
{
  int n[NINPUTS], m[NINPUTS], ngood[NINPUTS], mgood[NINPUTS], integer[NINPUTS],
	  nout[NOUTMAX], mout[NOUTMAX];
  double min[NINPUTS], max[NINPUTS];
  char errstring[200];
  int ii;
  int mm,rdval;
  double *IPTRS[NINPUTS], *OPTRS[NOUTMAX];
  
  /* get m here to assign dimensions */
  mm = *mxGetPr(prhs[2]);

  /* handle the case of DD==0 */
  rdval = choose_max(NN*DD,1);
  
  /* set ngood and mgood arrays with expected input dimensions */
  mgood[0] = PP;			ngood[0] = 1;
  mgood[1] = PMAX;			ngood[1] = 1;
  mgood[2] = 1;				ngood[2] = 1;
  mgood[3] = KK;			ngood[3] = NN;
  mgood[4] = rdval;			ngood[4] = mm;
  mgood[5] = GG;			ngood[5] = CC;
  mgood[6] = CC;			ngood[6] = CC;
  mgood[7] = CC;			ngood[7] = CC;
  mgood[8] = NN;			ngood[8] = mm;
  mgood[9] = NN;			ngood[9] = 1;

  /* set min and max arrays with allowable range of inputs */
  /* (enter 9999 or -9999 to represent +/- infinity)  */
  /* integer[ii]=1 indicates input must be an integer */
  min[0] = -9999;		max[0] = 9999;		integer[0] = 0;
  min[1] = 0;			max[1] = 1;			integer[1] = 1;
  min[2] = 0;			max[2] = 9999;		integer[2] = 1;
  min[3] = -9999;		max[3] = 9999;		integer[3] = 0;
  min[4] = -9999;		max[4] = 9999;		integer[4] = 0;
  min[5] = 0;			max[5] = 1;			integer[5] = 1;
  min[6] = 0;			max[6] = 1;			integer[6] = 1;
  min[7] = 0;			max[7] = CC-1;		integer[7] = 1;
  min[8] = -9999;		max[8] = 9999;		integer[8] = 0;
  min[9] = 0;			max[9] = CC-1;		integer[9] = 1;

  /*set nout and mout arrays with output dimensions */
  mout[0] = PP;				nout[0] = PP;
  
  /* check number of arguments */
  if(nrhs != NINPUTS)
    mexErrMsgTxt("NINPUTS inputs required.");
  if(nlhs > NOUTMAX)
    mexErrMsgTxt("At most NOUTMAX outputs allowed.");

  /* get pointers and dimensions of each input element */
  for (ii=0; ii<NINPUTS; ii++) {
	  IPTRS[ii] = mxGetPr(prhs[ii]);
	  n[ii] = mxGetN(prhs[ii]);
	  m[ii] = mxGetM(prhs[ii]);
  }

  /* check dimensions of inputs */
  for (ii=0; ii<NINPUTS; ii++) {
	  if(n[ii] != ngood[ii]) {
		  sprintf(errstring,"Input number %d: column dimension is %d but should be %d", ii+1, n[ii], ngood[ii]);
		  mexErrMsgTxt(errstring);
	  }
	  if(m[ii] != mgood[ii]) {
		  sprintf(errstring,"Input number %d: row dimension is %d but should be %d", ii+1, m[ii], mgood[ii]);
		  mexErrMsgTxt(errstring);
	  }
  }

  /* check range of inputs */
  for (ii=0; ii<NINPUTS; ii++) {
	  if( outofrange(IPTRS[ii],n[ii]*m[ii],min[ii],max[ii]) ) {
		  sprintf(errstring,"Input number %d: value out of range.", ii+1);
		  mexErrMsgTxt(errstring);
	  }
	  if(integer[ii]==1){
		  if( noninteger(IPTRS[ii],n[ii]*m[ii]) ) {
			sprintf(errstring,"Input number %d: values must be integers.", ii+1);
			mexErrMsgTxt(errstring);
		  }
	  }
  }

  /* initialize output elements */
  for (ii=0; ii< ((nlhs>0)? nlhs:1) ; ii++) {
	  plhs[ii] = mxCreateDoubleMatrix(mout[ii],nout[ii],mxREAL);
	  OPTRS[ii] = mxGetPr(plhs[ii]);
  }
  
  
  /* call C function but with NULL in place of Y7 to force calculation with one-day data */
  deltamat(OPTRS[0], IPTRS[0],IPTRS[1],(int)*IPTRS[2],IPTRS[3],IPTRS[4],IPTRS[5],
			IPTRS[6],IPTRS[7],IPTRS[8],IPTRS[9],NULL);
}




/**********************************************************************
 *
 * OUTOFRANGE: Returns 1 if there is a value in vector greater
 *				than max or less than min; returns 0 otherwise.
 *				min/max values of +/-9999 are interpreted as +/- infinity.
 *				Returns 0 otherwise.
 *
 ***********************************************************************/

int outofrange(double *vector, int length, double min, double max) 
{

	int ii;

	for (ii=0; ii<length; ii++) {
		if ( (max!=9999 & vector[ii]>max) | (min!=-9999 & vector[ii]<min) ) {
			return(1);
		}
	}

	return(0);
}


/**********************************************************************
 *
 * NONINTEGER: Returns 1 if there is a value in vector whose
 *				decimal portion is not zero. Returns 0 otherwise
 *
 ***********************************************************************/

int noninteger(double *vector, int length) 
{

	int ii;

	for (ii=0; ii<length; ii++) {
		if ( fmod(vector[ii],1) ) {
			return(1);
		}
	}

	return(0);
}



