/***********************************************************************************************
 *
 * LIKEIHOOD7: Mex function to evaluate logit likelihood.
 *			Likelihood is weighted for importance sampling estimation.
 *
 * Matthew Gentzkow
 * July 15, 2003
 * Online Editions
 * 
 * MATLAB prototype:
 * [L,dLc] = likelihood7(bc,m,excl,Y1,Y7,X,RD,CM,Bcons,Mcons,Mincl,Bex,T,A,weights)
 *
 * compiler command: 
 *	mex likelihood7.c oedcalc.c oedcalcbig.c oedobjectives.c
 *
 * note: a*b*c in dimension means vectors stacked as a groups of b groups of c
 * note: to run with no unobservables, simply set d=0 and either m=0 or m=1.
 * note: # before range means only integer values are valid
 *
 * INPUTS:
 *	bc			ZZ			(-inf,inf)		(constrained) parameter vector
 *	m			scalar		#(0,inf)		number of simulation draws
 *	excl		PMAX		#(0,1)			vec of exclusion restrictions (=1 if excl)
 *	Y1			NN			#(0,CC-1)		one-day choices
 *	Y7			NN			#(0,CC-1)		seven-day choices
 *	X			KK x NN		(-inf,inf)		observable rhs vars
 *	RD			NN*DD x m	(-inf,inf)		random draws of unobservable rhs vars
 *	CM			GG x CC		#(0,1)			i x j=1 if good i included in choice j
 *	Bcons		CC x CC		#(0,1)			i x j=1 if choice j potentially consist with choice i
 *	Bex			CC x CC		#(0,CC-1)		i x j=k where 1-day ch i takes overall ch from j to k
 *	T			PP x ZZ		(-inf,inf)		constraint matrix
 *	A			PP			(-inf,inf)		constraint vector
 *	weights		NN x m		(-inf,inf)		weights of each obs (eg for imp sampling)
 *
 * OUTPUTS:
 *	L			scalar		(-inf,inf)		simulated log likelihood (w/ bias correction)
 *	dLc			ZZ			(-inf,inf)		derivative of log likelihood w.r.t. constrained parameters
 *
 *********************************************************************************************/

#include <math.h>
#include "mex.h"
#include "choose_max.h"
#include "oedcalc.h"
#include "oedobjectives.h"

#define NINPUTS 13
#define NOUTMAX 2

/* ****************************************************** */
/* mexFunction: MATLAB gateway routine                    */
/*														  */
/* remember: m is rows, n is columns					  */
/*													      */
/* ****************************************************** */


void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[] )
{
  int n[NINPUTS], m[NINPUTS], ngood[NINPUTS], mgood[NINPUTS], integer[NINPUTS],
	  nout[NOUTMAX], mout[NOUTMAX];
  double min[NINPUTS], max[NINPUTS], rdval;
  char errstring[200];
  int ii;
  int mm;
  double *IPTRS[NINPUTS], *OPTRS[NOUTMAX];
  
  /* get m here to assign dimensions */
  mm = *mxGetPr(prhs[1]);
  
  /* handle the case of DD==0 */
  rdval = choose_max(NN*DD,1);
  
  /* set ngood and mgood arrays with expected input dimensions */
  mgood[0] = ZZ;			ngood[0] = 1;
  mgood[1] = 1;				ngood[1] = 1;
  mgood[2] = PMAX;			ngood[2] = 1;
  mgood[3] = NN;			ngood[3] = 1;
  mgood[4] = NN;			ngood[4] = 1;
  mgood[5] = KK;			ngood[5] = NN;
  mgood[6] = rdval;         ngood[6] = mm;
  mgood[7] = GG;			ngood[7] = CC;
  mgood[8] = CC;			ngood[8] = CC;
  mgood[9] = CC;			ngood[9] = CC;
  mgood[10] = PP;			ngood[10] = ZZ;
  mgood[11] = PP;			ngood[11] = 1;
  mgood[12] = NN;			ngood[12] = mm;

  /* set min and max arrays with allowable range of inputs */
  /* (enter 9999 or -9999 to represent +/- infinity)  */
  /* integer[ii]=1 indicates input must be an integer */
  min[0] = -9999;		max[0] = 9999;		integer[0] = 0;
  min[1] = 0;			max[1] = 9999;		integer[1] = 1;
  min[2] = 0;			max[2] = 1;			integer[2] = 1;
  min[3] = 0;			max[3] = CC-1;		integer[3] = 1;
  min[4] = 0;			max[4] = CC-1;		integer[4] = 1;
  min[5] = -9999;		max[5] = 9999;		integer[5] = 0;
  min[6] = -9999;		max[6] = 9999;		integer[6] = 0;
  min[7] = 0;			max[7] = 1;			integer[7] = 1;
  min[8] = 0;			max[8] = 1;			integer[8] = 1;
  min[9] = 0;			max[9] = CC-1;		integer[9] = 1;
  min[10] = -9999;		max[10] = 9999;		integer[10] = 0;
  min[11] = -9999;		max[11] = 9999;		integer[11] = 0;
  min[12] = -9999;		max[12] = 9999;		integer[12] = 0;


  /*set nout and mout arrays with output dimensions */
  mout[0] = 1;				nout[0] = 1;
  mout[1] = ZZ;				nout[1] = 1;

  
  /* check number of arguments */
  if(nrhs != NINPUTS)
    mexErrMsgTxt("NINPUTS inputs required.");
  if(nlhs > NOUTMAX)
    mexErrMsgTxt("At most NOUTMAX outputs allowed.");


  /* get pointers and dimensions of each input element */
  for (ii=0; ii<NINPUTS; ii++) {
	  IPTRS[ii] = mxGetPr(prhs[ii]);
	  n[ii] = mxGetN(prhs[ii]);
	  m[ii] = mxGetM(prhs[ii]);
  }

  /* check dimensions of inputs */
  for (ii=0; ii<NINPUTS; ii++) {
	  if(n[ii] != ngood[ii]) {
		  sprintf(errstring,"Input number %d: column dimension is %d but should be %d", ii+1, n[ii], ngood[ii]);
		  mexErrMsgTxt(errstring);
	  }
	  if(m[ii] != mgood[ii]) {
		  sprintf(errstring,"Input number %d: row dimension is %d but should be %d", ii+1, m[ii], mgood[ii]);
		  mexErrMsgTxt(errstring);
	  }
  }

  /* check range of inputs */
  for (ii=0; ii<NINPUTS; ii++) {
	  if( outofrange(IPTRS[ii],n[ii]*m[ii],min[ii],max[ii]) ) {
		  sprintf(errstring,"Input number %d: value out of range.", ii+1);
		  mexErrMsgTxt(errstring);
	  }
	  if(integer[ii]==1){
		  if( noninteger(IPTRS[ii],n[ii]*m[ii]) ) {
			sprintf(errstring,"Input number %d: values must be integers.", ii+1);
			mexErrMsgTxt(errstring);
		  }
	  }
  }

  /* initialize output elements */
  for (ii=0; ii< ((nlhs>0)? nlhs:1) ; ii++) {
	  plhs[ii] = mxCreateDoubleMatrix(mout[ii],nout[ii],mxREAL);
	  OPTRS[ii] = mxGetPr(plhs[ii]);
  }

  /* call C function */
  if (nlhs<=1) {
    /* don't calculate the derivative */
	likelihood(OPTRS[0],NULL, IPTRS[0],(int)*IPTRS[1],IPTRS[2],IPTRS[3],IPTRS[4],IPTRS[5],
			IPTRS[6],IPTRS[7],IPTRS[8],IPTRS[9],IPTRS[10],IPTRS[11],IPTRS[12]);
  } else if (nlhs==2) {
    /* do calculate the derivative */
	likelihood(OPTRS[0],OPTRS[1], IPTRS[0],(int)*IPTRS[1],IPTRS[2],IPTRS[3],IPTRS[4],IPTRS[5],
			IPTRS[6],IPTRS[7],IPTRS[8],IPTRS[9],IPTRS[10],IPTRS[11],IPTRS[12]);
  }


}



/**********************************************************************
 *
 * OUTOFRANGE: Returns 1 if there is a value in vector greater
 *				than max or less than min; returns 0 otherwise.
 *				min/max values of +/-9999 are interpreted as +/- infinity.
 *				Returns 0 otherwise.
 *
 ***********************************************************************/

int outofrange(double *vector, int length, double min, double max) 
{

	int ii;

	for (ii=0; ii<length; ii++) {
		if ( (max!=9999 & vector[ii]>max) | (min!=-9999 & vector[ii]<min) ) {
			return(1);
		}
	}

	return(0);
}


/**********************************************************************
 *
 * NONINTEGER: Returns 1 if there is a value in vector whose
 *				decimal portion is not zero. Returns 0 otherwise
 *
 ***********************************************************************/

int noninteger(double *vector, int length) 
{

	int ii;

	for (ii=0; ii<length; ii++) {
		if ( fmod(vector[ii],1) ) {
			return(1);
		}
	}

	return(0);
}



