/******************************************************************************************************
 *
 * MSMOBJ: Objective function for MSM estimation of OED model. Returns both sum of squared
 *			moments and gradient vector.
 *
 * Matthew Gentzkow
 * July 15, 2003
 * Online Editions
 * 
 * MATLAB prototype:
 * [Gn,dGn] = msmobj(bc,excl,Z,Y1,Y7,Cdum,X,RD,weights,T,A,CM,Cincl,Bcons,Mcons,Mincl,Bex,m)
 *
 * compiler command: 
 *	mex msmobj.c oedcalc.c oedcalcbig.c oedobjectives.c
 *
 * note: a*b*c in dimension means vectors stacked as a groups of b groups of c
 * note: # before range means only integer values are valid
 *
 * INPUTS:
 *	bc			ZZ			(-inf,inf)		(constrained) parameter vector
 *	excl		PMAX		#(0,1)			vec of exclusion restrictions (=1 if excl)
 *	Z			NN*CCW x PP	(-inf,inf)		vec of instruments (grad at prelim param val)
 *	Y1			NN			#(0,CC-1)		one-day choices
 *	Y7			NN			#(0,CC-1)		seven-day choices
 *	Cdum		CCW x NN	#(0,1)			i x j=1 if cons j chose combo choice i; else =0
 *	X			KK x NN		(-inf,inf)		observable rhs vars
 *	RD			NN*DD x m	(-inf,inf)		random draws of unobservable rhs vars
 *	weights		NN x m		(-inf,inf)		weights of each obs (eg for imp sampling)
 *	T			PP x ZZ		(-inf,inf)		constraint matrix
 *	A			PP			(-inf,inf)		constraint vector
 *	CM			GG x CC		#(0,1)			i x j=1 if good i included in choice j
 *	Cincl		CCW x 2		#(0,CC-1)		row i is 1 and 7-day choices incl in combo choice i
 *	Bcons		CC x CC		#(0,1)			i x j=1 if choice j potentially consist with choice i
 *	Bex			CC x CC		#(0,CC-1)		i x j=k where 1-day ch i takes overall ch from j to k
 *	m			scalar		#(0,inf)		number of simulation draws
 *
 * OUTPUTS:
 *	Gn			scalar		(0,inf)			objective
 *	dGn			ZZ			(-inf,inf)		gradient vector (w.r.t. cons parameters)
 *
 ********************************************************************************************************/

#include <math.h>
#include "mex.h"
#include "oedcalc.h"
#include "oedobjectives.h"

#define NINPUTS 16
#define NOUTMAX 2


/* ****************************************************** */
/* mexFunction: MATLAB gateway routine                    */
/*														  */
/* remember: m is rows, n is columns					  */
/*													      */
/* ****************************************************** */


void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[] )
{
  int n[NINPUTS], m[NINPUTS], ngood[NINPUTS], mgood[NINPUTS], integer[NINPUTS],
	  nout[NOUTMAX], mout[NOUTMAX];
  double min[NINPUTS], max[NINPUTS];
  char errstring[200];
  int ii;
  int mm;
  double *IPTRS[NINPUTS], *OPTRS[NOUTMAX];
  
  /* get m here to assign dimensions */
  mm = *mxGetPr(prhs[17]);
  
  /* set ngood and mgood arrays with expected input dimensions */
  mgood[0] = ZZ;			ngood[0] = 1;
  mgood[1] = PMAX;			ngood[1] = 1;
  mgood[2] = NN*CCW;		ngood[2] = PP;
  mgood[3] = NN;			ngood[3] = 1;
  mgood[4] = NN;			ngood[4] = 1;
  mgood[5] = CCW;			ngood[5] = NN;
  mgood[6] = KK;			ngood[6] = NN;
  mgood[7] = NN*DD;			ngood[7] = mm;
  mgood[8] = NN;			ngood[8] = mm;
  mgood[9] = PP;			ngood[9] = ZZ;
  mgood[10] = PP;			ngood[10] = 1;
  mgood[11] = GG;			ngood[11] = CC;
  mgood[12] = CCW;			ngood[12] = 2;
  mgood[13] = CC;			ngood[13] = CC;
  mgood[14] = CC;			ngood[14] = CC;
  mgood[15] = 1;			ngood[15] = 1;

  /* set min and max arrays with allowable range of inputs */
  /* (enter 9999 or -9999 to represent +/- infinity)  */
  /* integer[ii]=1 indicates input must be an integer */
  min[0] = -9999;		max[0] = 9999;		integer[0] = 0;
  min[1] = 0;			max[1] = 1;			integer[1] = 1;
  min[2] = -9999;		max[2] = 9999;		integer[2] = 0;
  min[3] = 0;			max[3] = CC-1;		integer[3] = 1;
  min[4] = 0;			max[4] = CC-1;		integer[4] = 1;
  min[5] = 0;			max[5] = 1;			integer[5] = 1;
  min[6] = -9999;		max[6] = 9999;		integer[6] = 0;
  min[7] = -9999;		max[7] = 9999;		integer[7] = 0;
  min[8] = -9999;		max[8] = 9999;		integer[8] = 0;
  min[9] = -9999;		max[9] = 9999;		integer[9] = 0;
  min[10] = -9999;		max[10] = 9999;		integer[10] = 0;
  min[11] = 0;			max[11] = 1;		integer[11] = 1;
  min[12] = 0;			max[12] = CC-1;		integer[12] = 1;
  min[13] = 0;			max[13] = 1;		integer[13] = 1;
  min[14] = 0;			max[14] = CC-1;		integer[14] = 1;
  min[15] = 0;			max[15] = 9999;		integer[15] = 1;


  /*set nout and mout arrays with output dimensions */
  mout[0] = 1;				nout[0] = 1;
  mout[1] = ZZ;				nout[1] = 1;
  
  /* check number of arguments */
  if(nrhs != NINPUTS)
    mexErrMsgTxt("NINPUTS inputs required.");
  if(nlhs > NOUTMAX)
    mexErrMsgTxt("At most NOUTMAX outputs allowed.");

  /* get pointers and dimensions of each input element */
  for (ii=0; ii<NINPUTS; ii++) {
	  IPTRS[ii] = mxGetPr(prhs[ii]);
	  n[ii] = mxGetN(prhs[ii]);
	  m[ii] = mxGetM(prhs[ii]);
  }

  /* check dimensions of inputs */
  for (ii=0; ii<NINPUTS; ii++) {
	  if(n[ii] != ngood[ii]) {
		  sprintf(errstring,"Input number %d: column dimension is %d but should be %d", ii+1, n[ii], ngood[ii]);
		  mexErrMsgTxt(errstring);
	  }
	  if(m[ii] != mgood[ii]) {
		  sprintf(errstring,"Input number %d: row dimension is %d but should be %d", ii+1, m[ii], mgood[ii]);
		  mexErrMsgTxt(errstring);
	  }
  }

  /* check range of inputs */
  for (ii=0; ii<NINPUTS; ii++) {
	  if( outofrange(IPTRS[ii],n[ii]*m[ii],min[ii],max[ii]) ) {
		  sprintf(errstring,"Input number %d: value out of range.", ii+1);
		  mexErrMsgTxt(errstring);
	  }
	  if(integer[ii]==1){
		  if( noninteger(IPTRS[ii],n[ii]*m[ii]) ) {
			sprintf(errstring,"Input number %d: values must be integers.", ii+1);
			mexErrMsgTxt(errstring);
		  }
	  }
  }

  /* initialize output elements */
  for (ii=0; ii< ((nlhs>0)? nlhs:1) ; ii++) {
	  plhs[ii] = mxCreateDoubleMatrix(mout[ii],nout[ii],mxREAL);
	  OPTRS[ii] = mxGetPr(plhs[ii]);
  }

  /* call C function */
  if (nlhs<=1) {
	msmobjectivend(OPTRS[0], IPTRS[0],IPTRS[1],IPTRS[2],IPTRS[3],IPTRS[4],IPTRS[5],
			IPTRS[6],IPTRS[7],IPTRS[8],IPTRS[9],IPTRS[10],IPTRS[11],IPTRS[12],IPTRS[13],IPTRS[14],(int)*IPTRS[15]);
  } else if (nlhs==2) {
	msmobjective(OPTRS[0],OPTRS[1], IPTRS[0],IPTRS[1],IPTRS[2],IPTRS[3],IPTRS[4],IPTRS[5],
			IPTRS[6],IPTRS[7],IPTRS[8],IPTRS[9],IPTRS[10],IPTRS[11],IPTRS[12],IPTRS[13],IPTRS[14],(int)*IPTRS[15]);
  }


}

/**********************************************************************
 *
 * OUTOFRANGE: Returns 1 if there is a value in vector greater
 *				than max or less than min; returns 0 otherwise.
 *				min/max values of +/-9999 are interpreted as +/- infinity.
 *				Returns 0 otherwise.
 *
 ***********************************************************************/

int outofrange(double *vector, int length, double min, double max) 
{

	int ii;

	for (ii=0; ii<length; ii++) {
		if ( (max!=9999 & vector[ii]>max) | (min!=-9999 & vector[ii]<min) ) {
			return(1);
		}
	}

	return(0);
}


/**********************************************************************
 *
 * NONINTEGER: Returns 1 if there is a value in vector whose
 *				decimal portion is not zero. Returns 0 otherwise
 *
 ***********************************************************************/

int noninteger(double *vector, int length) 
{

	int ii;

	for (ii=0; ii<length; ii++) {
		if ( fmod(vector[ii],1) ) {
			return(1);
		}
	}

	return(0);
}



