%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% ESTsml.M: Run SML estimation routine.
%
% Matthew Gentzkow
% September 16, 2002
% Online Editions
%
% note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
%
% INPUTS:
%	b0			PP					starting value of parameter vector
%	excl		PMAX				vector of exclusion restrictions (=1 if excluded)
%	Y1			NN					one-day choices
%	Y7			NN					seven-day choices
%	X		    NN x KK    			observable rhs vars
%   RD          NN*DD x mf          random draws of unobservables
%	weights		NN x mf				weights of each obs (not = 1 in imp sampling, e.GG.)
%	T			PP x ZZ				constraint matrix
%	A			PP					constraint vector
%	CM			CC x GG				i*j=1 if good j included in choice i
%	Bcons		CC x CC				i*j=1 if choice i potentially consistent with choice j
%	Mcons		CC					first vector for missing 1-day data
%	Mincl		CC					second vector for missing 1-day data
%	Bex			CC x CC				i*j=KK where 1-day choice j takes overall choice from i to KK
%
%	
% OUTPUTS:
%	bsml		PP					parameter vector CC groups of (KK+DD) followed by f(pi)
%	SE1			PP					vector of standard errors (inv Hessian)
%   S1          PP x PP             parameter cov matrix (inv Hessian)
%	SE2			PP					vector of standard errors (outer product of gradients)
%   S2          PP x PP             parameter cov matrix (outer product of gradients)
%	SE3			PP					vector of standard errors (robust)
%   S3          PP x PP             parameter cov matrix (robust)
%	L			scalar				(maximized) log likelihood
%	dL			PP					gradient vector (derivatives of log likelihood)
%
% FILES SAVED:
%	tempout (preliminary parameter estimate)
%	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [bsml,SE1,S1,SE2,S2,SE3,S3,L,dL,dLc,Hc] = ESTsml(b0,excl,Y1,Y7,X,RD,weights,T,A,CM,Bcons,Bex)

global UseDyn ImpSamp NN GG CC KK DD PP ZZ tolf0 tolff tolx0 tolxf mfe0 mfef...
       miter0 miterf msteps mf

b0c = (b0'*T)';

% optimization steps
if size(msteps,1)~=0 & DD>0 & UseDyn==1
	OPTIONS = optimset('Display','iter','MaxFunEvals',mfe0,'MaxIter',miter0,'TolFun',tolf0,'TolX',tolx0,...
                        'GradObj','on','LargeScale','off');
	for i = 1:length(msteps)
		sprintf('Estimation step %1d',i)		
		tic
        RDcur = RD(:,1:msteps(i));
        weightscur = weights(:,1:msteps(i));
   		b0c = fminunc('likelihood7',b0c,OPTIONS,msteps(i),excl,Y1,Y7,X',RDcur,CM',Bcons,Bex,T,A,weightscur);
  		toc  	
		b0 = (b0c'*T'+A')'
        lastdone = 'Prelim SML';
        save tempout b0 lastdone
	end
end

% final optimization
OPTIONS = optimset('Display','iter','MaxFunEvals',mfef,'MaxIter',miterf,'TolFun',tolff,'TolX',tolxf,...
                    'GradObj','on','LargeScale','off');
sprintf('Final estimation step')
if DD==0, mf=1; end
tic
if UseDyn==1
	[b0c,L,exitflag,output,dLc,Hc] = fminunc('likelihood7',b0c,OPTIONS,mf,excl,Y1,Y7,X',RD,...
                                                CM',Bcons,Bex,T,A,weights);
elseif UseDyn==0
     [b0c,L,exitflag,output,dLc,Hc] = fminunc('likelihood1',b0c,OPTIONS,mf,excl,Y1,Y7,X',RD,...
                                                 CM',Bcons,Bex,T,A,weights);
     lastdone = 'Final Step';
     save tempout b0c L dLc Hc exitflag lastdone
end
 toc
 bsml = (b0c'*T'+A')'
 
% transform
L = -L; dLc = -dLc; Hc = -Hc;
dL = (dLc'*T'+A')';

% save
lastdone = 'Final SML';
save tempout bsml L exitflag output dLc Hc lastdone


if UseDyn==1
	DM = deltamat(bsml,excl,mf,X',RD,CM',Bcons,Bex,weights,Y1,Y7);
else
    DM = deltamat1(bsml,excl,mf,X',RD,CM',Bcons,Bex,weights,Y1);
end
DMc = T'*DM*T;

% std errors as inverse of hessian
S1c = -inv(Hc);
S1 = T*S1c*T';
SE1 = sqrt(diag(S1));

% std errors as inverse of outer product of gradients
S2c = inv(DMc);
S2 = T*S2c*T';
SE2 = sqrt(diag(S2));

% robust standard errors (see McFadden&Train 2000)
S3c = S1c*inv(S2c)*S1c;
S3 = T*S3c*T';
SE3 = sqrt(diag(S3));


