%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% BUILDR.M: Function used to restrict interaction terms for combinations
% 				of more than 2 goods. (Interaction for big combination is
% 				sum of interactions of all 2-good subsets).
%
% Matthew Gentzkow
% September 16, 2002
% Online Editions
%
% note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
%
% INPUTS:
%	CM			CC x GG				i*j=1 if good j included in choice i
%	GG			scalar				number of goods
%	CC			scalar				number of choices
%	PMAX  		scalar				total number of possible parameters
%	KK			scalar				number of observables
%	DD			scalar				number of unobservables
%   XX          scalar              number of ch set restrictions
%   excl        PMAX x 1            exclusion restrictions
%   chsetrhs    1 x XX              index of variables that govern ch set
%   chsetlhs    GG x XX             i*j=1 if good j is excluded when i==1
%   UseDyn      scalar              indicator =1 if run w/ dynamics =0
%                                   otherwise
%   const       PMAX x 1            vector of constraints. each
%                                   element=-999 if unconstrained or else 
%                                   the constrained value of the parameter 
%
%	
% OUTPUTS:
%	R			(PP-ZZ) x PP		matrix of linear restrictions
%	r			(PP-ZZ)				vector of linear restrictions (Rb=r)
%	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [R,r] = buildR(CM,GG,CC,PMAX,KK,DD,XX,excl,chsetrhs,chsetlhs,UseDyn,const,highorder)

% check consistency of parameters and data
test = 0;
if  CC ~= size(CM,1) | GG ~= size(CM,2) | PMAX ~= size(excl,1) | ...
      XX ~= size(chsetrhs,2) | XX ~= size(chsetlhs,2) | GG ~= size(chsetlhs,1),
    
    error('parameter/data inconsistency');       
end

% large negative coefficient to put on noweb variable to effectively take
% web choices out of the choice set
NEGVAL = -15;

R = [];
r = [];

% create R for restricting interactions in bundles with >2 goods
%
% note: first create matrix as if there are NO exclusion restrictions
%       then impose exclusion restrictions at the end.

if highorder==1
    for ii =(GG+2):CC
        n=sum(CM(ii,:));

        if n>2
            base1 = GG*(KK+DD)+(ii-GG-2)*(KK+DD);

            for kk = 1:KK+DD
                if excl(base1+kk)==0     
                    indices=[];

                    for jj=1:GG 
                        if CM(ii,jj)==1, indices=[indices;jj]; end
                    end

                    pairs = nchoosek(indices,2);
                    nprs = size(pairs,1);
                    row = zeros(1,PMAX);
                    row(base1+kk)=-1;

                    for jj=1:nprs
                        v = zeros(1,GG);
                        v(pairs(jj,1))=1;
                        v(pairs(jj,2))=1;
                        choice = findinmat(v,CM);
                        base = GG*(KK+DD)+(choice-GG-2)*(KK+DD);
                        row(base+kk)=1;
                    end

                    R = [R;row];
                end
            end
        end
    end
    r = zeros(size(R,1),1);
end

    
% now add a row to exclude web from choice set
for ii = 1:XX
    if chsetrhs(ii)>0
        newrow = zeros(1,PMAX);
        for jj = 1:GG
            if chsetlhs(jj,ii)==1
                newrow = zeros(1,PMAX);
                base = (jj-1)*(KK+DD);
                newrow(base+chsetrhs(ii))=1;
                R = [R;newrow];
            end
        end
    end

    numchoices = sum(chsetlhs(:,ii));
    r = [r; NEGVAL*ones(numchoices,1)];
end

% impose constraints
for ii = 1:PMAX
    if const(ii)~=-9999
            R = [R;[zeros(1,ii-1) 1 zeros(1,PMAX-ii)]];
            r = [r;const(ii)];
    end
end

% if UseDyn==0, add a row restricting pi to be zero
if UseDyn==0
    R = [R;[zeros(1,PMAX-1) 1]];
    r = [r;0];  
end
    
% now cut down R according to exclusion restrictions
select = (1-excl);
R = selectinmat(R',select)';