%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CROSSDERIVATIVES.M: Calculate cross-derivatives of demand
%
% Matthew Gentzkow
% September 25, 2003
% Online Editions
%
% note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
%
% INPUTS:
%   mcd         scalar              number of draws for calculating cross-derivatives
%   mstderr     scalar              number of monte carlo draws
%   bstarmat    PP x mstderr        each column is one random draw on the parameter vector
%	excl		PP					vector of exclusion restrictions
%	Xall		NNtot x KK			observable rhs vars
%	CM			CC x GG				i*j=1 if good j included in choice i
% 	surveyweights	NNtot			sampling weights of each observation
%   mweights    NNtot*mcd           importance sampling weights
%	
% OUTPUTS:
%   CDuw        NNtot x GG*GG       unweighted cross-derivatives of choice probabilities for each consumer
%   CDw         NNtot x GG*GG       weighted cross-derivatives of choice probabilities for each consumer
%   CDuwAVG     GG x GG             unweighted average cross-derivatives of demand in sample
%   CDwAVG      GG x GG             weighted average cross-derivatives of demand in sample
%   CDwTOT      GG x GG             weighted cross-derivatives of total demand
%   CDwTOTse    GG x GG             standard errors on cross-derivatives of total demand
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [CDwTOT,CDwTOTcov,CDwTOTse,CDwTRUE] = crossderivatives(mcd,mstderr,bstar,bstarmat,excl,Xall,CM,surveyweights,mweights);

global GG CC KK DD PP ZZ NNtot msee

CDwTOTmat = zeros(GG*GG,mstderr);

if DD>0
  	RDcd = randn(NNtot*DD,mcd);
    muse = mcd;
else
   	RDcd = 0;
    muse = 1;
end

bstarmat = [bstar bstarmat];
for mm = 1:mstderr+1
    b = bstarmat(:,mm);
	CD = crossderivw(b,excl,muse,NNtot,Xall',RDcd,CM',mweights)';
    CDw = CD.*repmat(surveyweights,1,GG*GG);
    CDwTOTmat(:,mm) = mean(CDw)'*NNtot;

    % use only the true parameter vector for CDwTOT
    if mm==1
        CDwTOT = reshape(CDwTOTmat(:,mm),GG,GG);
        CDwTRUE = CDw;
    end
end

if mstderr>1
    CDwTOTcov=cov(CDwTOTmat');
    CDwTOTse=reshape(sqrt(var(CDwTOTmat')),GG,GG);
else
    CDwTOTcov=zeros(GG*GG,GG*GG);
    CDwTOTse =zeros(GG,GG);
end

