function analysis
% Calculate scores and Jacobians
% Output score_and_jacobian.mat

    % Add path
    addpath('source/lib/YAMLMatlab_0.4.3');
    PATHS = ReadYaml('config_global.yaml');
    addpath(sprintf('%s/hendren_2013_orig', PATHS.source.analysis.hendren_2013))
    load(sprintf('%s/data/hendren_2013_rawdata.mat', PATHS.build.external.hendren_2013))

    L      = data(:, 3);
    z      = data(:, 4);
    x_d    = data(:, 6:end);
    other  = ones(size(x_d, 1), 1); % changing this allows for robustness to heterogeneity in elicitation error bias by covariates
    center = mean(L);
    
    % Make q = Pr[L|X] to collapse dimensionality of X in pre-step before model MLE
    b = glmfit(x_d, L, 'binomial', 'link', 'probit');
    q = glmval(b, x_d, 'probit');
    
    [scoredm, mppr_jacobian] = compute_score_and_jacobian(L, z, other, center, q, PATHS.build.external.hendren_2013);
        
    % Save score and jacobian
    save(sprintf('%s/score_and_jacobian.mat', PATHS.build.analysis.hendren_2013), 'scoredm', 'mppr_jacobian')
    exit
end 

function [scoredm, mppr_jacobian] = compute_score_and_jacobian(L, z, other, center, q, data_path)
    load(sprintf('%s/data/hendren_2013_param_estimates.mat', data_path)) 

    % Compute slope at +/- perc_a around the estimated values
    % Note as perc_a -> 0, the matlab quadv precision of the integral in ln_like_X_vec 
    % starts to get poor and % lead to some crazy estimates. But, things appear to be 
    % quite stable in the % 0.01-0.2 range, which is loosely in the standard error 
    % region of the estimates...I should probably be more precise about picking
    % this fraction, but things seemed generally robust so I didn't invest much
    % thought into the optimal perc_a to pick. 

    perc_a  = .1;
    t_cut   = 0.2; % alpha = 0.8 = range over which infimum is taken
    t_cut_p = .9;  % Restriction for calculating T(p) [greater than 1-t_cut]

    score = [];
    mppr_jacobian = [];

    pp = [.01:.003:.99]';
    pp = sort([pp; center]);
    zz = ones(size(pp)) .* center;

    for j = 1:10
        % Calculate score
        est_low     = est_final;
        est_high    = est_final;
        est_low(j)  = est_final(j) * (1 - perc_a); % Bottom point for slope calculation
        est_high(j) = est_final(j) * (1 + perc_a); % Top point for slope calculation
        ln_vec_low  = ln_like_X_vec(L, z, q, est_low,  other, 1); % Log likelihood at bottom point
        ln_vec_high = ln_like_X_vec(L, z, q, est_high, other, 1); % Log likelihood at top point
        score = [score, (ln_vec_high - ln_vec_low) ./ (est_high(j) - est_low(j))];
        
        % Calculate Slope of Minimum Pooled Price Ratio
        inf_T_low     = compute_inf_T(est_low,  center, pp, t_cut, t_cut_p);
        inf_T_high    = compute_inf_T(est_high, center, pp, t_cut, t_cut_p);
        mppr_jacobian = [mppr_jacobian, (inf_T_high - inf_T_low) / (est_high(j) - est_low(j))];
    end

    % Drop the 7th column of score since it is zero
    score = [score(:, 1:6), score(:, 8:10)];
    mppr_jacobian = [mppr_jacobian(:, 1:6), mppr_jacobian(8:end)]; % we should drop the 7th element of Jacobian instead of the 3rd since the 7th column of the score is zero

    % De-mean the score
    scoredm = score - ones(size(score(:, 1))) * mean(score);
end

function inf_T = compute_inf_T(est, center, pp, t_cut, t_cut_p)
    para_est_p = est(5:end);
    F_p_est    = zeros(size(pp));
    f_p        = zeros(size(pp));
    p_p        = zeros(size(pp));

    for jj = 1:size(pp, 1)
        F_p_est(jj) = f_cdf_p_X_use(pp(jj), center, para_est_p, 1, center);
        temp        = quadv(@(p) e_p_X_use(p, center, para_est_p, 1), pp(jj), 1);
        p_p(jj)     = temp / (1 - F_p_est(jj));
        f_p(jj)     = f_p_X_use(pp(jj), center, para_est_p, 1);
    end        
    
    T_est   = (p_p ./ (1 - p_p)) .* (1 - pp) ./ pp;    
    inf_T_t = T_est .* (F_p_est < 1 - t_cut) .* (T_est >= 1) .* (pp < t_cut_p) + 100 .* ((F_p_est >= 1 - t_cut) + (T_est < 1) + (pp >= t_cut_p));
    inf_T   = max(1, min(inf_T_t));
end