function compute
% Compute sensitivities and informativeness using score_and_jacobian.mat
% Output informativeness_results.mat
    % Add path
    addpath('source/lib/YAMLMatlab_0.4.3')
    addpath('source/analysis/hendren_2013')
    PATHS = ReadYaml('config_global.yaml');
    score_and_jacobian = load(sprintf('%s/score_and_jacobian.mat', ... 
                                     PATHS.build.analysis.hendren_2013));
    raw                = load(sprintf('%s/data/hendren_2013_rawdata.mat', ... 
                                     PATHS.build.external.hendren_2013));
   
    % Load prepared objects                       
    L      = raw.data(:, 3);
    Z      = raw.data(:, 4);
    score  = score_and_jacobian.scoredm;
    mppr_jacobian = score_and_jacobian.mppr_jacobian;    
    
    % Compute G, W, phi_g, phi_theta 
    nobs = size(score, 1);
    G = -score' * score / nobs;
    W = eye(size(score,2));
    phi_g = score';
    phi_theta = inv(G' * W * G) * (G' * W * phi_g);
    
    % Compute phi_c for the Minimum pooled price ratio
    cf_jacobian = mppr_jacobian;
    phi_c = -cf_jacobian * phi_theta;
        
    % Iterating over different possible choices of gamma-hat (see function
    % description for the possible choices)
    for i = 1:5 
        % Compute influence function for given gamma-hat
        phi_gam = get_phi_gamma(Z, L, i);
        
        % compute informativeness
        cf{i} = compute_informativeness(phi_c, phi_gam); 
    end 
    
    % Compute phi_c implied asymptotic standard error
    se = (phi_c * phi_c') ^ 0.5 / nobs;
    
    save(sprintf('%s/informativeness_results.mat', PATHS.build.analysis.hendren_2013), 'cf', 'se')
    exit
end

function phi_gam = get_phi_gamma(Z, L, gammahat_choice)
    % gammahat_choice: 1 --- focal point group; 2 ---- nonfocal point group; 
    % 3 ---- LTC; 4 ---- focal + nonfocal; 5 ---- focal + nonfocal + LTC
    
    % calculate all possible individual descriptive stats
    ind.zero = (Z == 0);
    ind.one  = (Z == 1);
    ind.half = (Z == .5);
    ind.intvl2  = (Z > .1) .* (Z <= .2);
    ind.intvl3  = (Z > .2) .* (Z <= .3);
    ind.intvl4  = (Z > .3) .* (Z <= .4);
    ind.intvl5  = (Z > .4) .* (Z <  .5);
    ind.intvl6  = (Z > .5) .* (Z <= .6);
    ind.intvl7  = (Z > .6) .* (Z <= .7);
    ind.intvl8  = (Z > .7) .* (Z <= .8);
    ind.intvl9  = (Z > .8) .* (Z <= .9);
    ind.intvl10 = (Z > .9) .* (Z <  1.);
    
    % different possible combinations
    focal               = {'zero', 'one', 'half'};
    nonfocal            = {'intvl2', 'intvl3', 'intvl4', ...
                           'intvl5', 'intvl6', 'intvl7', ...
                           'intvl8', 'intvl9', 'intvl10'};
    ltc                 = {'zero_loss', 'one_loss', 'half_loss', ...
                          'intvl2_loss', 'intvl3_loss', 'intvl4_loss', ...
                          'intvl5_loss', 'intvl6_loss', 'intvl7_loss', ...
                          'intvl8_loss', 'intvl9_loss', 'intvl10_loss'};
    focal_nonfocal      = [focal, nonfocal];
    focal_nonfocal_ltc  = [focal_nonfocal, ltc];  

    % select appropriate gamma-hat
    gammahats = {focal, nonfocal, ltc, focal_nonfocal, focal_nonfocal_ltc}; 
    gammahat  = gammahats{gammahat_choice};
    
    % calculate phi_gam
    phi_gam = cellfun(@(var) compute_inf(ind, var, ltc, L), gammahat, 'UniformOutput', false);
    phi_gam = [phi_gam{:}]';
end

function inf = compute_inf(ind, var, ltc, y)
    if ismember(var, ltc) % Calculate influential function for conditional expectation
        x    = ind.(var(1:end-5)); % select the group indicator variable by removing "_loss" suffix
        beta = sum(y .* x) / sum(x);
        inf  = x .* (y - x .* beta);
        inf  = inf ./ mean(x .* x);
    else % Calculate influential function for expectation                 
        inf = ind.(var) - mean(ind.(var));        
    end
end

function Delta = compute_informativeness(phi_c, phi_gam)
    % compute informativeness based on recipe 
    nobs = size(phi_c, 2); 
    Sigma_cc = phi_c * phi_c' / nobs;
    Sigma_gamgam = phi_gam * phi_gam' / nobs;
    Sigma_cgam = phi_c * phi_gam' / nobs;
    Lambda = Sigma_cgam / Sigma_gamgam;
    Delta = (Sigma_cgam * inv(Sigma_gamgam) * Sigma_cgam') ./ (Sigma_cc);
end