%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Implementation of Gentzkow and         %
% Shapiro (2013) measure of sensitivity  %
% of model estimates to data             %
%                                        %
% Also adds inf_T calculation            %
%                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all


%% Load data
orig_path = 'source/analysis/hendren_2013/hendren_2013_orig';
analysis_path = 'build/analysis/hendren_2013';
addpath(orig_path);
load(sprintf('%s/using_matlab_ltc_r.mat', orig_path))
indid = data(:,1);
hhid = data(:,2);
L = data(:,3);
z = data(:,4);
r = data(:,5);
x_d = data(:,6:end);
other = ones(size(x_d,1),1); % changing this allows for robustness to heterogeneity in elicitation error bias by covariates

center = mean(L);


%% Make q = Pr[L|X] to collapse dimensionality of X in pre-step before model MLE

b = glmfit(x_d,L,'binomial','link','probit');
q = glmval(b,x_d,'probit');

%% Calculate Score
load(sprintf('%s/est_ltc_r_uf.mat', orig_path)) % Load final estimates

% Compute slope at +/- perc_a around the estimated values
% Note as perc_a -> 0, the matlab quadv precision of the integral in ln_like_X_vec 
% starts to get poor and % lead to some crazy estimates. But, things appear to be 
% quite stable in the % 0.01-0.2 range, which is loosely in the standard error 
% region of the estimates...I should probably be more precise about picking
% this fraction, but things seemed generally robust so I didn't invest much
% thought into the optimal perc_a to pick. 

perc_a = .1;

t_cut = 0.2 % alpha = 0.8 = range over which infimum is taken
t_cut_p = .9; % Restriction for calculating T(p) [greater than 1-t_cut]

score = []
mppr = []

pp = [.01:.003:.99]';
pp = sort([pp;center]);
zz = ones(size(pp)).*center;

for j = 1:10
    est_low = est_final;
    est_high = est_final;
    est_low(j) = est_final(j)*(1 - perc_a); % Bottom point for slope calculation
    est_high(j) = est_final(j)*(1 + perc_a); % Top point for slope calculation
    ln_vec_low = ln_like_X_vec(L,z,q,est_low,other,1); % Log likelihood at bottom point
    ln_vec_high = ln_like_X_vec(L,z,q,est_high,other,1); % Log likelihood at top point
    score = [score , (ln_vec_high - ln_vec_low)./(est_high(j) - est_low(j))];
    
    % Calculate Slope of Minimum Pooled Price Ratio
    
    para_est_p = est_low(5:end)
    F_p_est = zeros(size(pp));
    f_p = zeros(size(pp));
    p_p = zeros(size(pp));
        for jj = 1:size(pp,1);
            F_p_est(jj) = f_cdf_p_X_use(pp(jj),center,para_est_p,1,center);
            temp = quadv(@(p) e_p_X_use(p,center,para_est_p,1) , pp(jj),1);
            p_p(jj) = temp / (1-F_p_est(jj));
            f_p(jj) = f_p_X_use(pp(jj),center,para_est_p,1);
        end        
    
    T_est = (p_p./(1-p_p)).*(1-pp)./pp;    
    inf_T_t = T_est.*(F_p_est < 1-t_cut).*(T_est >= 1).*(pp<t_cut_p) + 100.*((F_p_est >= 1-t_cut) + (T_est < 1) + (pp>=t_cut_p));
    inf_T_low = max(1,min(inf_T_t));
    
    
    para_est_p = est_high(5:end)
    F_p_est = zeros(size(pp));
    f_p = zeros(size(pp));
    p_p = zeros(size(pp));
        for jj = 1:size(pp,1);
            F_p_est(jj) = f_cdf_p_X_use(pp(jj),center,para_est_p,1,center);
            temp = quadv(@(p) e_p_X_use(p,center,para_est_p,1) , pp(jj),1);
            p_p(jj) = temp / (1-F_p_est(jj));
            f_p(jj) = f_p_X_use(pp(jj),center,para_est_p,1);
        end        
    
    T_est = (p_p./(1-p_p)).*(1-pp)./pp;    
    inf_T_t = T_est.*(F_p_est < 1-t_cut).*(T_est >= 1).*(pp<t_cut_p) + 100.*((F_p_est >= 1-t_cut) + (T_est < 1) + (pp>=t_cut_p));
    inf_T_high = max(1,min(inf_T_t));
    
    mppr = [mppr , (inf_T_high - inf_T_low)/(est_high(j) - est_low(j))];
    
end

% An issue to flag:
% I drop Score(7) from the exercise because it has a score of 0. 
% Score(7) is flat for all observations for the LTC Reject sample. In principle, the 7th parameter governs
% how much higher the top beta is relative to the mean E[L|X]. However, the likelihood maximization 
% puts this beta in a region where it's location remains close to the top of the distribution 
% regardless of q -- the center is instead pinned down by the mass on the upper beta and the 
% restriction that the mean is E[L|X]. As a result, the likelihood is
% locally flat. Intuitively, those who know they're going to a nursing home have 
% probabilities close to 1 that doesn't vary much with observables. 
% So, I omit the 7th parameter from this exercise...the
% impact of observations on the location of this beta will be governed by
% its impact on the mass on the beta (6th parameter).

score = [score(:,1:6),score(:,8:10)];
mppr = [mppr(:,1:2),mppr(4:end)];

%% Compute normalized score for each observation

% Because perc_a > 0, the score estimate doesn't average out to zero...so I
% de-mean it here.

scoredm = score - ones(size(score(:,1)))*mean(score);

% This normalizes the score by its covariance matrix
H = cov(scoredm);
score_norm = (inv(H)*scoredm')';

% This multiplies by mppr

mppr_score = score * mppr';


%% Compute gammas

% To start, I focus on 3 moments of the data: 
% 1) the covariance between Z and L (m_1)
% 2) the mean bias (m_2)
% 3) the fraction of focal point respondents (m_3)

% m_1 = Covariance Between Elicitation and Loss
temp = cov(z,L);
m_1 = (z-mean(z)*ones(size(z))).*(L-mean(L)*ones(size(L))) - temp(1,2)*ones(size(L));

% m_2 = Mean Bias of Elicitations
m_2 = L - mean(L)*ones(size(L)) - (z - mean(z)*ones(size(z)));

% m_3 = Fraction of Focal Point Respondents
focal = (z==0) + (z==.5) + (z==1);
m_3 = focal - mean(focal)*ones(size(z));

m = [ m_1 , m_2 , m_3 ];

% m_4 = Fraction z >= .5 and L = 1 

bigz = (z >= .5) .* (L == 1);
m_4 = bigz - mean(bigz)*ones(size(z));

% m_5 = Fraction z >= .5 and L = 0
bigzz = (z >= .5) .* (L == 0);
m_5 = bigzz - mean(bigz)*ones(size(z));

% m_6 = Fraction z <= .5 and L = 1 
lowz = (z<=.5).*(L==1);
m_6 = lowz - mean(lowz)*ones(size(z));

m_all = [m_4, m_5, m_6];

% Normalize by inverse variance-covariance matrix
% Question to Matt & Jesse: My sense was that I should be normalizing thes
% here by the covariance matrix; but perhaps I'm wrong. 
V_m = cov(m);
m_norm = (inv(V_m)*(m'))';

V_m_all = cov(m_all);
m_all_norm = (inv(V_m_all)*(m_all'))';


% %% We commented Hendren's following script for simplicity:
% %% Sensitivity Calculations
% 
% barlabel = ['Cov','Bias','Focal'];
% % Variance of Elicitation Error
% sensitivity_1 = inv((m_norm'*m_norm))*m_norm'*score_norm(:,1)
% 
% figure(1)
% bar(sensitivity_1)
% set(gca,'XTickLabel',{'Cov', 'Bias', 'Focal'},'fontsize',16,'Color','White')
% set(gcf, 'Color', 'None')
% xlabel('Gammas','FontSize',20)
% title('Sensitivity of Variance of Elicitation Error','FontSize',24)
% saveas(1, sprintf('%s/varelic', analysis_path),'eps')
% 
% % Fraction of Focal Point Respondents
% sensitivity_2 = inv((m_norm'*m_norm))*m_norm'*score_norm(:,2)
% 
% 
% figure(2)
% bar(sensitivity_2)
% set(gca,'XTickLabel',{'Cov', 'Bias', 'Focal'},'fontsize',16,'Color','White')
% set(gcf, 'Color', 'None')
% xlabel('Gammas','FontSize',20)
% title('Sensitivity of Fraction of Focal Point Respondents','FontSize',24)
% saveas(2, sprintf('%s/focal', analysis_path),'eps')
% 
% 
% % Ordered Probit Window
% sensitivity_3 = inv((m_norm'*m_norm))*m_norm'*score_norm(:,3)
% 
% figure(3)
% bar(sensitivity_3)
% set(gca,'XTickLabel',{'Cov', 'Bias', 'Focal'},'fontsize',16,'Color','White')
% set(gcf, 'Color', 'None')
% xlabel('Gammas','FontSize',20)
% title('Sensitivity of Ordered Probit Window','FontSize',24)
% saveas(3, sprintf('%s/window', analysis_path), 'eps')
% 
% 
% % Mean Bias
% sensitivity_4 = inv((m_norm'*m_norm))*m_norm'*score_norm(:,4)
% 
% figure(4)
% bar(sensitivity_4)
% set(gca,'XTickLabel',{'Cov', 'Bias', 'Focal'},'fontsize',16,'Color','White')
% set(gcf, 'Color', 'None')
% xlabel('Gammas','FontSize',20)
% title('Sensitivity of Mean Elicitation (Negative) Bias','FontSize',24)
% saveas(4, sprintf('%s/bias', analysis_path), 'eps')
% 
% % Minimum Pooled Price Ratio
% 
% 
% sensitivity_5 = inv((m_all_norm'*m_all_norm))*m_all_norm'*mppr_score;
% 
% figure(5)
% bar(sensitivity_5)
% set(gca,'XTickLabel',{'Z>=.5;L=1','Z>=.5;L=0','Z<.5,L=1'},'fontsize',16,'Color','White')
% set(gcf, 'Color', 'None')
% xlabel('Gammas','FontSize',20)
% title('Sensitivity of Minimum Pooled Price Ratio','FontSize',24)
% saveas(5, sprintf('%s/mppr', analysis_path),'eps')


%% We modify Hendren's original script slightly to save his sensitivities, score and Jacobian of MPPR 
model.score = score;
model.scoredm = scoredm;
model.elic_param_indices = 1:4;
model.sensitivity = (inv((m_norm'*m_norm))*m_norm'*score_norm)';
    
MPPR.jacobian = mppr;
MPPR.sensitivity = (inv((m_all_norm'*m_all_norm))*m_all_norm'*mppr_score)';

save(sprintf('%s/sensitivity_orig.mat', analysis_path), 'model', 'MPPR')
exit



